package fbg.gui;

// Title:       Image Constructor
// Version:     1.0
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.MemoryImageSource;

// Version 1.0: Abr'99 Recta (Algoritmo de Bresenham) + Crculo

public class ImageConstructor
{
  int width;
  int height;
  int pixels[];             // Pixels: ALPHA+RED+GREEN+BLUE -> 0xAARRGGBB

  // Constructor

  public ImageConstructor (int width, int height)
  {
    this.width  = width;
    this.height = height;
    this.pixels = new int[width * height];
  }

  // Image

  public Image getImage ()
  {
    return Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(width, height, pixels, 0, width));
  }

  // NOTA: Los bytes en Java se consideran codificados en complemento a 2

  private static int raw (byte b)
  {
    if (b<0)
       return 256+b;
    else
       return b;
  }

  private static int abs (int i)
  {
    if (i<0)
       return -i;
    else
       return i;
  }

  // Acceso individual a los pxeles de la imagen

  public void setPixel (int x, int y, int color)
  {
    pixels[y*width+x] = color;
  }

  public int getPixel (int x, int y)
  {
    return pixels[y*width+x];
  }

  // Operaciones sobre la imagen

  public void clear (int color)
  {
    int  x,y;
    int  index;
    
    index = 0;

    for (y=0; y<height; y++) {
        for (x=0; x<width; x++) {
            pixels[index++] = color;
            index++;
        }
    }
  }

  // Dibuja una lnea segn el algoritmo Bresenham

  public void line ( int x1, int y1, int x2, int y2, int color )
  {
    int d, dx, dy, aincr, bincr, xincr, yincr, x, y, tmp;

    if ( abs(x2-x1) < abs(y2-y1) ) {

       if ( y1 > y2 ) {
          tmp = x1; x1 = x2; x2 = tmp;
          tmp = y1; y1 = y2; y2 = tmp;
       }

       if ( x2 > x1 )
          xincr = 1;
       else
          xincr = -1;

       dy    = y2 - y1;
       dx    = abs( x2-x1 );
       d     = 2 * dx - dy;
       aincr = 2 * (dx - dy);
       bincr = 2 * dx;
       x     = x1;
       y     = y1;

       setPixel ( x, y, color );

       for (y=y1+1; y<=y2; y++) {

           if ( d >= 0 ) {
              x += xincr;
              d += aincr;
           } else
              d += bincr;

           setPixel( x, y, color );
       }

    } else {

       if ( x1 > x2 ) {
          tmp = x1; x1 = x2; x2 = tmp;
          tmp = y1; y1 = y2; y2 = tmp;
       }

       if ( y2 > y1 )
          yincr = 1;
       else
          yincr = -1;

       dx    = x2 - x1;
       dy    = abs( y2-y1 );
       d     = 2 * dy - dx;
       aincr = 2 * (dy - dx);
       bincr = 2 * dy;
       x     = x1;
       y     = y1;

       setPixel( x, y, color );

       for (x=x1+1; x<=x2; x++) {
           if ( d >= 0 ) {
              y += yincr;
              d += aincr;
           } else
              d += bincr;

           setPixel( x, y, color );
       }
    }
  }

  // Dibuja un rectngulo (su contorno)

  public void rectangle ( int x1, int y1, int x2, int y2, int color)
  {
    line ( x1, y1, x1, y2, color );
    line ( x1, y2, x2, y2, color );
    line ( x2, y2, x2, y1, color );
    line ( x2, y1, x1, y1, color );
  }

  // Dibuja un crculo

  public void circle ( int ox, int oy, int r, int color)
  {
    int r2 = r*r;
    int x,y;
    int xmin,xmax;
    int ymin,ymax;

    // Limits

    xmin = -r;

    if (xmin+ox<0)
       xmin = -ox;

    xmax = r;

    if (xmax+ox>width)
       xmax = width-ox;

    ymin = -r;

    if (ymin+oy<0)
       ymin = -oy;

    ymax = r;

    if (ymax+oy>height)
       ymax = height-oy;


    // Draw circle

    for (x=xmin; x<xmax; x++) {
        for (y=ymin; y<ymax; y++) {
            if (x*x+y*y<r2)
               setPixel (x+ox,y+oy, color);
        }
    }
  }

  // Diagrama de "sectores"

  public void sector ( int ox, int oy, int r, double pct, int color1, int color2)
  {
    int r2=r*r;
    int x,y;
    int dx,dy;
    int xmin,xmax;
    int ymin,ymax;
    double arc = Math.PI*pct;

    // Limits

    xmin = -r;

    if (xmin+ox<0)
       xmin = -ox;

    xmax = r;

    if (xmax+ox>width)
       xmax = width-ox;

    ymin = -r;

    if (ymin+oy<0)
       ymin = -oy;

    ymax = r;

    if (ymax+oy>height)
       ymax = height-oy;

    // Draw pie

    for (x=xmin; x<xmax; x++) {
        for (y=ymin; y<ymax; y++) {
            if (x*x+y*y<r2) {
               if ( Math.abs(Math.atan2((double)y,(double)x))<arc )
                  setPixel (x+ox,y+oy, color1);
               else
                  setPixel (x+ox,y+oy, color2);
            }
        }
    }
  }
}