package fbg.gui;

// Title:       ImagePanel
// Version:     1.0
// Copyright:   February 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;

/**
 * ImagePanel is an extension of java.awt.Panel 
 * which adds a background image to the panel.
 */

public class ImagePanel extends java.awt.Panel 
{
  /**
    * The background image to be displayed in the panel
    */
	
  private Image background;

  /**
   * Constructor (empty panel with no background)
   */

  public ImagePanel() 
  {
	super();
  } 

  /**
   * Constructor (empty panel with specified background)
   *
   * @param image the background to be displayed display in the ImagePanel
   */

  public ImagePanel (Image image) 
  {
	super();
	background = image;
  }

  /**
   * Constructor (empty panel with no background using the specified layout)
   *
   * @param	layout	the layout manager to use with the panel
   */

  public ImagePanel (java.awt.LayoutManager layout) 
  {
	super(layout);
  }

  /**
   * Paint the panel
   *
   * @param g Graphics
   */

  public void paint (Graphics g) 
  {
    if ( background != null )
	   g.drawImage(background,0,0,this);
  }

  /**
   * Set the ImagePanel's background image
   * @param image the Image to be displayed in the background
   */

  public void setBackground (Image image) 
  {
	background = image;
  }

}