package fbg.gui.html;

// Title:       SWING HTML Browser
// Version:     2.1
// Copyright:   January 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Vector;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;

import fbg.gui.GUI;
import fbg.util.Environment;

// SWING HTML Browser
// ------------------

// v1.0 Oct'98 JDK 1.1 & SWING 1.0.1
// v1.1 Jan'99 set & reset
// v2.0 Jan'99 JDK 1.2
// v2.1 Mar'99 URLs

public class HTML extends    JPanel
                  implements HyperlinkListener
{
  URL    home      = null;
  Vector historial = new Vector();
  int    actual    = -1;

  // Componentes

  ImageIcon upLeftButton;
  ImageIcon downLeftButton;
  ImageIcon disabledLeftButton;
  ImageIcon upHomeButton;
  ImageIcon downHomeButton;
  ImageIcon upRightButton;
  ImageIcon downRightButton;
  ImageIcon disabledRightButton;

  JScrollPane scroller      = new JScrollPane();
  JEditorPane html          = new JEditorPane();
  JPanel      jPanelButtons = new JPanel();

  JButton jButtonLeft = new JButton();
  JButton jButtonHome = new JButton();
  JButton jButtonRight = new JButton();

  BorderLayout borderLayoutPanel = new BorderLayout();
  FlowLayout flowLayoutButtons = new FlowLayout();

  // Layout

  public void initGUI()
  {
    if (Environment.settings==Environment.ENGLISH) {
       getAccessibleContext().setAccessibleName("HTML Viewer");
       getAccessibleContext().setAccessibleDescription("HTML Viewer");
    } else {
       getAccessibleContext().setAccessibleName("Visor HTML");
       getAccessibleContext().setAccessibleDescription("Visor HTML");
    }

    upLeftButton = loadImageIcon("buttons/left.gif");
    downLeftButton = loadImageIcon("buttons/leftDown.gif");
    disabledLeftButton = loadImageIcon("buttons/leftDisabled.gif");
    upHomeButton = loadImageIcon("buttons/home.gif");
    downHomeButton = loadImageIcon("buttons/homeDown.gif");
    upRightButton = loadImageIcon("buttons/right.gif");
    downRightButton = loadImageIcon("buttons/rightDown.gif");
    disabledRightButton = loadImageIcon("buttons/rightDisabled.gif");

    this.setMaximumSize(new Dimension(2048, 2048));
    this.setPreferredSize(new Dimension(300, 300));
    this.setMinimumSize(new Dimension(300, 300));

    html.setBackground(new Color(222, 222, 222));
    html.setFont(new Font("TimesRoman", 0, 12));
    html.setEditable(false);
    html.addHyperlinkListener(this);

    scroller.getViewport().add(html);

    jButtonLeft.addActionListener(new HTML_jButtonLeft_actionAdapter(this));
    jButtonHome.addActionListener(new HTML_jButtonHome_actionAdapter(this));
    jButtonRight.addActionListener(new HTML_jButtonRight_actionAdapter(this));

    if (Environment.settings==Environment.ENGLISH) {
       jButtonLeft.setToolTipText("Previous");
       jButtonHome.setToolTipText("Home");
       jButtonRight.setToolTipText("Next");
    } else {
       jButtonLeft.setToolTipText("Anterior");
       jButtonHome.setToolTipText("Inicio");
       jButtonRight.setToolTipText("Siguiente");
    }

    jButtonLeft.setBorderPainted(false);
    jButtonLeft.setFocusPainted(false);
    jButtonLeft.setIcon(upLeftButton);
    jButtonLeft.setPressedIcon(downLeftButton);
    jButtonLeft.setDisabledIcon(disabledLeftButton);

    jButtonHome.setBorderPainted(false);
    jButtonHome.setFocusPainted(false);
    jButtonHome.setIcon(upHomeButton);
    jButtonHome.setPressedIcon(downHomeButton);

    jButtonRight.setBorderPainted(false);
    jButtonRight.setFocusPainted(false);
    jButtonRight.setIcon(upRightButton);
    jButtonRight.setPressedIcon(downRightButton);
    jButtonRight.setDisabledIcon(disabledRightButton);

    jPanelButtons.setLayout(flowLayoutButtons);
    jPanelButtons.add(jButtonLeft, null);
    jPanelButtons.add(jButtonHome, null);
    jPanelButtons.add(jButtonRight, null);

    this.setLayout(borderLayoutPanel);
    this.add(scroller, BorderLayout.CENTER);
    this.add(jPanelButtons, BorderLayout.NORTH);
  }

  // Constructor

  public HTML (String file)
  {
    initGUI();
    reset(file);
  }

  public HTML (URL url)
  {
    initGUI();
    reset(url);
  }

  // RESET

  public void reset (String file)
  {
    File   f;
    String str;
    URL    url;

    try {

        f    = new File(file);

        /*
        str  = "file:"+f.getAbsolutePath();
        home = new URL(str);
        */
        
        url = f.toURL();

        reset (url);

    } catch (MalformedURLException e) {

        JOptionPane.showMessageDialog ( (Component)null,
                                        "URL incorrecta: "+file, // e.toString()
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
    } catch (IOException e) {

        JOptionPane.showMessageDialog ( (Component)null,
                                        "Archivo incorrecto: "+file, // e.toString()
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
    }
  }

  public void reset (URL url)
  {
    home      = url;
    historial = new Vector();

    if (home!=null) {
       jButtonHome.setVisible(true);
       actual = 0;
       historial.addElement(home);
       linkActivated(home);
    } else {
       jButtonHome.setVisible(false);
       actual = -1;
    }
  }

  // SET

  public void set (String file)
  {
    File   f;
    String str;
    URL    url;

    try {

        f   = new File(file);
        str = "file:"+f.getAbsolutePath();
        url = new URL(str);

        set(url);

    } catch (MalformedURLException e) {

        JOptionPane.showMessageDialog ( (Component)null,
                                        "URL incorrecta: "+file, // e.toString()
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
    } catch (IOException e) {

        JOptionPane.showMessageDialog ( (Component)null,
                                        "Archivo incorrecto: "+file, // e.toString()
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
    }
  }

  public void set (URL url)
  {
    actual++;
    historial.addElement(url);
    linkActivated(url);
  }

  // Imgenes

  private ImageIcon loadImageIcon (String file)
  {
    byte[] iconData = loadResource(getClass(),file);

    return new ImageIcon(iconData);
  }


  private byte[] loadResource (final Class baseClass, String file)
  {
    byte[] buffer = null;

    try {

      InputStream resource = baseClass.getResourceAsStream(file);

      if (resource == null) {
         System.err.println(baseClass.getName() + "/" + file + " not found.");
         return null;
      }

      BufferedInputStream   in  = new BufferedInputStream(resource);
      ByteArrayOutputStream out = new ByteArrayOutputStream(1024);

      buffer = new byte[1024];

      int n;

      while ((n = in.read(buffer)) > 0) {
            out.write(buffer, 0, n);
      }

      in.close();
      out.flush();

      buffer = out.toByteArray();

      if (buffer.length == 0) {
          System.err.println("Warning: " + file + " is zero-length");
          return null;
      }

    } catch (IOException ioe) {
       System.err.println(ioe.toString());
       return null;
    }

    return buffer;
  }

  // Botones

  private void showButtons ()
  {
    if (actual>0)
       jButtonLeft.setEnabled(true);
    else
       jButtonLeft.setEnabled(false);

    if (actual<historial.size()-1)
       jButtonRight.setEnabled(true);
    else
       jButtonRight.setEnabled(false);
  }

  void jButtonLeft_actionPerformed(ActionEvent e)
  {
    actual--;
    linkActivated((URL) historial.elementAt(actual));
  }

  void jButtonHome_actionPerformed(ActionEvent e)
  {
    actual++;

    while (historial.size()>actual)
          historial.removeElementAt(actual);

    historial.addElement(home);
    linkActivated(home);
  }

  void jButtonRight_actionPerformed(ActionEvent e)
  {
    actual++;
    linkActivated((URL) historial.elementAt(actual));
  }

  // Navegacin

  public void hyperlinkUpdate (HyperlinkEvent event)
  {
    URL url = event.getURL();

    if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {

       actual++;

       while (historial.size()>actual)
             historial.removeElementAt(actual);

       historial.addElement(url);
       linkActivated(url);
    }
  }

  // Activacin de una URL

  protected void linkActivated (URL url)
  {
    GUI.setWaitCursor(html);

    showButtons();

    SwingUtilities.invokeLater(new PageLoader(url));
  }

  // Carga sncrona de un documento

  class PageLoader implements Runnable
  {
    URL url;

    PageLoader(URL location)
    {
      url = location;
    }

    public void run()
    {
      if (url == null) {

	 GUI.setDefaultCursor(html);

	 Container parent = html.getParent();
	 parent.repaint();

      } else {

         Document doc = html.getDocument();

	 try {
	     html.setPage(url);
	 } catch (IOException ioe) {
	     html.setDocument(doc);
	     getToolkit().beep();
	 } finally {
	     // Restauracin del cursor
	     url = null;
	     SwingUtilities.invokeLater(this);
	 }
      }
    }
  }

}

// Manejo de eventos
// -----------------

class HTML_jButtonLeft_actionAdapter implements java.awt.event.ActionListener
{
  HTML adaptee;

  HTML_jButtonLeft_actionAdapter(HTML adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonLeft_actionPerformed(e);
  }
}

class HTML_jButtonHome_actionAdapter implements java.awt.event.ActionListener
{
  HTML adaptee;

  HTML_jButtonHome_actionAdapter(HTML adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonHome_actionPerformed(e);
  }
}

class HTML_jButtonRight_actionAdapter implements java.awt.event.ActionListener
{
  HTML adaptee;

  HTML_jButtonRight_actionAdapter(HTML adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonRight_actionPerformed(e);
  }
}

