package fbg.jdbc.server;

// Title:       JDBC Server Data
// Version:     1.0
// Copyright:   February 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import java.util.StringTokenizer;

// Data
// ----
// Format = ASCII file (vg: C4.5)

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers
// v1.2  Feb'99 JDBC Server


/**
 * <p>
 * Input data model
 *
 * <p>
 * Format: ASCII file (vg: C4.5)
 *
 * @version 1.2
 * @author  Fernando Berzal Galiano
 */


public class AsciiData 
{
  // Variables de instancia
  // ----------------------

  int              atributos = 0;     // Nmero de atributos
  String           id        = null;  // Identificador de la tabla
  int              index     = -1;    // Cdigo de la tabla en la base de datos 
  AsciiDatabase    database  = null;  // Base de datos

  String           file      = null;
  RandomAccessFile fichero   = null;


  // Constructor
  // -----------

  public AsciiData (AsciiDatabase db, String table)
  {
    database = db; 
    id       = table;
    index    = db.getTable(id);

    if (index>=0) {
    
       atributos = db.getAttributes(index).length;
       file      = db.getFile(index);

       try {
  
           fichero = new RandomAccessFile (file, "r" );

       } catch (IOException e) {
 
           System.err.println(e.getMessage());
       }
    }
  }

  // Acceso secuencial

  public String[] next ()
  {
    String array[] = null;
    String ascii;

    try {

        if (fichero!=null) {

           ascii = fichero.readLine();

           if (ascii!=null) {
              array = parse (ascii.trim());
           }
           
           if (ascii==null || array==null) {
              fichero.close();
              fichero = null;
           }
        }

    } catch (IOException e) {

        System.out.println(e.getMessage());
    }

    return array;
  }

  // Anlisis de una lnea del fichero ASCII

  private String[] parse (String ascii)
  {
    String[]        val   = new String[atributos];
    String          token = "";
    StringTokenizer st    = new StringTokenizer(ascii, ", \t\r\n"); // ",\r\n"
    int             pos   = 0;

    do {
       token = nextToken(st);
       val[pos] = token;
       pos++;
    } while (pos<atributos && token!=null);

    if (token!=null) 
       return val;
    else
       return null;
  }

  // nextToken

  private static String nextToken (StringTokenizer st)
  {
    if (st.hasMoreTokens())
       return st.nextToken().trim();
    else
       return null;
  }

  // Metainformacin

  public int getAttributes ()
  {
    return atributos;
  }

  public String getAttribute (int i)
  {
    return database.getAttribute(index,i);
  } 

  public String getType (int i)
  {
    return database.getType(index,i);
  }

  public String ID ()
  {
    return id;
  }

}