package fbg.jdbc.server;

// Title:       JDBC Server for ASCII files
// Version:     1.0
// Copyright:   February 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import fbg.util.ConfigFile;

/**
    Configuration file format
    -------------------------

    * Similar to MS Windows INI files

    * Sections can be disordered

    Common sections
    ---------------

    [DESCRIPTION]
    Database description (several lines if needed)

    [TABLES]
    Table identifiers

    [FILES]
    ASCII data files

    [<table>]
    Attribute IDs (one per line)

*/


public class AsciiDatabase
{
  // Dataset description

  String description[] = null;

  // Tables, files & attributes

  String path;

  String tables[]       = null;
  String files[]        = null;
  String attributes[][] = null;
  String types[][]      = null;


  // Constructor
  // -----------

  public AsciiDatabase (String fileID)
  {
    int        i;
    ConfigFile cfg = new ConfigFile(fileID);

    // Path

    path = cfg.path();

    // Description

    description = cfg.Section("[DESCRIPTION]");

    // Tables

    tables     = cfg.Section("[TABLES]");
    files      = cfg.Section("[FILES]");
    attributes = new String[tables.length][];
    types      = new String[tables.length][];

    for (i=0; i<tables.length; i++) {
        attributes[i] = cfg.Section("["+tables[i]+" ATTRIBUTES]");
        types[i]      = cfg.Section("["+tables[i]+" TYPES]");
    }
  }

  // Access to data
  // --------------

  public String[] getDescription ()
  {
    return description;
  }

  public String getDescriptionString ()
  {
    String str = "";
    int    i;

    if (description!=null)
       for (i=0; i<description.length; i++)
           str += description[i]+"\n";

    return str;
  }

  public int getTables ()
  {
    return tables.length;
  }

  public int getTable (String id)
  {
    int pos;

    for (pos=0; pos<tables.length; pos++)
        if (tables[pos].toUpperCase().equals(id))
           return pos;

    return -1;
  }

  public String getID (int i)
  { 
    return tables[i];
  }

  public String getFile (int i)
  {
    return path+"/"+files[i];
  }

  public String[] getAttributes (int i)
  {
    return attributes[i];
  }

  public String getAttribute (int i, int j)
  {
    return attributes[i][j];
  }

  public String[] getTypes (int i)
  {
    return types[i];
  }

  public String getType (int i, int j)
  {
    return types[i][j];
  }
  // Report
  // ------

  public void print ()
  {
    int i,j;

    // Database description

    printStringArray(description);

    // Database tables

    for (i=0; i<tables.length; i++) {

        System.out.println("TABLE "+tables[i]);
        System.out.println("FILE "+files[i]);

        for (j=0; j<attributes[i].length; j++)
            System.out.println("- "+attributes[i][j] + " ("+types[i][j]+")");
    }
  }

  private void printStringArray (String array[])
  {
    int i;

    if (array!=null)
       for (i=0; i<array.length; i++)
           System.out.println(array[i]);
  }



  // TEST

  public static void main (String args[])
  {
    // DataConfiguration cfg = new DataConfiguration("igaliko.cfg");
    // DataConfiguration cfg = new DataConfiguration("galaxy.cfg");
    // DataConfiguration cfg = new DataConfiguration("bupa.cfg");
    // DataConfiguration cfg = new DataConfiguration("iono.cfg");
    // DataConfiguration cfg = new DataConfiguration("jdbc.cfg");

    AsciiDatabase db = new AsciiDatabase(args[0]);

    db.print();
  }

}