package fbg.jdbc.server;

// Title:       JDBC Server for ASCII files
// Version:     1.0
// Copyright:   February 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.sql.*;

import java.util.StringTokenizer;
import java.util.Vector;

// Consulta NanoSQL: SELECT
// ------------------------

// SELECT [DISTINCT|ALL]
//        { * | field [, field ]... } [, COUNT(*)]
//        FROM table
//        [GROUP BY field ]
//        [ORDER BY field [ASC|DESC] ]

public class AsciiQuery
{
  String          sql;
  StringTokenizer st;
  String          token;

  AsciiData tabla    = null;

  Vector   records    = null;         // Tuplas 
  Vector   columns    = new Vector(); // columnas
  int      fields[];
  boolean  distinct   = false;        // DISTINCT
  boolean  wildcard   = false;        // *
  boolean  count      = false;        // COUNT(*)
  String   group      = null;         // GROUP BY group
  String   orden      = null;         // ORDER BY orden
  boolean  ascendente = true;         // Orden ascendente


  // Constructor: Anlisis sintctico y ejecucin de la consulta
  // -----------------------------------------------------------

  public AsciiQuery (AsciiDatabase db, String select)
  {
    sql = select.toUpperCase();
    st  = new StringTokenizer(sql, " ,;\t\r\n");

    nextToken();

    if (token!=null && token.equals("SELECT")) {

       nextToken();

       // DISTINCT

       if (token!=null && token.equals("DISTINCT")) {
          distinct = true;
          nextToken();
       }

       // ALL

       if (token!=null && token.equals("ALL")) {
          distinct = false;
          nextToken();
       }

       // COLUMNAS

       columnas();

       // FROM

       nextToken();

       if (token!=null) {

          tabla = new AsciiData(db,token);

          if (tabla!=null)
             dataset();    // Tabla resultado de la consulta
       }

       nextToken();

       // Clasulas

       if (tabla!=null) {

          while (token!=null) {

             if (token.equals("WHERE")) {         // WHERE

                // Not supported yet

             } else if (token.equals("GROUP")) {  // GROUP BY

                agrupamiento();

             } else if (token.equals("ORDER")) {  // ORDER BY

                orden();
             }
          }
       }

       // Depuracin:
       // debug();

       // Ejecutar la consulta analizada

       if (tabla!=null)
          execute();
    }
  }

  // COLUMNAS

  private void columnas ()
  {
    while (token!=null && !token.equals("FROM")) {

       if (token.equals("*")) {

          // Comodn: todas las columnas

          wildcard = true;

       } else if (token.equals("COUNT(*)")) {

          // COUNT(*)

          distinct = true;
          count    = true;

       } else {

          // Columna

          columns.addElement(token);
       }

       nextToken();
    }
  }

  // DATASET

  private void dataset ()
  {
    int   i,j;
    String col;


    fields = new int[tabla.getAttributes()];

    for (i=0; i<fields.length; i++)
        fields[i] = -1;

    if (wildcard) {

       columns = new Vector();

       for (i=0; i<tabla.getAttributes(); i++) {
           columns.add ( tabla.getAttribute(i) );
           fields[i] = i;
       }

    } else {

       for (j=0; j<columns.size(); j++) {

           col = (String) columns.elementAt(j);

           for (i=0; i<tabla.getAttributes(); i++)
               if (col.equals(tabla.getAttribute(i))) {
                  fields[i] = j;
               }
       }
    }
  }

  // GROUP BY

  private void agrupamiento ()
  {
    nextToken();

    if (token!=null && token.equals("BY")) {

       nextToken();

       if (token!=null) {
          group = token;
          nextToken();
       }
    }
  }

  // ORDER BY

  private void orden ()
  {
    nextToken();

    if (token!=null && token.equals("BY")) {

       nextToken();

       if (token!=null) {
          orden = token;
          nextToken();
       }

       if (token!=null) {

          if (token.equals("ASC")) {
             ascendente = true;
             nextToken();
          } else if (token.equals("DESC")) {
             ascendente = false;
             nextToken();
          }
       }
    }
  }

  // nextToken

  private void nextToken ()
  {
    if (st.hasMoreTokens())
       token = st.nextToken();
    else
       token = null;
  }

  // Quedan ms tokens

  private boolean more ()
  {
    return st.hasMoreTokens();
  }

  // Depuracin
  // ----------
  // Informacin relativa a la consulta SQL

  private void debug ()
  {
    int i;

    if (tabla!=null) {

       System.out.println("Tabla original: "+tabla.ID());

       for (i=0; i<tabla.getAttributes(); i++)
           System.out.println(tabla.getAttribute(i));
    }

    System.out.println("Tabla obtenida:");

    for (i=0; i<columns.size(); i++)
        System.out.println((String)columns.elementAt(i));

    System.out.println ("Opciones de la consulta");

    if (distinct)
       System.out.println("Sin duplicados");

    if (count)
       System.out.println("COUNT");

    if (orden!=null) {
       System.out.print("Ordenar por "+orden);

       if (ascendente)
          System.out.println (" (orden ascendente)");
       else
          System.out.println (" (orden descendente)");
    }

    if (group!=null)
       System.out.println("Agrupar por "+group);
  }

  // EJECUCIN DE LA CONSULTA SQL
  // ----------------------------

  int actual = 0;

  public String[] next ()
  {
    String tupla[];
    
    if (records!=null) {

       if (actual<records.size()) {
          tupla = (String[])records.elementAt(actual);
          actual++;
          return tupla;
       } else {
          return null;
       }

    } else {

       tupla = tabla.next();

       if (tupla!=null) {
          return result(tupla);
       } else {
          return null;
       }
    }
  }

  // Ejecucin previa de la consulta completa

  private void execute ()
  {
    String tupla[];
    String nueva[];
    String aux[];
    int    contador;
    int    n = 0;

    if (orden!=null || distinct || count) {

       records = new Vector();

       tupla = tabla.next();

       while (tupla!=null) {

             n++;

             if (n%100==0)
                System.out.print(".");

             // Clasula WHERE
             // ...

             if (!distinct) {

                nueva = result (tupla);
                records.addElement(nueva);

             } else {

                // DISTINCT

                aux = find(tupla);

                if (aux!=null) {

                   if (count) {   // COUNT ++

                      contador = Integer.parseInt(aux[aux.length-1]);

                      contador ++;

                      aux[aux.length-1] = ""+contador;
                   }

                } else {

                   nueva = result (tupla);
                   records.addElement(nueva);

                }
             }

             tupla = tabla.next();
       }

       System.out.println();
    }

    // Ordenacin: ORDER BY && GROUP BY

    if (orden!=null || group!=null) {

       if (orden!=null)
          sort(orden);
       else
          sort(group);
    }
  }

  // Nueva tupla en la tabla resultado

  private String[] result (String tupla[])
  {
    int    i;
    int    dim = (count)? columns.size()+1: columns.size();
    String nuevo[] = new String[dim];

    for (i=0; i<tabla.getAttributes(); i++) {

        if (fields[i]!=-1) {
           nuevo[fields[i]] = tupla[i];
        }
    }

    if (count)
       nuevo[dim-1] = "1";

    return nuevo;
  }

  // Busca un registro en el vector de registros

  private String[] find (String reg[])
  {
    String tmp[];
    int    i;

    for (i=0; i<records.size(); i++) {

        tmp = (String[])records.elementAt(i);

        if (compatible(tmp,reg))
           return tmp;
    }

    return null;
  }


  private boolean compatible (String regquery[], String regtabla[])
  {
    String     strq, strt;
    int        i;

    for (i=0; i<tabla.getAttributes(); i++) {

        if (fields[i]!=-1) {

           strq = regquery[fields[i]];
           strt = regtabla[i];

           if (  (strq!=null && strt!=null && !strq.equals(strt))
              || (strq==null && strt!=null)
              || (strq!=null && strt==null) )
              return false;
        }
    }

    return true;
  }

  // Ordenacin del vector de registros
  // MERGESORT ITERATIVO

  private void sort (String field)
  {
    int       col    = sortField(field);
    int       N      = records.size();
    int       ancho  = 2;
    boolean   number = type(col).toUpperCase().equals("NUMBER");
    int       inicio, pos, izq, der, med, ext, i;
    String    tmp[][] = new String[N][];
    String    r1[], r2[];

    if ( col>=0 ) {

       while (ancho<2*N) {

          for (inicio=0; inicio<N-1; inicio+=ancho) {

              pos = 0;
              izq = inicio;
              med = inicio + ancho/2;
              der = med;
              ext = (inicio+ancho<N)? inicio+ancho: N;

              if (med<N) {

                 while (izq<med && der<ext) {

                       r1 = (String[])records.elementAt(izq);
                       r2 = (String[])records.elementAt(der);

                       if ( CmpRecords(r1,r2,col,number)<=0 ) {
                          tmp[pos++] = r1;
                          izq++;
                       } else {
                          tmp[pos++] = r2;
                          der++;
                       }
                 }

                 while (izq<med) {
                       tmp[pos++] = (String[])records.elementAt(izq);
                       izq++;
                 }

                 for (i=0; i<pos; i++) {
                     // tmp[i] ---> records[inicio+i]
                     records.setElementAt ( tmp[i], inicio+i);
                 }
              }
          }

          ancho *= 2;
       }
    }
  }

  // Comparacin de registros (campo n)

  private int CmpRecords ( String r1[], String r2[], int n, boolean number)
  {
    int    cmp = 0;
    String str1 = r1[n];
    String str2 = r2[n];
    float  n1;
    float  n2;

    if (number) {

       // Nmeros

       if (str1!=null && str2!=null) {

          n1 = Float.parseFloat(str1);
          n2 = Float.parseFloat(str2);

          if (n1<n2) {
             cmp = -1;
          } else if (n1==n2) {
             cmp = 0;
          } else {
             cmp = 1;
          }

       } else if (str1==null && str2!=null) {
          cmp = -1;
       } else if (str1!=null && str2==null) {
          cmp = 1;
       } else {
          cmp = 0;
       }

    } else {

       // Cadenas
       
       if (str1!=null && str2!=null) {
          cmp = CmpStrings(str1,str2);
       } else if (str1==null && str2!=null) {
          cmp = -1;
       } else if (str1!=null && str2==null) {
          cmp = 1;
       } else {
          cmp = 0;
       }
    }

    if (ascendente)
       return cmp;
    else
       return -cmp;
  }


  private int CmpStrings (String s1, String s2)
  {
    return s1.compareTo(s2);
  }

  private int sortField (String field)
  {
    int i;

    for (i=0; i<tabla.getAttributes(); i++)
        if (field.equals(tabla.getAttribute(i)))
           return fields[i];

    if (field.equals("COUNT(*)"))
       return columns.size();

    return -1;
  }

  // Metainformacin
  // ---------------

  // Nmero de columnas resultado de la consulta

  public int size ()
  {
    return (count)? columns.size()+1: columns.size();
  }

  public String type (int i)
  {
    int pos;

    if (count && i==columns.size()) {
       return "Number";                 // COUNT(*) = INTEGER
    } else {

       for (pos=0; pos<tabla.getAttributes(); pos++)
           if (fields[pos]==i)
              return tabla.getType(pos);

       return "Null";
    }
  }

  public String field (int i)
  {
    int pos;

    if (count && i==columns.size()) {
       return "Count(*)";
    } else {

       for (pos=0; pos<tabla.getAttributes(); pos++)
           if (fields[pos]==i)
              return tabla.getAttribute(pos);

       return "Null";
    }
  }

  // Salida
  // ------

  public void print ()
  {
    int i,j;

    for (i=0; i<records.size(); i++) {
        for (j=0; j<columns.size(); j++) {
            System.out.println (((String[])records.elementAt(i))[j]);
        }
    }
  }

  // PRUEBA
  // ------
  // AsciiQuery <db> <query>

  public static void main (String args[])
  {
    AsciiDatabase db = new AsciiDatabase(args[0]);
    AsciiQuery    query = new AsciiQuery(db,args[1]);
    int           contador = 0;
    String        tupla[] = null;

    do {

      tupla = query.next();

      if (tupla!=null) {
         contador++;
         printStringArray(tupla);
      }

    } while (tupla!=null);

    System.out.println(contador+" tuplas");
  }

  private static void printStringArray (String array[])
  {
    int i;

    if (array!=null)
       for (i=0; i<array.length; i++)
           System.out.print(array[i]+" ");

    System.out.println();
  }

}