package fbg.jdbc.server;

// Title:       JDBC Server
// Version:     1.1
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.net.*;
import java.io.*;

import java.util.Vector;

// Protocolo de comunicacin

// v1.0 Feb'99 Nano Server
// v1.1 May'99 Garbage Collector

public class ServerThread extends Thread 
{
  // Conexin de red 

  private Socket socket = null;

  // I/O

  // private PrintStream out;      // JDK 1.0
  // private DataInputStream in;
  
  private PrintWriter    out;      // JDK 1.1
  private BufferedReader in;

  // Constructor
  // -----------
  // Establece enlace entre cliente y servidor

  public ServerThread (Socket socket) 
  {
    super("JDBC Server Thread");

    // Socket
        
    this.socket = socket;
  }

  // Hebra
  // -----
  // Comunicacin con el cliente

  public void run () 
  {
    boolean ok = true;

    // I/O STREAMS

    try {

        // out = new PrintStream(socket.getOutputStream());     // JDK 1.0
        // in  = new DataInputStream(socket.getInputStream());
        
        out = new PrintWriter(socket.getOutputStream(), true);  // JDK 1.1
        in  = new BufferedReader(new InputStreamReader(socket.getInputStream()));

    } catch (IOException e) {
        System.err.println("Error opening I/O Streams");
        ok = false;
    }

    // Protocolo

    if (ok)
       protocol();

    // Cierre de las conexiones
    
    if (ok)
       try {
           socket.close();
           out.close();
           in.close();
       } catch (IOException e) {
           System.out.println("Error closing network connection");
           ok = false;
       }

    // Recolector de basura

    System.gc();
  }

  // PROTOCOLO
  // ---------

  void protocol()
  {
    boolean       end;
    String        message[];

    AsciiDatabase db = null;

    try {

       // Comienzo del dilogo

       send(Protocol.INIT+"\nFBG NanoServer\n\n");

       // Bucle principal

       end = false;

       while (!end) {

         System.gc();

         message = receive();

         if (message.length<1 || message[0].equals("") ) {

            // Mensaje vaco 
            
            send(Protocol.MESSAGE+"\nEmpty message\n\n");

         } else if ( message[0].equals(Protocol.CLOSE) ) {

            // Cierre de la conexin

            System.out.println("Connection closed");

            send(Protocol.CLOSE+"\n\n");

            end = true;

         } else if ( message[0].equals(Protocol.LOGIN) ) {

            // Autentificacin

            if (message.length<4) 
               throw new ServerException("Login incorrect");
            
            System.out.println("Database: "+message[1]);
            System.out.println("Login:    "+message[2]);
            System.out.println("Password: "+message[3]);

            db = new AsciiDatabase(message[1]);

            send(Protocol.MESSAGE+"\n"+message[1]+": Access granted to user '"+message[2]+"'\n\n");

         } else if (message[0].equals(Protocol.SQL) ) {

            if (message.length<2)
               throw new ServerException("NanoSQL message error");
            else if (db==null)
               throw new ServerException("Not connected to any database");

            NanoSQL (db,message);

         } else if (message[0].equals(Protocol.TABLES) ) {

            Tables (db);

         } else if (message[0].equals(Protocol.COLUMNS) ) {

            if (message.length<2)
               throw new ServerException("Request error");

            Columns(db, message[1]);

         } else {

            send(Protocol.MESSAGE+"\nUnknown option\n\n");

         }
       }

    } catch (ServerException error) {

       System.out.println("ERROR: "+error.toString());
       // send(Protocol.MESSAGE+"\n"+error.toString()+"\n\n");
       send(Protocol.CLOSE+"\n\n");

    } catch(IOException e){

       System.out.println("Error en la comunicacin cliente-servidor");
       send(Protocol.MESSAGE+"\nError en la comunicacin cliente-servidor\n\n");
    }
  }

  // Envo de datos

  void send (String cadena)
  {
    out.print(cadena);
    out.flush();
  }

  // Recepcin de datos (cada peticin finaliza con "")

  String[] receive () throws IOException 
  {
    Vector command = new Vector();
    String array[];
    String tmp;
    int    i;

    do {

       tmp = in.readLine();

       if (!tmp.equals(""))
          command.add(tmp);
          
    } while (!tmp.equals(""));

    array = new String[command.size()];

    for (i=0; i<array.length; i++)
        array[i] = (String) command.elementAt(i);

    command = null;

    return array;
  }

  // -------
  // -------
  // NanoSQL
  // -------
  // -------

  // SELECT [DISTINCT|ALL]
  //        { * | field [, field ]... } [, COUNT(*)]
  //        FROM table
  //        [GROUP BY field ]
  //        [ORDER BY field [ASC|DESC] ]

  public void NanoSQL (AsciiDatabase db, String message[])
  {
    int    i;
    String str;

    // Consulta

    String sql = message[1];

    for (i=2; i<message.length; i++) 
        sql += "\n"+message[i];

    AsciiQuery query = new AsciiQuery (db, sql);

    // Envo de los datos

    String tuple[];
    int    field;
    int    dim = query.size();

    send(Protocol.RESULT+"\n");
    send(dim+"\n");

    for (field=0; field<dim; field++) {
        send(query.field(field)+"\n");
        send(query.type(field)+"\n");
    }

    i = 0;

    do {

       tuple = query.next();

       if (tuple!=null) {

          i++;

          str = "";

          for (field=0; field<dim; field++)
              str = str + tuple[field]+ "\n";

          send(str);

          if (i%100==0)
             System.out.print(".");

       } else {
          send("\n");
       }

    } while (tuple!=null);

    System.out.println();

  }

  // Consulta al catlogo
  
  public void Tables (AsciiDatabase db)
  {
    // Envo de los datos

    int    i;

    send(Protocol.RESULT+"\n");
    send("1\n");
    send("TABLE_NAME\n");
    send("CHAR\n");

    for (i=0; i<db.getTables(); i++) {
        send(db.getID(i)+"\n");
    }

    send("\n");
  }

  public void Columns (AsciiDatabase db, String table)
  {
    // Envo de los datos

    int    i;
    String id = table.toUpperCase();
    int    index = db.getTable(id);
    String attr[];
    String type[];

    if (index>=0) {

       attr = db.getAttributes(index);
       type = db.getTypes(index);

       send(Protocol.RESULT+"\n");
       send("6\n");
       send("TABLE_NAME\n");
       send("CHAR\n");
       send("COLUMN_NAME\n");
       send("CHAR\n");
       send("DATA_TYPE\n");
       send("NUMBER\n");
       send("TYPE_NAME\n");
       send("CHAR\n");
       send("COLUMN_SIZE\n");
       send("NUMBER\n");
       send("DECIMAL_DIGITS\n");
       send("NUMBER\n");

       for (i=0; i<attr.length; i++) {

           send(id+"\n");
           send(attr[i].toUpperCase()+"\n");

           if (type[i].startsWith("N")) {
              send(java.sql.Types.NUMERIC+"\n");
           } else {
              send(java.sql.Types.CHAR+"\n");
           }

           send(type[i].toUpperCase()+"\n");

           send("10\n");
           send("3\n");
       }
    
    } else {

       send(Protocol.RESULT+"\n");
       send("0\n");
    }

    send("\n");
  }

}

