/* ------------------------------------------------------------- */
/*    Fernando Berzal Galiano       ETS Ingeniera Informtica   */
/* ------------------------------------------------------------- */
/*                         TDA  Bosque                           */
/* ------------------------------------------------------------- */

package fbg.tda;

// Puede considerarse como:
//
//   Un bosque de rboles:
//      - Representacin HIJO_IZQUIERDA - HERMANO_DERECHA 
//
//   Un rbol binario

/* ------------------------------------------------------------- */

public class Arbol
{
  private NodoArbol cabecera;


  // Constructor

  public Arbol ()
  {
    cabecera = new NodoArbol();
  }


  public boolean Vacio ()
  {
    return (cabecera.hijo==null);
  }


  public NodoArbol Raiz ()
  {
    return cabecera.hijo;
  }

  public NodoArbol Raiz (Object dato)
  {
    NodoArbol nodo = new NodoArbol (dato);
  
    if (nodo!=null) {
       nodo.padre    = cabecera;
       cabecera.hijo = nodo; 
    }
   
    return nodo;
  }


  public NodoArbol Hijo (NodoArbol nodo, Object dato)
  {
    NodoArbol aux = new NodoArbol (dato);

    if (aux!=null) {
       aux.padre = nodo;
       nodo.hijo = aux;
    }

    return aux;
  }

  public NodoArbol Hermano (NodoArbol nodo, Object dato)
  {
    NodoArbol aux = new NodoArbol (dato);

    if (aux!=null) {
       aux.padre    = nodo.padre;
       nodo.hermano = aux;
    }

    return aux;
  }

  // Listados de un rbol
  // --------------------
  
  // PREORDEN
 
  private void PreOrden (NodoArbol nodo, Lista lista)
  {
    NodoArbol aux;

    if (nodo!=null) {
       lista.Insertar(lista.Fin(), nodo.info);
       aux = nodo.hijo;

       while (aux!=null) {
	   PreOrden (aux, lista);
	   aux = aux.hermano;
       }
    }
  }

  Lista PreOrden ()
  {
    Lista     lista = new Lista ();
    NodoArbol nodo  = Raiz();

    while (nodo!=null) {
	  PreOrden (nodo, lista);
	  nodo = nodo.hermano;
    }

    return lista;
  }

  // INORDEN
  
  private void InOrden (NodoArbol nodo, Lista lista)
  {
    NodoArbol aux;

    if (nodo!=null) {
       aux = nodo.hijo;
       InOrden (aux, lista);
       lista.Insertar (lista.Fin(), nodo.info);

       while (aux!=null) {
             aux = aux.hermano;
             InOrden (aux,lista);
       }
    }
  }

  Lista InOrden ()
  {
    Lista     lista = new Lista();
    NodoArbol nodo  = Raiz();

    while (nodo!=null) {
	  InOrden (nodo, lista);
	  nodo = nodo.hermano;
    }

    return lista;
  }

  // POSTORDEN

  private void PostOrden (NodoArbol nodo, Lista lista)
  {
    NodoArbol aux;

    if (nodo!=null) {
       aux = nodo.hijo;

       while (aux!=null) {
	     PostOrden (aux, lista);
	     aux = aux.hermano;
       }

       lista.Insertar (lista.Fin(), nodo.info);
    }
  }

  Lista PostOrden ()
  { 
    Lista     lista = new Lista();
    NodoArbol nodo  = Raiz();

    while (nodo!=null) {
	  PostOrden (nodo, lista);
	  nodo = nodo.hermano;
    }

    return lista;
  }

  // POR NIVELES

  Lista PorNiveles ()
  {
    NodoArbol nodo  = Raiz();
    Lista     lista = new Lista();
    Cola      cola  = new Cola();

    cola.Encolar(nodo);

    if (nodo!=null)
       do {
	   nodo = nodo.hermano;
	   cola.Encolar(nodo);
       } while (nodo!=null);

    while (!cola.Vacia()) {
	
       nodo = (NodoArbol) cola.Extraer();

       if (nodo!=null)
	  lista.Insertar (lista.Fin(), nodo.info);
       else
	  lista.Insertar (lista.Fin(), null);

       if (nodo!=null) {
	  nodo = nodo.hijo;;
	  cola.Encolar(nodo);

	  if (nodo!=null)
	     do {
		 nodo = nodo.hermano;
		 cola.Encolar (nodo);
	     } while (nodo!=null);
       }
    }

    return lista;
  }

  // Constructor de un rbol a partir de un listado por niveles 

  public Arbol (Lista l)
  {
    this();   // Llama al constructor habitual

    Cola      cola = new Cola ();
    Object    dato;
    NodoLista nodo_lista;
    NodoArbol nodo_arbol;

    /* Races de los rboles del bosque */

    nodo_lista = l.Inicio();
    dato       = l.Leer(nodo_lista);

    if (dato!=null) {
       nodo_arbol = Raiz ( dato );
       cola.Encolar ( nodo_arbol );
       nodo_lista = l.Siguiente ( nodo_lista );
       dato = l.Leer ( nodo_lista );

       while (dato!=null) {
             nodo_arbol = Hermano ( nodo_arbol, dato );
	     cola.Encolar ( nodo_arbol );
	     nodo_lista = l.Siguiente ( nodo_lista );
	     dato = l.Leer ( nodo_lista );
       }
    }

    /* Resto de los nodos */

    while (!cola.Vacia()) {
          nodo_arbol = (NodoArbol) cola.Extraer ();
	  nodo_lista = l.Siguiente ( nodo_lista );
	  dato = l.Leer ( nodo_lista);

  	  if (dato!=null) {
             nodo_arbol = Hijo (nodo_arbol, dato);
	     cola.Encolar ( nodo_arbol );
	     nodo_lista = l.Siguiente ( nodo_lista );
	     dato = l.Leer ( nodo_lista );

	     while (dato!=null) {
                   nodo_arbol = Hermano ( nodo_arbol, dato );
	           cola.Encolar ( nodo_arbol );
	           nodo_lista = l.Siguiente ( nodo_lista );
	           dato = l.Leer ( nodo_lista );
	     }
	   }
    }
  }
}

/* ------------------------------------------------------------- */

