package fbg.util.db;

// Title:       TMiner
// Version:     1.1
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.sql.*;

import fbg.tda.Lista;

// Campo
// -----

// Versin 1.0: Ago'98 Modelizacin de campos (columnas de una tabla)
// Versin 1.1: Jun'99 JDBC Metadata (Object->String)

abstract public class Campo
{
  String id;
  Tabla  tabla;

  public Valores  valores;
  public Dominios dominios;

  // Constructor
  // -----------

  public Campo (Tabla t, String nombre)
  {
    tabla      = t;
    id         = new String(nombre);
    valores    = new Valores(this);
    dominios   = new Dominios(this);
  }

  // Metadata
  // --------

       // Oracle Lite
       // -----------
       // Tipo NUMBER (data_type 2): PRECISION (size) + SCALE (0=Integer)
       // Tipo CHAR   (data_type 1): LENGTH

       // Oracle 8
       // -----------
       // Tipo NUMBER (data_type 2): COLUMN_SIZE + DECIMAL_DIGITS (0=Integer)
       // Tipo CHAR   (data_type 1): COLUMN_SIZE

  public static String Type (ResultSet metadata)
                       throws SQLException
  {
    ResultSetMetaData aux = metadata.getMetaData();

    int    total = aux.getColumnCount();
    int    i;
    String id;

    for (i=1; i<=total; i++) {
        id = aux.getColumnName(i);

        if ( id.equals("TYPE_NAME") ) // JDBC
           return id;
    }

    return null;
  }

  public static String Length (ResultSet metadata)
                       throws SQLException
  {
    ResultSetMetaData aux = metadata.getMetaData();

    int    total = aux.getColumnCount();
    int    i;
    String id;

    for (i=1; i<=total; i++) {
        id = aux.getColumnName(i);

        if (  id.equals("LENGTH")         // Oracle Lite
           || id.equals("COLUMN_SIZE") )  // Oracle 8
           return id;
    }

    return null;
  }

  public static String Size (ResultSet metadata)
                       throws SQLException
  {
    ResultSetMetaData aux = metadata.getMetaData();

    int    total = aux.getColumnCount();
    int    i;
    String id;

    for (i=1; i<=total; i++) {
        id = aux.getColumnName(i);

        if (  id.equals("PRECISION")      // Oracle Lite
           || id.equals("COLUMN_SIZE") )  // Oracle 8
           return id;
    }

    return null;
  }
 
  public static String Scale (ResultSet metadata)
                       throws SQLException
  {
    ResultSetMetaData aux = metadata.getMetaData();

    int    total = aux.getColumnCount();
    int    i;
    String id;

    for (i=1; i<=total; i++) {
        id = aux.getColumnName(i);

        if (  id.equals("SCALE")             // Oracle Lite
           || id.equals("DECIMAL_DIGITS") )  // Oracle 8
           return id;
    }

    return null;
  }

  // ID
  // --

  public String ID ()
  {
    return id;
  }

  // Valores
  // -------

  // Consulta

  public int Valores ()
  {
    return valores.Total();
  }

  public Valor Valor (int i)
  {
    return valores.Valor(i);
  }

  // Devuelve el valor correspondiente a ID

  public Object ObjetoValor (int ID)
  {
    return valores.ObjetoValor (ID);
  }

  // Devuelve ID de un valor: [0..Valores]

  abstract public int CodigoValor (String str);

  // Creacin

  public void Valor (Object obj, int sup)
  {
    valores.NuevoValor (obj, sup);
  }

  // Creacin automtica: Anlisis de los valores de un atributo

  public int AnalisisValores ()
  {
    return valores.Analisis();
  }

  // Dominios
  // --------

  // Consulta

  public int Dominios ()
  {
    return dominios.Total();
  }

  abstract public Lista DominiosValor (String str);
  abstract public int   DominioValor  (String str);

  // Creacin automtica

  public void DominioPorValor ()
  {
    dominios = new Dominios (this, valores);
  }

  // Dominio ---> String

  public String StringDominio (int i)
  {
    return dominios.toString (i);
  }

  // Salida estndar
  // ---------------

  abstract public void Print ();
  abstract public void FullPrint ();
}


