package fbg.util.db;

// Title:       TMiner
// Version:     1.2
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import fbg.tda.Lista;
import fbg.util.math.Decimal;

// CampoNUMBER
// -----------

// Versin 1.0: Ago'98 Modelizacin de campos de tipo numrico (NUMBER)
// Versin 1.1: May'99 fbg.util.math.Decimal
// Versin 1.2: Jun'99 Campo 1.1

public class CampoNUMBER extends Campo
{
  int size;
  int scale;

  // Constructor

  public CampoNUMBER (Tabla tabla, String nombre)
  {
    super (tabla, nombre);
    size  = -1;
    scale = -1;
  }

  public CampoNUMBER (Tabla tabla, String nombre, int total, int decimales)
  {
    super (tabla, nombre);
    size  = total;
    scale = decimales;
  }

  // Longitud

  public int Longitud ()
  {
    return size;
  }

  // Decimales

  public int Decimales ()
  {
    return scale;
  }

  // Valores
  // -------
  // Devuelve ID de un valor: [0..Valores]

  public int CodigoValor (Decimal num)
  {
    return valores.CodigoValor (num);
  }

  public int CodigoValor (String str)
  {
    return valores.CodigoValor (new Decimal(str));
  }

  // Dominios
  // --------

  // Creacin

  public void Dominio (Decimal min, Decimal max)
  {
    dominios.NuevoIntervalo (min,max);
  }

  // Consulta

  public Lista DominiosValor (Decimal num)
  {
    return dominios.DominiosValor (num);
  }

  public Lista DominiosValor (String str)
  {
    return dominios.DominiosValor (new Decimal(str));
  }

  public int DominioValor (Decimal num)
  {
    return dominios.DominioValor (num);
  }

  public int DominioValor (String str)
  {
    return dominios.DominioValor (new Decimal(str));
  }

  // Salida estndar

  public void Print ()
  {
    System.out.println (id+" (NUMBER "+size+":"+scale+")");
  }

  public void FullPrint ()
  {
    System.out.println (id+" (NUMBER "+size+":"+scale+") ["+Dominios()+" dominios]");

    dominios.Print();
  }

  // Discretizacin de atributos continuos
  // -------------------------------------

  // CLUSTERING
  // ----------

  public void Cluster ( byte algoritmo, int clusters)
  {
    int        i;
    Clustering agrupamiento = new Clustering(algoritmo,clusters,valores);

    if (agrupamiento!=null) {

       dominios = new Dominios(this);

       for (i=0; i<agrupamiento.clusters(); i++)
           dominios.NuevoIntervalo (agrupamiento.min(i), agrupamiento.max(i));
    }
  }

  // UMBRALIZACIN
  // -------------

  public void Threshold ( byte algoritmo )
  {
    Thresholding division = new Thresholding(algoritmo,valores);

    // Creacin de los dominios: [0,valor] - [valor+1,max]

    if (division.umbral>=0 && division.umbral<valores.Total()-1) {

       dominios = new Dominios(this);

       Decimal min = (Decimal) valores.Valor(0).valor;
       Decimal max = (Decimal) valores.Valor(valores.Total()-1).valor;
       Decimal th1 = (Decimal) valores.Valor(division.umbral).valor;
       Decimal th2 = (Decimal) valores.Valor(division.umbral+1).valor;

       dominios.NuevoIntervalo (min,th1);
       dominios.NuevoIntervalo (th2,max);
    }
  }

}

