package fbg.util.db;

// Title:       TMiner
// Version:     1.4
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import fbg.util.math.Decimal;

import java.util.Vector;

// DOMINIOS
// --------

// Versin 1.0: Ago'98 Clase contenedora
// Versin 1.1: Sep'98 Identificadores descriptivos
// Versin 1.2: May'99 Modelo: Lista -> Vector
// Versin 1.3: May'99 fbg.util.math.Decimal
// Versin 1.4: Jun'99 Interfaz: Lista -> Vector

public class Dominios
{
  Campo  campo;
  Vector dominios;
  Vector id;
  int    total;     // Dominios diferentes existentes

  // Constructor
  // -----------

  public Dominios (Campo c)
  {
    campo    = c;
    dominios = new Vector();
    id       = new Vector();
    total    = 0;
  }

  // Generacin automtica de dominios

  public Dominios (Campo c, Valores valores)
  {
    this (c);

    int   total = valores.Total();
    int   i;
    Valor v;

    for (i=0; i<total; i++) {

	  v = valores.Valor(i);

          if (campo instanceof CampoCHAR)
             NuevoValor ((String)(v.valor));
          else if (campo instanceof CampoNUMBER)
             NuevoValor ((Decimal)(v.valor));
    }
  }

  // Insercin de nuevos dominios
  // ----------------------------

  // CHAR

  public void NuevoValor (String str)
  {
    dominios.add(new DominioCHAR(total,str));
    id.add(null);
    total++;
  }

  public void NuevoDominio (String str[])
  {
    int i;

    for (i=0; i<str.length; i++)
        dominios.add(new DominioCHAR(total,str[i]));

    id.add(null);
    total++;
  }

  // NUMBER

  public void NuevoValor (Decimal num)
  {
    DominioNUMBER d = new DominioNUMBER ( total, num, num );

    dominios.add ( d );
    id.add (null);

    total ++;
  }

  public void NuevoIntervalo (Decimal min, Decimal max)
  {
    DominioNUMBER d = new DominioNUMBER ( total, min, max );

    dominios.add( d );
    id.add (null);

    total ++;
  }

  public void NuevoDominio (int n[])
  {
    int     i;
    Decimal d;

    for (i=0; i<n.length; i++) {
        d = (Decimal) campo.valores.ObjetoValor(n[i]);
        dominios.add ( new DominioNUMBER(total,d,d));
    }

    id.add(null);
    total++;
  }

  // Nuevos valores para dominios conocidos
  // --------------------------------------

  // CHAR

  public void AgregarValor (int i, String str)
  {
    if (i>=0 && i<total)
       dominios.add(new DominioCHAR(i,str));
  }

  public void AgregarValores (int i, String str[])
  {
    int j;

    if (i>=0 && i<total)
       for (j=0; j<str.length; j++)
           dominios.add(new DominioCHAR(i,str[j]));
  }

  // NUMBER

  public void AgregarValor (int i, Decimal num)
  {
    if (i>=0 && i<total)
       dominios.add(new DominioNUMBER(i,num,num));
  }

  public void AgregarIntervalo (int i, Decimal min, Decimal max)
  {
    if (i>=0 && i<total)
       dominios.add(new DominioNUMBER(i,min,max));
  }

  public void AgregarValores (int i, int v[])
  {
    int     j;
    Decimal n;

    if (i>=0 && i<total)
       for (j=0; j<v.length; j++) {
           n = (Decimal) campo.valores.ObjetoValor(v[j]);
           dominios.add(new DominioNUMBER(i,n,n));
       }
  }

  // Eliminar un dominio
  // -------------------

  public void Eliminar (int i)
  {
    int     pos;
    Dominio d;

    if (i>=0 && i<total) {

       pos = 0;

       while (pos<dominios.size()) {

             d = (Dominio) dominios.get(pos);

             if (d.id==i) {

                dominios.remove (pos);

             } else {

                if (d.id>i)
                   d.id --;

                pos++;
             }
       }

       id.remove(i);
       total--;
    }
  }

  // Identificadores descriptivos
  // ----------------------------

  public void setID (int n, String str)
  {
    id.set (n, str);
  }

  public String getID (int n)
  {
    return (String) id.get(n);
  }

  // Informacin
  // -----------

  // Nmero total de dominios

  public int Total ()
  {
    return total;
  }

  // Dominios a los que pertenece un valor dado

  // CHAR

  public Vector DominiosValor (String str)
  {
    int         pos;
    Vector      vector = new Vector();
    DominioCHAR d;

    if (str!=null) {

       for (pos=0; pos<dominios.size(); pos++) {

	   d = (DominioCHAR) dominios.get (pos);

	   if (str.equals(d.str))
	      vector.add ( new Integer(d.id) );
       }
    }

    return vector;
  }

  public int DominioValor (String str)
  {
    int         pos;
    DominioCHAR d;

    if (str!=null) {

       for (pos=0; pos<dominios.size(); pos++) {

	   d = (DominioCHAR) dominios.get (pos);

	   if (str.equals(d.str))
	      return d.id;
       }
    }

    return 0;
  }

  // NUMBER

  public Vector DominiosValor (Decimal num)
  {
    int           pos;
    Vector        vector = new Vector();
    DominioNUMBER d;

    if (num!=null) {

       for (pos=0; pos<dominios.size(); pos++) {
 
	   d = (DominioNUMBER) dominios.get (pos);

	   if (  num.compareTo(d.min)!=-1
              && num.compareTo(d.max)!= 1 )
              vector.add ( new Integer(d.id) );
       }
    }

    return vector;
  }

  public int DominioValor (Decimal num)
  {
    int           pos;
    DominioNUMBER d;

    if (num!=null) {

       for (pos=0; pos<dominios.size(); pos++) {

	   d = (DominioNUMBER) dominios.get (pos);

	   if (  num.compareTo(d.min)!=-1
              && num.compareTo(d.max)!= 1 )
	      return d.id;
       }
    }

    return 0;
  }

  // Dominio ---> String

  public String toFullString (int i)
  {
    int           pos;
    DominioCHAR   dc;
    DominioNUMBER dn;
    String        str = "{";

    if (campo instanceof CampoCHAR) {

       for (pos=0; pos<dominios.size(); pos++) {

           dc = (DominioCHAR) dominios.get (pos);

           if (dc.id==i)
              str += " '"+dc.str+"'";
       }

    } else if (campo instanceof CampoNUMBER) {

       for (pos=0; pos<dominios.size(); pos++) {

           dn = (DominioNUMBER) dominios.get (pos);

           if (dn.id==i)
              if (dn.min==dn.max)
                 str += " "+dn.min;
              else
                 str += " ["+dn.min+","+dn.max+"]";
       }
    }

    str += " }";

    return str;
  }

  public String toString (int i)
  {
    String descr;

    if (i>=0 && i<id.size()) {

       descr = (String) id.get(i);

       if (descr!=null)
          return descr;
       else
          return toFullString (i);

    } else {

       return null;
    }
  }

  public String[] toStringValores (int i)
  {
    int     n;
    int     pos;
    int     dim;
    String  array[];

    Dominio       d;
    DominioCHAR   dc;
    DominioNUMBER dn;

    // Recorrido inicial

    dim = 0;

    for (pos=0; pos<dominios.size(); pos++) {

        d = (Dominio) dominios.get (pos);

        if (d.id==i)
           dim++;
    }

    // Creacin del array

    array = null;

    if (dim>0) {

       array = new String[dim];
       n     = 0;

       if (campo instanceof CampoCHAR) {

          for (pos=0; pos<dominios.size(); pos++) {

              dc = (DominioCHAR) dominios.get (pos);

              if (dc.id==i) {
                 array[n] = dc.str;
                 n++;
              }
          }

       } else if (campo instanceof CampoNUMBER) {

          for (pos=0; pos<dominios.size(); pos++) {

              dn = (DominioNUMBER) dominios.get (pos);

              if (dn.id==i) {

                 if (dn.min==dn.max)
                    array[n] = ""+dn.min;
                 else
                    array[n] = "["+dn.min+","+dn.max+"]";

                 n++;
              }
          }
       }
    }

    // Resultado

    return array;
  }

  // Salida estndar
  // ---------------

  public void Print ()
  {
    int     pos;
    int     dim = dominios.size();
    Dominio d;

    for (pos=0; pos<dim; pos++) {
        d = (Dominio)dominios.get (pos);
	System.out.println(d.toString());
    }
  }

}