package fbg.util.db;

// Title:       TMiner
// Version:     1.5
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.sql.*;
import java.util.Vector;

import fbg.tda.Lista;
import fbg.util.math.Decimal;

// Tabla
// -----

// Versin 1.0: Ago'98 Modelizacin de tablas
// Versin 1.1: Feb'99 SCHEMAS
// Versin 1.2: May'99 Garbage Collector
// Versin 1.3: May'99 fbg.util.math.Decimal
// Versin 1.4: May'99 DatabaseMetaData bug
// Versin 1.5: Jun'99 JDBC: Object -> String


public class Tabla
{
  public String     id       = null;  // Identificador de la tabla
  public Conexion   conexion = null;  // Conexin al servidor
  public Connection acceso   = null;  // Conexin a la base de datos
  public int        tuplas   = 0;     // Nmero de tuplas de la tabla

  Campo  campos[]    = null; // Lista de campos "activos"
  Campo  definidos[] = null; // Campos disponibles

  // Constructor
  // -----------

  public Tabla (Connection connect, String nombre)
  {
    String schema;
    String table;
    int    pos = nombre.indexOf(".");
 
    if (pos!=-1) {
       schema = nombre.substring(0,pos);
       table  = nombre.substring(pos+1);
    } else {
       schema = null;
       table  = nombre;
    }

    id     = nombre;
    acceso = connect;

    try {

       // Campos de la tabla

       DatabaseMetaData metadata = acceso.getMetaData();

       // NOTAS: metadata.getCatalogs() ---> Not Supported (Oracle)
       //        metadata.getSchemas()  ---> Users (Oracle)
       //        metadata.getColumns()  ---> Oracle: Error al utilizar el parmetro SCHEMA

       ResultSet columnas = metadata.getColumns(null,null,table,"%");

       // MetaData

       String    owner    = Schema(columnas);

       // Informacin adicional
       // String    length   = Campo.Length(columnas);
       // String    size     = Campo.Size(columnas);
       // String    scale    = Campo.Scale(columnas);

       while (columnas.next()) {
        
          if ( schema==null || owner==null || schema.equals(columnas.getString(owner)) )
        
             switch (columnas.getShort("Data_Type")) {

               case java.sql.Types.CHAR:
               case java.sql.Types.VARCHAR:
               case java.sql.Types.LONGVARCHAR:
                    // CHAR
                    // NuevoCampo ( new CampoCHAR ( this,
                    //                              columnas.getString("Column_Name"),
                    //                              columnas.getInt(length) ) );
                    NuevoCampo ( new CampoCHAR ( this, columnas.getString("COLUMN_NAME") ) );
                    break;

               case java.sql.Types.NUMERIC:
               case java.sql.Types.DECIMAL:
               case java.sql.Types.REAL:
               case java.sql.Types.FLOAT:
               case java.sql.Types.DOUBLE:
               case java.sql.Types.BIT:
               case java.sql.Types.INTEGER:
               case java.sql.Types.TINYINT:
               case java.sql.Types.SMALLINT:
               case java.sql.Types.BIGINT:
                    // NUMBER
                    // NuevoCampo ( new CampoNUMBER ( this,
                    //                              columnas.getString("Column_Name"),
                    //                              columnas.getInt(size),
                    //                              columnas.getInt(scale) ) );
                    NuevoCampo ( new CampoNUMBER ( this, columnas.getString("COLUMN_NAME") ) );
                    break;

               case java.sql.Types.DATE:
               case java.sql.Types.TIME:
               case java.sql.Types.TIMESTAMP:
                    break;

               case java.sql.Types.BINARY:
               case java.sql.Types.VARBINARY:
               case java.sql.Types.LONGVARBINARY:
	            break;

	       case java.sql.Types.NULL:
                    break;
             }
       }

       // Campos definidos para la tabla

       definidos = new Campo[campos.length];

       int i;

       for (i=0; i<campos.length; i++)
           definidos[i] = campos[i];

       metadata = null;
       columnas = null;
       System.gc();

    } catch (Exception error) {

       if (schema!=null) {
          System.out.println(schema+"."+table+": "+error.getMessage());
       } else {
          System.out.println(table+": "+error.getMessage());
       }
  
    }

  }

  public Tabla (Conexion con, String nombre)
  {
    this(con.connect, nombre);

    conexion = con;
  }

  private void NuevoCampo (Campo c)
  {
    int   i;
    Campo aux[] = campos;

    if (aux==null) {

       campos = new Campo[1];

       campos[0] = c;

    } else {

       campos = new Campo[aux.length+1];

       for (i=0; i<aux.length; i++)
           campos[i] = aux[i];

       campos[aux.length] = c;
    }
  }


  // DataBase MetaData
  // -----------------

  public static String Schema (ResultSet metadata)
                       throws SQLException
  {
    ResultSetMetaData aux = metadata.getMetaData();

    int    total = aux.getColumnCount();
    int    i;
    String id;

    for (i=1; i<=total; i++) {
        id = aux.getColumnName(i);

        if (  id.equals("TABLE_OWNER")         // Oracle 
           || id.equals("TABLE_SCHEM") )       // JDBC
           return id;
    }

    return null;
  }

  public static String Type (ResultSet metadata)
                       throws SQLException
  {
    ResultSetMetaData aux = metadata.getMetaData();

    int    total = aux.getColumnCount();
    int    i;
    String id;

    for (i=1; i<=total; i++) {
        id = aux.getColumnName(i);

        if ( id.equals("TABLE_TYPE") ) // JDBC
           return id;
    }

    return null;
  }

  // Campos de la tabla
  // ------------------

  public void AddCampo (String n)
  {
    int   i;
    int   pos   = -1;
    Campo aux[] = campos;


    for (i=0; i<definidos.length; i++)
        if (n.equals(definidos[i].ID()))
           pos = i;

    if (pos!=-1) {

       campos = new Campo[aux.length+1];

       for (i=0; i<aux.length; i++)
           campos[i] = aux[i];

       campos[aux.length] = definidos[pos];
    }
  }

  public void DelCampo (String n)
  {
    int   i;
    int   pos = -1;
    Campo aux[] = campos;


    for (i=0; i<aux.length; i++)
        if (n.equals(aux[i].ID()))
           pos = i;

    if (pos!=-1) {

       campos = new Campo[aux.length-1];

       for (i=0; i<pos; i++)
           campos[i] = aux[i];

       for (i=pos+1; i<aux.length; i++)
           campos[i-1] = aux[i];
    }
  }

  // Valores de los atributos
  // ------------------------

  public void AnalisisValores ()
  {
    AnalisisValoresSerie();
  }

  // En serie

  public void AnalisisValoresSerie ()
  {
    int i;

    for (i=0; i<Columnas(); i++)
        if (Columna(i).Valores()==0)              // Anlisis an no realizado
           tuplas = Columna(i).AnalisisValores();
  }

  // En paralelo (todas las columnas simultneamente)

  public void AnalisisValoresParalelo ()
  {
    Vector    fields;
    Vector    valores;
    Campo     actual;
    Valor     v;
    String    fieldsID;
    boolean   found;
    int       i,j,dim;
    Statement SQL    = null;
    ResultSet val    = null;

    // Vector de columnas no analizadas an

    fields = new Vector();

    for (i=0; i<campos.length; i++)
        if (campos[i].Valores()==0)
           fields.add(campos[i]);

    dim = fields.size();

    if (dim>0) {

       actual   = (Campo) fields.elementAt(0);
       fieldsID = new String(actual.id);

       for (i=1; i<dim; i++) {
           actual = (Campo) fields.elementAt(i);
           fieldsID += ","+actual.id;         
       }

       try {

            SQL = acceso.createStatement();

            val = SQL.executeQuery ( "SELECT " + fieldsID + " FROM " + id );

            tuplas = 0;


            while (val.next()) {

                  tuplas++;
            
                  for (i=0; i<dim; i++) {

                      actual = (Campo) fields.elementAt(i);
            
                      if (actual instanceof CampoCHAR) {

                         String obj = val.getString(i+1);
                         String tmp;

                         if (obj!=null) {

                            valores = actual.valores.valores;
                            found   = false;

                            for (j=0; j<valores.size() && !found; j++) {
                                v = (Valor) valores.elementAt(j);

                                if ( obj.equals((String)(v.valor)) ) {
                                   v.support++;
                                   found = true;
                                }
                            }

                            if (!found)
                               valores.addElement ( new Valor(obj,1) );
                         }

                      } else if (actual instanceof CampoNUMBER) {

                         Decimal obj = new Decimal(val.getString(i+1));

                         if (obj!=null) {

                            valores = actual.valores.valores;
                            found   = false;

                            for (j=0; j<valores.size() && !found; j++) {
                                v = (Valor) valores.elementAt(j);

                                if ( obj.equals((Decimal)(v.valor)) ) {
                                   v.support++;
                                   found = true;
                                }
                            }

                            if (!found)
                               valores.addElement ( new Valor(obj,1) );
                          }
                      }
                  }
            }

            SQL = null;

        } catch (SQLException e) {

            while (e != null) {
                  System.err.println(e.getMessage());
                  e = e.getNextException();
            }
        }
    }

    fields = null;
  }

  // GENERACIN AUTOMTICA DE DOMINIOS
  // ---------------------------------
/*
  public void AtributoPorValor (int i)
  {
    Campo     c   = Columna(i);
    Statement SQL = null;
    ResultSet val = null;

    // c.Print();

    try {

	SQL = acceso.createStatement();

	val = SQL.executeQuery ( "SELECT DISTINCT " + c.id
	                       + " FROM " + this.id );

        if (c instanceof CampoCHAR) {

	   while (val.next()) {

		 String aux = val.getString(1);

		 if (aux!=null)
		    ((CampoCHAR)c).Dominio (aux);
	   }

        } else if (c instanceof CampoNUMBER) {

	   while (val.next()) {

		 Decimal aux = new Decimal(val.getString(1));

		 if (aux!=null)
	            ((CampoNUMBER)c).Dominio (aux,aux);
	   }

        }

	SQL = null;

    } catch (SQLException e) {
        while (e != null) {
              System.out.println(e.getMessage());
              e = e.getNextException();
        }
    }

  }

  // ---

  public void AtributosPorValor ()
  {
    int i;

    for (i=0; i<Columnas(); i++)
	AtributoPorValor(i);
  }
*/
  // ---

  public void DominioPorValor ()
  {
    int i;

    for (i=0; i<Columnas(); i++)
        Columna(i).DominioPorValor();
  }

  // Columnas
  // --------

  public int Columnas ()
  {
    if (campos!=null)
       return campos.length;
    else
       return 0;
  }


  public Campo Columna (int i)
  {
    return campos[i];
  }

  public Campo Columna (String s)
  {
    int   i;
    int   pos = -1;

    for (i=0; i<campos.length; i++)
        if (s.equals(campos[i].ID()))
           pos = i;

    if (pos!=-1)
       return campos[pos];
    else
       return null;
  }

  // Dominios a los que pertenece un valor
  // -------------------------------------

  public Lista Dominios (int atributo, String str)
  {
    return Columna(atributo).DominiosValor(str);
  }

  public int Dominio (int atributo, String str)
  {
    return Columna(atributo).DominioValor(str);
  }

  // Valores

  public int CodigoValor (int atributo, String valor)
  {
    return Columna(atributo).CodigoValor(valor);
  }

  // Salida estndar
  // ---------------

  public void Print ()
  {
    int   i;

    System.out.println ("Tabla "+id+" ("+campos.length+" columnas):");

    for (i=0; i<campos.length; i++) {
	System.out.print(i+" ");
        campos[i].Print();
    }
  }

  public void FullPrint ()
  {
    int   i;

    System.out.println ("Tabla "+id+" ("+campos.length+" columnas):");

    for (i=0; i<campos.length; i++) {
	System.out.print(i+" ");
        campos[i].FullPrint();
    }
  }

}

