package rf;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.MemoryImageSource;

import java.io.RandomAccessFile;
import java.io.IOException;

import rf.data.*;

/**
 * <p>
 * Image viewer
 *
 * @version 1.0
 * @author  Fernando Berzal Galiano
 * @see     rf.data.ImageBand
 */

public class ImageViewer
{
  // ------------------------
  // Lectura desde un fichero
  // ------------------------

  private Image getRawImage (String fileID, int width, int height)
  {
    ImageBand band = new ImageBand(fileID,width,height);

    return band.getRawImage();
  }

  // NOTA: Los bytes en Java se consideran codificados en complemento a 2

  private int raw (byte b)
  {
    if (b<0)
       return 256+b;
    else
       return b;
  }


  // Imagen RGB a partir de arrays de bytes

  private Image getRGBImage ( byte arrayR[],
                             byte arrayG[],
                             byte arrayB[],
                             int  width,
                             int  height)
  {
    int  r,g,b;
    int  x,y;
    int  pixels[] = new int[width * height];
    int  i;
    int  max,min,rango;

    max = 0;
    min = 255;

    for (i=0; i<arrayR.length; i++) {

        r = raw(arrayR[i]);

        if (r>max)
           max = r;
        else if (r<min)
           min = r;

        g = raw(arrayG[i]);

        if (g>max)
           max = g;
        else if (g<min)
           min = g;

        b = raw(arrayB[i]);

        if (b>max)
           max = b;
        else if (b<min)
           min = b;
    }

    i = 0;

    if (max>min) {

       rango = max-min+1;

       for (y=0; y<height; y++) {
           for (x=0; x<width; x++) {

               // Copia

               // r = raw(arrayR[i]);
               // g = raw(arrayG[i]);
               // b = raw(arrayB[i]);

               // Transformacin lineal

               r = (int) (((raw(arrayR[i])-min)*255.0)/rango);
               g = (int) (((raw(arrayG[i])-min)*255.0)/rango);
               b = (int) (((raw(arrayB[i])-min)*255.0)/rango);

               // Pixels: ALPHA+RED+GREEN+BLUE -> 0xAARRGGBB

               pixels[i] = (255<<24) | (r<<16) | (g<<8) | b;

               i++;
           }
       }
    }

    return Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(width, height, pixels, 0, width));
  }

  // TEST

  /**
   * Displays an image
   */

  public static void main (String args[])
  {
    /**/
    // Uso: Data IMAGEN (256x256)
    ImageBand  band = new ImageBand(args[0],256,256);
    Image      img  = band.getRawImage();
    /* *
    byte  bandaR[] = data.loadBand("C:/etsii/rf/data/galaxy/gal1",256,256);
    byte  bandaG[] = data.loadBand("C:/etsii/rf/data/galaxy/gal2",256,256);
    byte  bandaB[] = data.loadBand("C:/etsii/rf/data/galaxy/gal3",256,256);
    Image img      = data.getRGBImage(bandaR,bandaG,bandaB,256,256);
    /* *
    byte  bandaR[] = data.loadBand("C:/etsii/rf/data/igaliko/igal1",256,256);
    byte  bandaG[] = data.loadBand("C:/etsii/rf/data/igaliko/igal2",256,256);
    byte  bandaB[] = data.loadBand("C:/etsii/rf/data/igaliko/igal3",256,256);
    Image img      = data.getRGBImage(bandaR,bandaG,bandaB,256,256);
    /* */

    FrameImagen frame = new FrameImagen(img);
    frame.show();

  }
}


class ImageComponent extends Canvas
{
  Image img = null;

  public ImageComponent (Image imagen)
  {
    img = imagen;
  }

  public void paint (Graphics g)
  {
    if (img!=null) {

       Dimension d = getSize();

       // g.setXORMode(Color.white);
       // g.setXORMode(Color.black);
       // setBackground(Color.black);
       // g.setColor(Color.black);
       g.drawImage (img, (d.width-img.getWidth(null))/2, (d.height-img.getHeight(null))/2,this);

       // g.drawImage (img, x,y, anchura,altura, this);
       // g.drawImage (img, x,y, this);
    }
  }
}

// Visualizador de imgenes
// ------------------------

class FrameImagen extends Frame
{
  ImageComponent img = null;
  int            dim = 300;

  // Constructor

  public FrameImagen (Image image)
  {
    this(image,300);
  }

  public FrameImagen (Image image, int dim)
  {
    if (image!=null)
       img = new ImageComponent(image);

    try {
      this.setTitle("Imagen");
      this.setBackground(Color.lightGray);
      this.addWindowListener(new FrameImagen_this_windowAdapter(this));
      this.setSize(dim,dim);

      if (img!=null)
         this.add (img);

    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  // Cerrar ventana
  // --------------

  void this_windowClosing(WindowEvent e)
  {
    dispose();
    System.exit(0);
  }

  // Exit

  void menuItemExit_actionPerformed(ActionEvent e)
  {
    dispose();
    System.exit(0);
  }

}

// Control de eventos
// ------------------

class FrameImagen_this_windowAdapter extends java.awt.event.WindowAdapter {
  FrameImagen adaptee;

  FrameImagen_this_windowAdapter(FrameImagen adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }

}


