package rf;

// Dataset statistics

import fbg.util.math.Matrix;

import rf.data.*;

/**
 * <p>
 * Dataset statistics
 *
 * @version 1.0 Dec'98
 * @author  Fernando Berzal Galiano
 * @see     rf.data.Dataset
 */

public class Statistics
{
  // Variables

  Dataset dataset;

  int    N          = 0;
  int    noa        = 0;
  float  min[]      = null;
  float  max[]      = null;
  float  mean[]     = null;
  float  variance[] = null;

  Matrix covariance = null;         // Covariance matrix

  // Constructor
  // -----------

  public Statistics (Dataset d)
  {
    dataset = d;
    noa     = dataset.attributes();
    N       = dataset.instances();
  }

  // Attribute statistics
  // --------------------

  public int size ()
  {
    return N;
  }

  public float min (int n)
  {
    int   i;
    int   j;
    float v;
    float minimum;

    if (min==null) {

       min = new float[noa];

       for (i=0; i<noa; i++) {

           minimum = dataset.value(0,i);

           for (j=1; j<N; j++) {

               v = dataset.value(j,i);

               if (v<minimum)
                  minimum = v;
           }

           min[i] = minimum;
       }

    }

    return min[n];
  }

  public float max (int n)
  {
    int   i;
    int   j;
    float v;
    float maximum;

    if (max==null) {

       max = new float[noa];

       for (i=0; i<noa; i++) {

           maximum = dataset.value(0,i);

           for (j=1; j<N; j++) {

               v = dataset.value(j,i);

               if (v>maximum)
                  maximum = v;
           }

           max[i] = maximum;
       }

    }

    return max[n];
  }

  public float mean (int k)
  {
    int   i;
    int   j;
    float sum;

    if (mean==null) {

       mean = new float[noa];

       for (i=0; i<noa; i++) {

           sum = 0;

           for (j=0; j<N; j++)
               sum += dataset.value(j,i);

           mean[i] = sum / N;
       }
    }

    return mean[k];
  }

  public float variance (int k)
  {
    int   i;
    int   j;
    float sum;
    float dif;

    if (variance==null) {

       variance = new float[noa];

       for (i=0; i<noa; i++) {

           sum = 0;

           for (j=0; j<N; j++) {
               dif  = dataset.value(j,i)-mean(i);
               sum += dif*dif;
           }

           variance[i] = sum / N;
       }
    }

    return variance[k];
  }

  // Mean vector
  // -----------

  public Matrix mean ()
  {
    Matrix m = new Matrix (noa,1);
    int    i;

    for (i=0; i<noa; i++)
        m.setCell (i,0, (double) mean(i));

    return m;
  }

  // Covariance matrix
  // -----------------

  public Matrix covariance ()
  {
    if (covariance==null) {

       covariance = new Matrix (noa,noa);

       float s[][] = new float[noa][noa];
       int   i,j,k;

       for (k=0; k<N; k++)
           for (i=0; i<noa; i++)
               for (j=0; j<noa; j++)
                   s[i][j] += (dataset.value(k,i)-mean(i))*(dataset.value(k,j)-mean(j));

       for (i=0; i<noa; i++)
           for (j=0; j<noa; j++)
               covariance.setCell (i,j, s[i][j]/N );
    }

    return covariance;
  }

  public float covariance (int i, int j)
  {
    return (float) covariance().getCell(i,j);
  }

  // TEST
  // ----

  public static void main (String args[])
  {
    DataConfiguration cfg = new DataConfiguration("c:\\etsii\\rf\\data\\igaliko\\igaliko.cfg");
    // DataConfiguration cfg = new DataConfiguration("c:\\etsii\\rf\\data\\galaxy\\galaxy.cfg");
    // DataConfiguration cfg = new DataConfiguration("c:\\etsii\\rf\\data\\bupa\\bupa.cfg");
    // DataConfiguration cfg = new DataConfiguration("c:\\etsii\\rf\\data\\iono\\iono.cfg");
    // DataConfiguration cfg = new DataConfiguration("c:\\etsii\\rf\\data\\jdbc.cfg");

    // DataConfiguration cfg = new DataConfiguration(args[0]);

    Data       data  = cfg.getData();

    System.out.println(data.instances());

    Dataset    lrn   = data.patterns();
    Statistics stats = new Statistics(lrn);

    int i;

    for (i=0; i<lrn.attributes(); i++)
        System.out.println( " MIN="+stats.min(i)
                          + " MAX="+stats.max(i)
                          + " MEAN="+stats.mean(i)
                          + " VAR="+stats.variance(i) );

  }

}

