package rf.classification;

// Clasificador
// ------------

import java.io.PrintStream;

import rf.*;
import rf.data.*;


public abstract class Classifier
{
  public DataConfiguration cfg;  // Configuracin de los datos

  public int               noc;  // Nmero de clases
  public int               noa;  // Nmero de atributos

  // Constructor

  public Classifier (DataConfiguration config)
  {
    cfg  = config;
    noc  = cfg.totalClasses();
    noa  = cfg.totalAttributes();
  }

  // Clasificacin de un patrn

  public abstract int classify (Pattern p);


  // Evaluacin del clasificador
  // ---------------------------
  // Devuelve la tabla de contingencias
  // derivada de la aplicacin del clasificador a 'set'

  public int[][] evaluation (Dataset set)
  {
    int     N   = set.instances();
    String  classes[] = cfg.getClasses();
    String  label;
    int     k,c;
    int     classification;
    int     tabla[][] = new int[noc][noc];

    for (k=0; k<N; k++) {

        label = set.label(k);

        for (c=0; c<noc && !label.equals(classes[c]); c++)
            ;

        if (c<noc) {
           classification = classify(set.pattern(k));

           if (classification>=0 && classification<noc)
              tabla[c][classification]++;
        }
    }

    return tabla;
  }

  // Salida estndar (System.err por defecto)

  public void printContingencyTable (int test[][])
  {
    printContingencyTable ( System.err, test );
  }

  public void printContingencyTable (PrintStream out, int test[][])
  {
    int   i,j;
    int   row;
    float pct;
    int   total;
    int   ok;

    total = 0;

    for (i=0; i<noc; i++) {

        // Instancias de la clase i

        row = 0;

        for (j=0; j<noc; j++) {

            out.print("\t"+test[i][j]);

            row += test[i][j];
        }

        total += row;

        out.println("\t"+pct((float)test[i][i]/(float)row));
    }

    ok = 0;

    for (i=0; i<noc; i++)
        ok += test[i][i];

    out.println();
    out.println( "Bondad estimada del clasificador: "
               + ok+"/"+total + " ("+pct((float)ok/(float)total)+")\n");
  }

  private String pct (float f)
  {
    float p = 100*f;
    int   n = (int) p;
    int   d = (int) ((p-n)*10);
    int   c = (int) (((p-n)*100)%10);

    if (n<10)
       return "  "+n+"."+d+c+"%";
    else if (n<100)
       return " "+n+"."+d+c+"%";
    else
       return n+"."+d+c+"%";
  }
}

