package rf.classification.nn;

// Clasificador del vecino ms prximo
// -----------------------------------

import rf.classification.Classifier;

import rf.*;
import rf.data.*;


public class NNClassifier extends Classifier
{
  Dataset set;          // Conjunto de datos
  int     N;            // Nmero de casos en 'set'
  String  classes[];    // Clases permitidas

  // Constructor

  public NNClassifier ( DataConfiguration config,
                        Dataset           data )
  {
    super (config);

    set     = data;
    N       = set.instances();
    classes = config.getClasses();
  }

  // Distancia entre patrones

  private float distance (Pattern p, Pattern q)
  {
    float diff;
    float d = 0;
    int   i;

    for (i=0; i<noa; i++) {
        diff  = p.attribute(i) - q.attribute(i);
        d    += diff*diff;
    }

    return d;
  }

  // Clasificador

  public int classify (Pattern p)
  {
    int     i,j,c;
    float   mindist;
    float   dist;
    String  label = null;
    Pattern q     = null;

    // Vecino ms cercano

    if (N>0) {
       q = set.pattern(0);
       mindist = distance(p,q);
       label   = q.label();

       for (i=1; i<N; i++) {
           q    = set.pattern(i);
           dist = distance(p,q);

           if (dist<mindist) {
              mindist = dist;
              label = q.label();
           }
       }
    }

    // Cdigo de la clase

    if (label!=null) {

       for (c=0; c<noc && !label.equals(classes[c]); c++)
           ;

    } else {

       c = noc;
    }

    return c;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration c;
    Data              d;
    NNClassifier      nn;

    if (args.length<1 || args.length>2) {

       System.out.println ("Clasificador 1-NN");
       System.out.println ();
       System.out.println ("Sintaxis: NNClassifier <fichero_cfg>");

    } else {

       c = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( c.getDescriptionString() );

       d = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();

       System.out.println ("Clasificador 1-NN");
       System.out.println ();

       nn = new NNClassifier (c, d.learningSet() );

       nn.printContingencyTable ( nn.evaluation(d.testSet()) );
    }
  }
}


