package rf.classification.nn;

// Clasificador k-NN (k vecinos ms prximos)
// ------------------------------------------

import rf.classification.Classifier;

import rf.*;
import rf.data.*;
import rf.edit.*;

import fbg.util.Benchmark;


public class kNNClassifier extends Classifier
{
  Dataset set;          // Conjunto de datos

  int     N;            // Nmero de casos en 'set'
  int     K;            // Clasificador K-NN
  String  classes[];    // Clases permitidas

  // Constructor

  public kNNClassifier ( DataConfiguration config,
                         Dataset           data,
                         int               k )
  {
    super (config);

    set     = data;
    N       = set.instances();
    K       = k;
    classes = cfg.getClasses();

    if (K>N)
       K = N;
  }

  // Distancia entre patrones

  private float distance (Pattern p, Pattern q)
  {
    float diff;
    float d = 0;
    int   i;

    for (i=0; i<noa; i++) {
        diff  = p.attribute(i) - q.attribute(i);
        d    += diff*diff;
    }

    return d;
  }

  // Clasificador

  public int classify (Pattern p)
  {
    int     i,j,c;
    float   dist[]  = new float[K];
    String  label[] = new String[K];
    float   dtmp;
    String  ltmp;
    Pattern q;
    int     freq[] = new int[noc];
    int     max;

    // Vecinos ms cercanos

    for (i=0; i<K; i++) {
        q = set.pattern(i);
        dist[i]  = distance(p,q);
        label[i] = q.label();

        for (j=i-1; j>=0 && dist[j]>dist[j+1]; j--) {
            dtmp = dist[j+1];
            ltmp = label[j+1];
            dist[j+1] = dist[j];
            label[j+1] = label[j];
            dist[j] = dtmp;
            label[j] = ltmp;
        }
    }

    for (i=K; i<N; i++) {
        q    = set.pattern(i);
        dtmp = distance(p,q);

        if (dist[K-1]>dtmp) {
           dist[K-1]  = dtmp;
           label[K-1] = q.label();

           for (j=K-2; j>=0 && dist[j]>dist[j+1]; j--) {
               dtmp = dist[j+1];
               ltmp = label[j+1];
               dist[j+1] = dist[j];
               label[j+1] = label[j];
               dist[j] = dtmp;
               label[j] = ltmp;
           }
        }
    }

    // Votacin

    for (i=0; i<K; i++) {

        ltmp = label[i];

        for (c=0; c<noc && !ltmp.equals(classes[c]); c++)
            ;

        if (c<noc)
           freq[c]++;
    }

    max = 0;

    for (c=1; c<noc; c++)
        if (freq[c]>freq[max])
           max = c;

    return max;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration c;
    Dataset           set;
    Data              d;
    kNNClassifier     knn;
    int               k;
    int               m = 2;
    int               I = 1;
    String            fichero     = null;
    boolean           doMultiEdit = false;
    boolean           doHart      = false;
    boolean           doReadAscii = false;
    Benchmark         crono;
    MultiEdit         edit;
    Hart              condense;
    int               actual;


    if (args.length<2) {

       System.out.println ("Clasificador k-NN");
       System.out.println ();
       System.out.println ("Sintaxis: kNNClassifier <fichero_CFG> <k>");
       System.out.println ("                        [dataset <fichero_DAT>]");
       System.out.println ("                        [multiedit <m> <I>]");
       System.out.println ("                        [Hart]");


    } else {

       c = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( c.getDescriptionString() );

       d = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();

       k = (new Integer(args[1])).intValue();

       if (k<1)
          k = 1;

       System.out.println ("Clasificador "+k+"-NN");
       System.out.println ();


       actual = 2;

       while (actual<args.length) {

          if (  args[actual].charAt(0) == 'd'
             || args[actual].charAt(0) == 'D' ) {

             doReadAscii = true;
             fichero = args[actual+1];

             actual += 2;

          } else if (  args[actual].charAt(0) == 'm'
                    || args[actual].charAt(0) == 'M' ) {

             doMultiEdit = true;

             m = (new Integer(args[actual+1])).intValue();
             I = (new Integer(args[actual+2])).intValue();

             if (m<2)
                m = 2;

             if (I<1)
                I = 1;

             actual += 3;

          } else if (  args[actual].charAt(0) == 'h'
                    || args[actual].charAt(0) == 'H' ) {

             doHart = true;
             actual ++;

          } else {

             System.out.println("Error en los parmetros");
             actual = args.length;

          }
       }


       // Construccin del clasificador

       crono = new Benchmark();

       crono.start();

       if (doReadAscii)
          set = AsciiData.load(d,fichero);
       else
          set = d.learningSet();

       System.out.println(set.instances()+" patrones");

       if (doMultiEdit) {

          edit = new MultiEdit(c,set,m,I);
          set  = edit.getResult();

          System.out.println ( "MULTIEDIT (m="+m+";I="+I+"): "
                             + edit.getOriginal().instances()
                             + " ---> "
                             + edit.getResult().instances() );
       }

       if (doHart) {

          condense = new Hart(c,set);
          set      = condense.getResult();

          System.out.println ( "CONDENSADO DE HART: "
                             + condense.getOriginal().instances()
                             + " ---> "
                             + condense.getResult().instances() );
       }

       knn = new kNNClassifier (c, set, k );

       crono.stop();

       System.out.println ("\nConstruccin del clasificador: "+crono+"\n");

       // Clasificacin del conjunto de datos

       crono = new Benchmark();

       crono.start();

       int test[][] = knn.evaluation(d.testSet());

       crono.stop();

       System.out.println ("Clasificacin de los datos: "+crono+"\n");

       knn.printContingencyTable (System.out, test);
    }
  }
}


