package rf.classification.parametric;

// Clasificador paramtrico lineal: Distancia eucldea
// ---------------------------------------------------
//  Variables estadsticamente independientes con la misma varianza


import rf.*;
import rf.data.*;

import rf.classification.Classifier;


public class EuclideanClassifier extends Classifier
{
  boolean useAprioriProbabilities;

  float   mean[][];     // Media de un atributo para los elementos de una clase
  float   prob[];       // Probabilidad de cada clase
  float   variance;     // Varianza global

  // Constructor

  public EuclideanClassifier ( DataConfiguration config,
                               Dataset           data )
  {
    this (config,data,false);
  }

  public EuclideanClassifier ( DataConfiguration config,
                               Dataset           set,
                               boolean           apriori )
  {
    super (config);

    useAprioriProbabilities = apriori;

    prob = new float[noc];
    mean = new float[noa][noc];

    int     N         = set.instances();
    float   freq[]    = new float[noc];
    float   avg[]     = new float[noa];
    String  classes[] = cfg.getClasses();
    String  label;
    int     i,k,c;
    float   val,dif;

    // Medias de cada variable para cada clase

    for (i=0; i<noc; i++)
        freq[i] = 0;

    for (k=0; k<N; k++) {

        label = set.label (k);

        for (c=0; c<noc && !label.equals(classes[c]); c++)
            ;

        if (c<noc) {

           freq[c]++;

           for (i=0; i<noa; i++) {
               val = set.value(k,i);
               mean[i][c] += val;
               avg[i] += val;
           }
        }
    }

    for (c=0; c<noc; c++) {

        if (freq[c]>0) {
           for (i=0; i<noa; i++)
               mean[i][c] /= freq[c];
        }
    }

    for (i=0; i<noa; i++)
        avg[i] /= N;

    for (c=0; c<noc; c++)
        prob[c] = freq[c] / N;

    // Varianza

    if (useAprioriProbabilities) {

       variance = 0;

       for (k=0; k<N; k++) {
           for (i=0; i<noa; i++) {
               dif = set.value(k,i)-avg[i];
               variance += dif*dif;
           }
       }

       variance /= (noa*N);
    }

    // Standard output
    // printStats();
  }

  private void printStats()
  {
    int i,c;

    for (c=0; c<noc; c++) {

        for (i=0; i<noa; i++)
            System.out.print("\t"+mean[i][c]);

        System.out.println();
    }
  }

  // Funcin discriminante

  private float discriminant (int c, Pattern p)
  {
    float diff;
    float g = 0;
    int   i;

    if (prob[c]>0) {

       for (i=0; i<noa; i++) {
           diff  = p.attribute(i) - mean[i][c];
           g    += diff*diff;
       }

       if (useAprioriProbabilities)
          g  = g/(2*variance) - (float) Math.log((double)prob[c]);

    } else {

       g = 1e30f;
    }

    return g;
  }

  // Clasificador

  public int classify (Pattern p)
  {
    float min   = discriminant (0,p);
    float g;
    int   label = 0;
    int   i;

    for (i=1; i<noc; i++) {

        g = discriminant (i,p);

        if (g<min) {
           min   = g;
           label = i;
        }
    }

    return label;
  }

  // TEST

  public static void main (String args[])
  {
    if (args.length<1 || args.length>2) {

       System.out.println ("Clasificador lineal tipo 1");
       System.out.println ();
       System.out.println ("Sintaxis: EuclideanClassifier <fichero_cfg> [apriori]");

    } else {

       DataConfiguration   c  = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( c.getDescriptionString() );

       Data                d  = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();

       EuclideanClassifier ec = new EuclideanClassifier (c, d.learningSet(), (args.length==2) );

       ec.printContingencyTable ( ec.evaluation(d.testSet()) );
    }
  }

}


