package rf.classification.parametric;

// Clasificador paramtrico lineal: Distancia eucldea normalizada

import rf.classification.Classifier;

import rf.*;
import rf.data.*;


public class NormalizedEuclideanClassifier extends Classifier
{
  boolean useAprioriProbabilities;

  float   mean[][];     // Media de un atributo para los elementos de una clase
  float   prob[];       // Probabilidad de cada clase
  float   range[];      // Rango de cada variable: max-min
  float   variance;     // Varianza global

  // Constructor

  public NormalizedEuclideanClassifier ( DataConfiguration config,
                                         Dataset           data )
  {
    this (config,data,false);
  }

  public NormalizedEuclideanClassifier ( DataConfiguration config,
                                         Dataset           set,
                                         boolean           apriori )
  {
    super (config);

    useAprioriProbabilities = apriori;

    prob  = new float[noc];
    mean  = new float[noa][noc];
    range = new float[noa];

    int     N         = set.instances();
    float   freq[]    = new float[noc];
    float   avg[]     = new float[noa];
    float   min[]     = new float[noa];
    float   max[]     = new float[noa];
    String  classes[] = cfg.getClasses();
    String  label;
    int     i,k,c;
    float   val,dif;

    // Medias de cada variable para cada clase

    for (i=0; i<noc; i++)
        freq[i] = 0;

    for (k=0; k<N; k++) {

        label = set.label (k);

        for (c=0; c<noc && !label.equals(classes[c]); c++)
            ;

        if (c<noc) {

           freq[c]++;

           for (i=0; i<noa; i++) {
               val = set.value(k,i);
               mean[i][c] += val;
               avg[i] += val;
           }
        }
    }

    for (c=0; c<noc; c++) {

        if (freq[c]>0) {
           for (i=0; i<noa; i++)
               mean[i][c] /= freq[c];
        }
    }

    for (i=0; i<noa; i++)
        avg[i] /= N;

    for (c=0; c<noc; c++)
        prob[c] = freq[c] / N;

    // Mnimos y mximos de cada variable

    for (i=0; i<noa; i++) {
        min[i] = set.value(0,i);
        max[i] = set.value(0,i);
    }

    for (k=1; k<N; k++) {
        for (i=0; i<noa; i++) {

            val = set.value(k,i);

            if (val>max[i])
               max[i] = val;
            else if (val<min[i])
               min[i] = val;
        }
    }

    for (i=0; i<noa; i++)
        range[i] = max[i] - min[i];

    // Varianza

    if (useAprioriProbabilities) {

       variance = 0;

       for (k=0; k<N; k++) {
           for (i=0; i<noa; i++) {
               if (range[i]>0) {
                  dif = (set.value(k,i)-avg[i])/range[i];
                  variance += dif*dif;
               }
           }
       }

       variance /= (noa*N);
    }

    // Standard output
    // printStats();
  }

  private void printStats()
  {
    int i,c;

    for (c=0; c<noc; c++) {

        for (i=0; i<noa; i++)
            System.out.print("\t"+mean[i][c]);

        System.out.println();
    }
  }

  // Funcin discriminante

  private float discriminant (int c, Pattern p)
  {
    float diff;
    float g = 0;
    int   i;

    if (prob[c]>0) {

       for (i=0; i<noa; i++) {
           if (range[i]>0) {
              diff  = ( p.attribute(i) - mean[i][c] ) / range[i];
              g    += diff*diff;
           }
       }

       if (useAprioriProbabilities)
          g  = g/(2*variance) - (float) Math.log((double)prob[c]);

    } else {

       g = 1e30f;
    }

    return g;
  }

  // Clasificador

  public int classify (Pattern p)
  {
    float min   = discriminant (0,p);
    float g;
    int   label = 0;
    int   i;

    for (i=1; i<noc; i++) {

        g = discriminant (i,p);

        if (g<min) {
           min   = g;
           label = i;
        }
    }

    return label;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration             c;
    Data                          d;
    NormalizedEuclideanClassifier ec;

    if (args.length<1 || args.length>2) {

       System.out.println ("Clasificador lineal (distancia euclidea normalizada)");
       System.out.println ();
       System.out.println ("Sintaxis: NormalizedEuclideanClassifier <fichero_cfg> [apriori]");

    } else {

       c = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( c.getDescriptionString() );

       d = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();

       ec = new NormalizedEuclideanClassifier (c, d.learningSet(), (args.length==2) );

       ec.printContingencyTable ( ec.evaluation(d.testSet()) );
    }
  }
}


