package rf.classification.parametric;

// Clasificador paramtrico: Mtodo de los paraleleppedos

import rf.classification.Classifier;

import rf.*;
import rf.data.*;


public class ParallelepipedClassifier extends Classifier
{
  int   freq[];       // Frecuencia de cada clase
  float mean[][];     // Media de un atributo para los elementos de una clase
  float dev[];        // Desviacin tpica de cada atributo

  // Constructor

  public ParallelepipedClassifier ( DataConfiguration config,
                                    Dataset           set )
  {
    super (config);

    freq = new int[noc];
    mean = new float[noa][noc];
    dev  = new float[noa];

    int     N         = set.instances();
    float   avg[]     = new float[noa];
    String  classes[] = cfg.getClasses();
    String  label;
    int     i,k,c;
    float   val;

    // Medias de cada variable para cada clase

    for (i=0; i<noc; i++)
        freq[i] = 0;

    for (k=0; k<N; k++) {

        label = set.label (k);

        for (c=0; c<noc && !label.equals(classes[c]); c++)
            ;

        if (c<noc) {

           freq[c]++;

           for (i=0; i<noa; i++) {
               val = set.value(k,i);
               mean[i][c] += val;
               avg[i] += val;
           }
        }
    }

    for (c=0; c<noc; c++) {

        if (freq[c]>0) {
           for (i=0; i<noa; i++)
               mean[i][c] /= freq[c];
        }
    }

    for (i=0; i<noa; i++)
        avg[i] /= N;

    // Desviacin tpica de cada atributo

    for (i=0; i<noa; i++) {

       dev[i] = 0;

       for (k=0; k<N; k++)
           dev[i] += Math.abs(set.value(k,i)-avg[i]);

       dev[i] /= N;
    }

    // Standard output
    // printStats();
  }

  private void printStats()
  {
    int i,c;

    for (c=0; c<noc; c++) {

        for (i=0; i<noa; i++)
            System.out.print("\t"+mean[i][c]);

        System.out.println();
    }
  }

  // Funcin discriminante

  private float discriminant (int c, Pattern p)
  {
    float g = 0;
    float k;
    int   i;

    if (freq[c]>0) {

       // MAX
/*
       g = Math.abs ((p.attribute(0)-mean[0][c])/dev[0]);

       for (i=1; i<noa; i++) {

           k = Math.abs ((p.attribute(i)-mean[i][c])/dev[i]);

           if (k>g)
              g = k;
       }
/**/
       // NORM
/**/
       g = 0;

       for (i=0; i<noa; i++) {
           k  = (p.attribute(i)-mean[i][c])/dev[i];
           g += k*k;
       }
/**/
    } else {

       g = 1e30f;
    }

    return g;
  }

  // Clasificador

  public int classify (Pattern p)
  {
    float min   = discriminant (0,p);
    float g;
    int   label = 0;
    int   i;

    for (i=1; i<noc; i++) {

        g = discriminant (i,p);

        if (g<min) {
           min   = g;
           label = i;
        }
    }

    return label;
  }


  // TEST

  public static void main (String args[])
  {
    DataConfiguration        c;
    Data                     d;
    ParallelepipedClassifier pc;

    if (args.length<1 || args.length>2) {

       System.out.println ("METODO DE LOS PARALELEPIPEDOS");
       System.out.println ();
       System.out.println ("Sintaxis: ParallelepipedClassifier <fichero_cfg>");

    } else {

       c = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( c.getDescriptionString() );

       d  = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();

       pc = new ParallelepipedClassifier (c, d.learningSet() );

       pc.printContingencyTable ( pc.evaluation(d.testSet()) );
    }
  }
}


