package rf.classification.parametric;

// Clasificador paramtrico cuadrtico
// -----------------------------------
//  Matrices de covarianza arbitrarias

import rf.classification.Classifier;

import rf.*;
import rf.data.*;

import fbg.util.math.Matrix;


public class QuadraticClassifier extends Classifier
{
  float  logp[];       // Logaritmo de las probabilidades de cada clase
  float  mean[][];     // Media de un atributo para los elementos de una clase
  Matrix inv[];        // Inversas de la matrices de covarianza
  float  logdet[];     // Logaritmo de los determinantes de las matrices de covarianza

  // Constructor

  public QuadraticClassifier ( DataConfiguration config,
                               Dataset           set )
  {
    super (config);

    logp   = new float[noc];
    mean   = new float[noa][noc];
    inv    = new Matrix[noc];
    logdet = new float[noc];

    Data         dat       = cfg.getData();
    String       classes[] = cfg.getClasses();
    TableDataset subset[]  = new TableDataset[noc];
    Statistics   stats[]   = new Statistics[noc];
    Matrix       cov[]     = new Matrix[noc];
    String       label;
    int          N         = set.instances();
    int          i,k,c;

    // Anlisis del conjunto de datos

    for (c=0; c<noc; c++)
        subset[c] = new TableDataset(dat);

    for (k=0; k<N; k++) {

        label = set.label (k);

        for (c=0; c<noc && !label.equals(classes[c]); c++)
            ;

        if (c<noc)
           subset[c].add ( set.pattern(k) );
    }

    for (c=0; c<noc; c++)
        stats[c] = new Statistics(subset[c]);

    // Logaritmos de las probabilidades de cada clase

    for (c=0; c<noc; c++)
        logp[c] = (float) Math.log ( (double) subset[c].instances() / (double) N );

    // Medias de cada atributo para cada clase

    for (i=0; i<noa; i++)
        for (c=0; c<noc; c++)
            mean[i][c] = stats[c].mean(i);

    // Matrices de covarianza

    for (c=0; c<noc; c++) {

        cov[c] = stats[c].covariance();

        inv[c] = cov[c].inverse();

        if (inv[c]==null)
           throw ( new ArithmeticException ("Singular covariance matrix (class "+classes[c]+")") );

        logdet[c] = (float) Math.log ( Math.abs ( cov[c].det() ) );
    }
  }

  // Funcin discriminante

  private float discriminant (int c, Pattern x)
  {
    Matrix diff;
    Matrix difft;
    Matrix mahalanobis;
    float  dist;
    int    i;

    diff = new Matrix (noa, 1);

    for (i=0; i<noa; i++)
        diff.setCell (i, 0, x.attribute(i) - mean[i][c]);

    difft = diff.transpose();

    mahalanobis = difft.mult ( inv[c].mult ( diff ) );

    dist = (float) mahalanobis.getCell(0,0);

    return (dist+logdet[c])/2 - logp[c];
  }

  // Clasificador

  public int classify (Pattern p)
  {
    float min   = discriminant (0,p);
    float g;
    int   label = 0;
    int   i;

    for (i=1; i<noc; i++) {

        g = discriminant (i,p);

        if (g<min) {
           min   = g;
           label = i;
        }
    }

    return label;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   c  = null;
    Data                d  = null;
    QuadraticClassifier qc = null;

    if (args.length!=1) {

       System.out.println ("Clasificador tipo 3");
       System.out.println ();
       System.out.println ("Sintaxis: QuadraticClassifier <fichero_CFG>");

    } else {

       c  = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( c.getDescriptionString() );

       d = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();

       try {

            qc = new QuadraticClassifier ( c, d.learningSet() );

       }catch (ArithmeticException e) {

            System.err.println("Matriz de covarianza singular");
       }

       if (qc!=null)
          qc.printContingencyTable ( qc.evaluation(d.testSet()) );
    }
  }
}


