package rf.clustering;

// Mtodo adaptativo de agrupamiento

import rf.*;
import rf.data.*;
import rf.distance.*;

/**
 * Clustering with an unknown number of classes:
 * Adaptive sample set construction (heuristic method)
 */

public class AdaptiveClustering extends ClusteringAlgorithm
{
  float radio;

  // Constructor

  public AdaptiveClustering (DataConfiguration cfg, Dataset set,
                             Distance dist, float tau, float theta)
  {
    super (cfg, set, dist);

    radio = tau;

    int     i,k;
    int     A;
    int     N = set.instances();
    Pattern p;
    Cluster c;
    Cluster asignacion[] = new Cluster[N];
    boolean cambios;
    float   d,mindist;

    c = addCluster();
    c.assign ( set.pattern(0) );
    asignacion[0] = c;
    A = 1;


    do {

        cambios = false;

        for (i=0; i<N; i++) {

            p = set.pattern(i);

            c       = cluster(0);
            mindist = metrics.distance(c,p);

            for (k=1; k<A; k++) {
                d = metrics.distance(cluster(k),p);

                if (d<mindist) {
                   c       = cluster(k);
                   mindist = d;
                }
            }

            if (mindist<=theta*tau) {

               if (asignacion[i]!=c) {
                  c.assign(p);

                  if (asignacion[i]!=null)
                     asignacion[i].deassign (p);

                  asignacion[i] = c;
                  cambios = true;
               }

            } else if (mindist>tau) {

               c = addCluster();

               c.assign (p);

               if (asignacion[i]!=null)
                  asignacion[i].deassign(p);

               asignacion[i] = c;
               cambios = true;

               A++;
            }
        }

    } while (cambios);
  }

  // Clasificacin

  public Cluster cluster (Pattern p)
  {
    Cluster cluster = super.cluster(p);

    if (metrics.distance(cluster,p)>radio)
       cluster = null;

    return cluster;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   cfg;
    Data                dat;
    Dataset             set;
    Distance            dist;
    float               tau,theta;
    ClusteringAlgorithm clustering;


    if (args.length!=3) {

       System.out.println ("Sintaxis: AdaptiveClustering <fichero_cfg> <tau> <theta>");

    } else {

       cfg   = new DataConfiguration (args[0]);
       tau   = (new Float(args[1])).floatValue();
       theta = (new Float(args[2])).floatValue();

       if (cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       if (tau<=0)
          tau = 1;

       if (theta<0 || theta>1)
          theta = 1;

       System.out.print ( cfg.getDescriptionString() );

       dat = cfg.getData();

       System.out.println ();
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+dat.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+dat.testInstances()+" patrones");
       System.out.println ();

       set  = dat.patterns();
       dist = new EuclideanDistance(set);

       System.out.println ("Agrupamiento adaptativo\n");

       clustering = new AdaptiveClustering (cfg,set,dist,tau,theta);

       clustering.printReport();
    }
  }
}