package rf.clustering;

// Algoritmo de agrupamiento

import java.util.Vector;

import rf.DataConfiguration;
import rf.data.Dataset;
import rf.data.Pattern;
import rf.distance.Distance;
import rf.classification.Classifier;


public abstract class ClusteringAlgorithm extends Classifier
{
  public Dataset  data;        // Conjunto de datos sobre los que se trabaja
  public Distance metrics;     // Medida de disimilaridad empleada

  Vector   clusters;    // Vector de agrupamientos

  String   label[];     // Etiquetas de las clases (si procede)

  // Constructor

  public ClusteringAlgorithm (DataConfiguration cfg, Dataset set, Distance dist)
  {
    super(cfg);

    data     = set;
    metrics  = dist;
    clusters = new Vector();
    label    = cfg.getClasses();
  }

  // Manejo de clusters individuales

  public int clusters ()
  {
    return clusters.size();
  }

  public Cluster addCluster ()
  {
    Cluster cluster = new Cluster (metrics, noa, label);

    clusters.addElement (cluster);
  
    return cluster;
  }

  public Cluster removeCluster (int n)
  {
    Cluster cluster = (Cluster) clusters.elementAt(n);

    clusters.removeElementAt(n);

    return cluster;
  }

  public Cluster cluster (int n)
  {
    return (Cluster) clusters.elementAt(n);
  }

  // Clasificacin

  public Cluster cluster (Pattern p)
  {
    int     k;
    float   dist;
    Cluster cluster = cluster(0);
    float   min     = metrics.distance(cluster,p);

    for (k=1; k<clusters(); k++) {
        dist = metrics.distance(cluster(k),p);

        if (dist<min) {
           min     = dist;
           cluster = cluster(k);
        }
    }

    return cluster;
  }

  public int classify (Pattern p)
  {
    Cluster cluster = cluster(p);

    if (cluster!=null) {
       return cluster.classID();
    } else {
       return -1;
    }
  }

  // Informe

  public void printReport ()
  {
    int k;

    for (k=0; k<clusters(); k++) {
        System.out.println("Cluster "+(k+1)+":");
        System.out.println(cluster(k).toString());
    }
  }

}