package rf.clustering;

// Algoritmo de agrupamiento secuencial

import rf.*;
import rf.data.*;
import rf.distance.*;

/**
 * Clustering with a "known" number of classes:
 * Sequential clustering
 */

public class SequentialClustering extends ClusteringAlgorithm
{
  // Constructor

  public SequentialClustering ( DataConfiguration cfg, Dataset set, Distance dm,
                                int K, float R, float C, int M, float T )
  {
    super (cfg,set,dm);

    int     i,j,c,mejor,mejori,mejorj,minsize;
    int     N = set.instances();
    int     A;
    Pattern p;
    float   dist,mindist;
    Cluster best;


    // ***** set.shuffle(); *****

    // Inicializacin

    addCluster();
    A = 1;
    cluster(0).assign (set.pattern(0));

    // Bucle principal

    for (c=1; c<N; c++) {

        p = set.pattern(c);

        // Agrupamiento ms cercano a 'p'

        best    = cluster(0);
        mindist = metrics.distance(best,p);

        for (j=1; j<A; j++) {
            dist = metrics.distance(cluster(j),p);

            if (dist<mindist) {
               mindist = dist;
               best    = cluster(j);
            }
        }

        // Asignacin del patrn 'p'
        // -------------------------

        if (mindist<=R) {

           // Asignar el patrn al agrupamiento ms cercano

           best.assign(p);

        } else {

           // Crear un nuevo agrupamiento

           addCluster();
           cluster(A).assign (p);
           A++;
        }


        // Reajuste de los agrupamientos
        // -----------------------------

        if ( c%M==0 || c==N-1 ) {

           // Mezcla de agrupamientos por cercana

           do {

              mindist = 1e30f;
              mejori  = -1;
              mejorj  = -1;

              for (i=0; i<A; i++)
                  for (j=i+1; j<A; j++) {
                      dist = metrics.distance(cluster(i),cluster(j));

                      if (dist<mindist) {
                         mindist = dist;
                         mejori = i;
                         mejorj = j;
                      }
                  }

              if (mindist<C) {
                 mezcla(mejori,mejorj);
                 A--;
              }

           } while (mindist<C);

           // Mezcla de agrupamientos por tamao (<T*M)

           if (A>K) {

              do {

                 mejori  = 0;
                 minsize = cluster(0).size();

                 for (i=1; i<A; i++)
                     if ( cluster(i).size()<minsize ) {
                        mejori  = i;
                        minsize = cluster(i).size();
                     }

                 if ( minsize<T*M ) {

                    // Mezclarlo con el agrupamiento ms cercano

                    mindist = 1e30f;
                    mejorj  = mejori;

                    for (j=0; j<A; j++)
                        if (j!=mejori) {
                           dist = metrics.distance(cluster(mejori),cluster(j));

                           if (dist<mindist) {
                              mindist = dist;
                              mejorj  = j;
                           }
                        }

                    if (mejorj!=mejori) {
                       mezcla(mejorj,mejori);
                       A--;
                    }
                 }

              } while (minsize<T*M);
           }

           // Mezcla forzada

           while (A>K) {

                 mindist = 1e30f;
                 mejori  = -1;
                 mejorj  = -1;

                 for (i=0; i<A; i++)
                     for (j=i+1; j<A; j++) {
                         dist = metrics.distance(cluster(i),cluster(j));

                         if (dist<mindist) {
                            mindist = dist;
                            mejori = i;
                            mejorj = j;
                         }
                     }

                 mezcla(mejori,mejorj);
                 A--;
           }
        }
    }
  }

  // Mezcla de clusters: cluster(i) = cluster(i) U cluster(j)

  private void mezcla (int i, int j)
  {
    cluster(i).add(cluster(j));

    removeCluster(j);
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   cfg;
    Data                dat;
    Dataset             set;
    Distance            dist;
    int                 k,m;
    float               r,c,t;
    ClusteringAlgorithm clustering;


    if (args.length!=6) {

       System.out.println ("Sintaxis: SequentialClustering <fichero_cfg> <k> <r> <c> <m> <t>");

    } else {

       cfg = new DataConfiguration (args[0]);
       k   = (new Integer(args[1])).intValue();
       r   = (new Float(args[2])).floatValue();
       c   = (new Float(args[3])).floatValue();
       m   = (new Integer(args[4])).intValue();
       t   = (new Float(args[5])).floatValue();

       if (cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       if (k<2)
          k = 2;

       System.out.print ( cfg.getDescriptionString() );

       dat = cfg.getData();

       System.out.println ();
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+dat.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+dat.testInstances()+" patrones");
       System.out.println ();

       set  = dat.patterns();
       dist = new EuclideanDistance(set);

       System.out.println ("Algoritmo de agrupamiento secuencial\n");

       clustering = new SequentialClustering (cfg,set,dist,k,r,c,m,t);

       clustering.printReport();
    }
  }

}