package rf.clustering;

// Variante del K-MEDIAS

import rf.*;
import rf.data.*;
import rf.distance.*;

/**
 * Clustering with a known number of classes:
 * Minimization of sum of squared distance (K-MEANS)
 */

public class kMeans extends ClusteringAlgorithm
{
  // Constructor

  public kMeans (DataConfiguration cfg, Dataset set, Distance dm, int K)
  {
    super (cfg,set,dm);

    int     i,j;
    int     N = set.instances();
    Pattern p;
    Cluster actual[] = new Cluster[N];
    int     cambios;
    float   dist,mindist;
    Cluster best;

    // Inicializacin

    if (K>N)
       K = N;

    // ALTERNATIVA A:
    // Seleccin inicial de los centros de los K agrupamientos
/**
    for (j=0; j<K; j++) {
        addCluster();

        do {
           i = (int)(N*Math.random());
        } while (actual[i]!=null);

        cluster(j).assign ( set.pattern(i) );
        actual[i] = cluster(j);
    }
/**/

    // ALTERNATIVA B:
    // Asignacin inicial aleatoria
/**/
    for (i=0; i<K; i++)
        addCluster();

    for (i=0; i<N; i++) {
        j = (int) (K*Math.random());
        cluster(j).assign ( set.pattern(i) );
        actual[i] = cluster(j);
    }
/**/

    // Bucle principal

    do {

        cambios = 0;

        // Asignacin al cluster ms cercano

        for ( i=0; i<N; i++) {

            p = set.pattern(i);

            best    = cluster(0);
            mindist = metrics.distance(cluster(0),p);

            for (j=1; j<K; j++) {

                dist = metrics.distance(cluster(j),p);

                if (dist<mindist) {
                   best = cluster(j);
                   mindist = dist;
                }
            }

            if (best!=actual[i]) {
               actual[i] = best;
               cambios++;
            }
        }

        // Actualizacin de los centroides de los clusters

        if (cambios>0) {

           for (j=0; j<K; j++)
               cluster(j).clear();

           for (i=0; i<N; i++)
               actual[i].assign (set.pattern(i));
        }

        // VARIANTE
        // Eliminacin de agrupamientos vacos

        for (j=K-1; j>=0; j--)
            if (cluster(j).size()==0)
               removeCluster(j);

        K = clusters();

    } while (cambios>0);

  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   cfg;
    Data                dat;
    Dataset             set;
    Distance            dist;
    int                 k;
    ClusteringAlgorithm clustering;


    if (args.length!=2) {

       System.out.println ("Sintaxis: kMeans <fichero_cfg> <k>");

    } else {

       cfg = new DataConfiguration (args[0]);
       k   = (new Integer(args[1])).intValue();

       if (cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       if (k<2)
          k = 2;

       System.out.print ( cfg.getDescriptionString() );

       dat = cfg.getData();

       System.out.println ();
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+dat.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+dat.testInstances()+" patrones");
       System.out.println ();

       set  = dat.patterns();
       dist = new EuclideanDistance(set);

       System.out.println ("Algoritmo de las K-Medias\n");

       clustering = new kMeans (cfg,set,dist,k);

       clustering.printReport();
    }
  }

}