package rf.data;

// DATASET
// -------
// ASCII file

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers

import java.util.Vector;

/**
 * <p>
 * Input dataset modelling
 *
 * <p>
 * Format: ASCII file
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.AsciiData
 */

public class AsciiDataset implements Dataset
{
  // Variables de instancia
  // ----------------------

  Data   data;
  Vector casos;

  // Constructor
  // -----------

  public AsciiDataset (Data d)
  {
    data  = d;
    casos = new Vector();
  }

  public void add (Pattern p)
  {
    casos.addElement (p);
  }

  // CONTABILIDAD GENERAL
  // --------------------

  // Nmero de atributos (caractersticas)

  public int attributes ()
  {
    return data.attributes();
  }

  // Nmero total de casos (patrones)

  public int instances ()
  {
    return casos.size();
  }

  // DATOS
  // -----

  // Valores correspondientes a un caso del conjunto de datos

  public Pattern pattern (int k)
  {
    return (Pattern) casos.elementAt(k);
  }

  // Valor del atributo n para el caso k

  public float  value (int k, int n)
  {
    return pattern(k).attribute(n);
  }

  public String label (int k)
  {
    return pattern(k).label();
  }

  // Valores de uno de los atributos

  public String[] label ()
  {
    int    i;
    String array[] = new String[instances()];

    for (i=0; i<array.length; i++)
        array[i] = label(i);

    return array;
  }

  public float[]  attribute (int n)
  {
    int   i;
    float array[] = new float[instances()];

    for (i=0; i<array.length; i++)
        array[i] = value(i,n);

    return array;
  }

  // Salida estndar

  public String toString ()
  {
    int    i;
    String str = "";

    for (i=0; i<instances(); i++)
        str += pattern(i).toString() + "\n";

    return str;
  }

  // Barajar los datos

  public void shuffle ()
  {
    int     i,j,N;
    Pattern pi,pj;

    N = instances();

    for (i=0; i<N; i++) {
        j = i + (int)((N-i)*Math.random()); // [i..CROMOSOMAS(ga)-1]
        pi = (Pattern) casos.elementAt(i);
        pj = (Pattern) casos.elementAt(j);
        casos.setElementAt ( pi, j);
        casos.setElementAt ( pj, i);
    }
  }
}
