package rf.data;

// DATA
// ----------
// Input data

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers & documentation

/**
 * <p>
 * Interface for input data modelling
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.Dataset
 */

public interface Data
{
  // CONTABILIDAD GENERAL
  // --------------------

  // Nmero de atributos (sin contar la clase)

  /**
   * Returns the number of pattern attributes (not counting the class label).
   */

  public int attributes ();

  // Nmero total de casos disponibles

  /**
   * Returns the number of patterns available.
   */

  public int instances ();

  // CONJUNTO DE CASOS (PATRONES)
  // ----------------------------

  /**
   * Returns all patterns available.
   */

  public Dataset patterns ();

  // CONJUNTO DE APRENDIZAJE

  /**
   * Returns all patterns available for learning.
   */

  public Dataset learningSet ();

  // Nmero de casos del conjunto de aprendizaje

  /**
   * Returns the cardinality of the learning set.
   */

  public int learningInstances ();

  // CONJUNTO DE PRUEBA

  /**
   * Returns all patterns available for testing.
   */

  public Dataset testSet ();

  // Nmero de casos del conjunto de prueba

  /**
   * Returns the cardinality of the test set.
   */

  public int testInstances ();

  // CONJUNTO DE DATOS POR CLASIFICAR

  /**
   * Returns all unlabeled patterns.
   */

  public Dataset unlabeledSet ();

  // Nmero de datos por clasificar

  /**
   * Returns the number of unlabeled instances.
   */

  public int unlabeledInstances ();
}
