package rf.data;

// DATASET
// -------------
// Input dataset

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers & documentation

/**
 * <p>
 * Interface for input dataset modelling
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.Data
 */

public interface Dataset
{
  // CONTABILIDAD GENERAL
  // --------------------

  // Nmero de atributos (caractersticas)

  /**
   * Returns the number of pattern attributes (not counting the class label).
   */

  public int attributes ();

  // Nmero total de casos (patrones)

  /**
   * Returns the number of patterns available.
   */

  public int instances ();

  // DATOS
  // -----

  // Valores de uno de los atributos

  /**
   * Returns an array which contains all class labels.
   */

  public String[] label ();

  /**
   * Returns an array which contains all values for attribute <code>n</code>
   */

  public float[]  attribute (int n);

  // Valores correspondientes a un caso del conjunto de datos

  /**
   * Returns the <code>k</code>th pattern in the dataset.
   */

  public Pattern pattern (int k);

  // Valor del atributo n para el caso k

  /**
   * Returns the <code>n</code>th attribute value of the <code>k</code>th pattern in the dataset.
   *
   * @param     k  pattern number
   * @param     n  attribute number
   * @return    attribute value
   */

  public float  value (int k, int n);

  // Etiqueta del caso k = Clase del caso k

  /**
   * Returns the class label of the <code>k</code>th pattern in the dataset.
   */

  public String label (int k);


  // Baraja las muestras

  /**
   * Shuffles data.
   */

  public void shuffle ();

}

