package rf.data;

// DATASET
// -------
// Image set
// Conjunto de imgenes

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers

import java.util.Vector;

/**
 * <p>
 * Input data modelling
 *
 * <p>
 * Format: Set of image files (8bpp)
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.ImageBand
 * @see     rf.data.ImageDataset
 */

public class ImageData implements Data
{
  // Variables de instancia
  // ----------------------

  int       anchura;        // Anchura de las imgenes
  int       altura;         // Altura de las imgenes

  Vector    bandas;         // Vector de bandas

  ImageBand classification; // Conjunto de entrenamiento (clasificacin)
  ImageBand learning;       // Conjunto de aprendizaje
  ImageBand test;           // Conjunto de test

  // Constructor
  // -----------

  public ImageData (int width, int height)
  {
    anchura        = width;
    altura         = height;
    bandas         = new Vector();
    classification = null;
    learning       = null;
    test           = null;
  }


  public void addBand (String fichero)
  {
    ImageBand band = new ImageBand (fichero, anchura, altura);

    bandas.addElement(band);
  }


  public void setClassification (String fichero)
  {
    classification = new ImageBand (fichero,anchura,altura);
  }

  public void setLearning (String fichero)
  {
    learning = new ImageBand (fichero,anchura,altura);
  }

  public void setTest (String fichero)
  {
    test = new ImageBand (fichero,anchura,altura);
  }

  // CONTABILIDAD GENERAL
  // --------------------

  // Nmero de atributos (sin contar la clase)

  public int attributes ()
  {
    return bandas.size();
  }

  // Banda n

  public ImageBand band (int n)
  {
    return (ImageBand) bandas.elementAt(n);
  }

  // Tamao de las imgenes

  public int getWidth ()
  {
    return anchura;
  }

  public int getHeight ()
  {
    return altura;
  }

  public Pattern getPattern (int x, int y)
  {
    int     i;
    float   valor;
    float[] attr;

    if (x>=0 && x<anchura && y>=0 && y<altura) {

       valor = classification.value(x,y);

       attr  = new float[attributes()];

       for (i=0; i<attributes(); i++)
           attr[i] = band(i).value(x,y);

       return new Pattern(attr,new Integer((int)valor).toString());

    } else {

       return null;
    }
  }

  // Nmero total de casos disponibles (etiquetados)

  public int instances ()
  {
    int k;
    int dim   = anchura*altura;
    int total = 0;

    for (k=0; k<dim; k++)
        if (classification.value(k)!=0)
           total++;

    return total;
  }

  // CONJUNTO DE CASOS (PATRONES)
  // ----------------------------

  public Dataset patterns ()
  {
    return new ImageDataset(this, ImageDataset.All);
  }

  // CONJUNTO DE APRENDIZAJE

  public Dataset learningSet ()
  {
    return new ImageDataset(this, ImageDataset.Learning);
  }

  // Nmero de casos del conjunto de aprendizaje

  public int learningInstances ()
  {
    int k;
    int dim   = anchura*altura;
    int total = 0;

    for (k=0; k<dim; k++)
        if (learning.value(k)!=0)
           total++;

    return total;
  }

  // CONJUNTO DE PRUEBA

  public Dataset testSet ()
  {
    return new ImageDataset (this, ImageDataset.Test);
  }

  // Nmero de casos del conjunto de prueba

  public int testInstances ()
  {
    int k;
    int dim   = anchura*altura;
    int total = 0;

    for (k=0; k<dim; k++)
        if (test.value(k)!=0)
           total++;

    return total;
  }

  // CONJUNTO DE DATOS POR CLASIFICAR

  public Dataset unlabeledSet ()
  {
    return new ImageDataset (this, ImageDataset.Classifier);
  }

  // Nmero de datos por clasificar

  public int unlabeledInstances ()
  {
    int k;
    int dim   = anchura*altura;
    int total = 0;

    for (k=0; k<dim; k++)
        if (classification.value(k)==0)
           total++;

    return total;
  }

  // PRUEBA
  // ------

  public static void main (String argv[])
  {
/**/
    // IGALIKO

    String    path = "C:/etsii/rf/data/";
    ImageData data = new ImageData(256,256);

    data.addBand(path+"igaliko/igal1");
    data.addBand(path+"igaliko/igal2");
    data.addBand(path+"igaliko/igal3");
    data.addBand(path+"igaliko/igal4");

    data.setClassification (path+"igaliko/igal_tra");
    data.setLearning       (path+"igaliko/igal_lrn");
    data.setTest           (path+"igaliko/igal_tst");
/**
    // GALAXY

    String    path = "C:/etsii/rf/data/";
    ImageData data = new ImageData(256,256);

    data.addBand(path+"galaxy/gal1");
    data.addBand(path+"galaxy/gal2");
    data.addBand(path+"galaxy/gal3");

    data.setClassification (path+"galaxy/galtrain");
    data.setLearning       (path+"galaxy/gallearn");
    data.setTest           (path+"galaxy/gal_test");
/**/
    // Nmero de casos

    System.out.println(" Total de casos = "+data.instances());
    System.out.println(" - Aprendizaje   "+data.learningInstances());
    System.out.println(" - Prueba        "+data.testInstances());

    // Primer dato

    Dataset dataset = data.patterns();

    System.out.println(dataset.pattern(0));
  }

}