package rf.data;

// DATASET
// -------
// Conjunto de imgenes

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers

import java.util.Vector;

/**
 * <p>
 * Input dataset modelling
 *
 * <p>
 * Format: Set of image bands
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.ImageBand
 * @see     rf.data.ImageData
 */

public class ImageDataset implements Dataset
{
  public static final byte All        = 1;  // Todos los casos conocidos
  public static final byte Learning   = 2;  // Casos de aprendizaje
  public static final byte Test       = 3;  // Casos de test
  public static final byte Classifier = 4;  // Casos por clasificar

  // Variables de instancia
  // ----------------------

  ImageData data;       // Datos de los que se deriva el conjunto
  Vector    casos;      // Vector de casos

  // Constructor
  // -----------

  public ImageDataset (ImageData d, byte tipo)
  {
    data  = d;
    casos = new Vector();

    int       k,i;
    float     valor;
    float[]   attr;
    ImageBand clase;  // Banda con la informacin de la clase


    switch (tipo) {

      case Learning:
           clase = data.learning;
           break;

      case Test:
           clase = data.test;
           break;

      default:                          // All | Classifier
           clase = data.classification;
           break;
    }


    for (k=0; k<data.anchura*data.altura; k++) {

        valor = clase.value(k);

        if (  valor!=0
           || (valor==0 && tipo==Classifier)) {

           attr = new float[data.attributes()];

           for (i=0; i<data.attributes(); i++)
               attr[i] = data.band(i).value(k);

           casos.addElement (new Pattern(attr,new Integer((int)valor).toString()));
        }
    }

  }

  // CONTABILIDAD GENERAL
  // --------------------

  // Nmero de atributos (caractersticas)

  public int attributes ()
  {
    return data.attributes();
  }

  // Nmero total de casos (patrones)

  public int instances ()
  {
    return casos.size();
  }

  // DATOS
  // -----

  // Valores correspondientes a un caso del conjunto de datos

  public Pattern pattern (int k)
  {
    return (Pattern) casos.elementAt(k);
  }

  // Valor del atributo n para el caso k

  public float  value (int k, int n)
  {
    return pattern(k).attribute(n);
  }

  public String label (int k)
  {
    return pattern(k).label();
  }


  // Valores de uno de los atributos

  public String[] label ()
  {
    int    i;
    String array[] = new String[instances()];

    for (i=0; i<array.length; i++)
        array[i] = label(i);

    return array;
  }

  public float[]  attribute (int n)
  {
    int   i;
    float array[] = new float[instances()];

    for (i=0; i<array.length; i++)
        array[i] = value(i,n);

    return array;
  }

  // Barajar los datos

  public void shuffle ()
  {
    // Not supported
  }

}
