package rf.data;

// PATTERN
// -------
// Patrn = Atributos + Clase

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers and documentation

/**
 * <p>
 * Models a pattern (set of attributes + class label)
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 */

public class Pattern
{
  float  attr[];
  String clase;

  // Constructor

  /**
   * Constructs a <code>n</code>-dimensional pattern
   *
   * @param     n       number of attributes
   */

  public Pattern (int n)
  {
    attr  = new float[n];
    clase = null;
  }

  /**
   * Constructs a pattern given its attribute values and class label.
   *
   * @param     a       attribute values
   * @param     c       class label
   */

  public Pattern (float a[], String c)
  {
    attr  = a;
    clase = c;
  }

  /**
   * Constructs a new pattern equal to a given one
   */

  public Pattern (Pattern p)
  {
    attr  = new float[p.attr.length];
    clase = p.clase;

    int i;

    for (i=0; i<attr.length; i++)
        attr[i] = p.attr[i];
  }

  // Acceso a las variables de instancia

  /**
   * Returns the length of the pattern
   *
   * @return number of attributes
   */

  public int attributes ()
  {
    return attr.length;
  }

  /**
   * Returns the pattern <code>n</code>th attribute value.
   *
   * @param     n       attribute number
   * @return            attribute value
   */

  public float attribute (int n)
  {
    return attr[n];
  }

  /**
   * Sets the pattern <code>n</code>th attribute value.
   *
   * @param     n       attribute number
   * @param     v       attribute value
   */

  public void attribute (int n, float v)
  {
    attr[n] = v;
  }

  /**
   * Returns the pattern class label.
   *
   * @return            class label
   */

  public String label ()
  {
    return clase;
  }

  /**
   * Sets the pattern label.
   *
   * @param     label       pattern label
   */

  public void label (String label)
  {
    clase = label;
  }

  // Salida estndar

  /**
   * String representation of the pattern.
   */

  public String toString ()
  {
    int    i;
    String str = "";

    for (i=0; i<attr.length; i++)
        str += attribute(i) + " ";

    str += label();

    return str;
  }

}
