package rf.data;

// DATA
// ---------------------------
// Input data (via JDBC)
// Datos de entrada (va JDBC)

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers

import java.sql.*;

import fbg.util.db.*;

/**
 * <p>
 * Input data modelling
 *
 * <p>
 * Format: Database table (accessed via JDBC)
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.TableDataset
 */

public class TableData implements Data
{
  // Variables de instancia
  // ----------------------

  Tabla        tabla;     // Tabla de la que se obtienen los datos
  boolean      label;     // Clase disponible ?
  String       id[];      // Atributos escogidos (la clase el ltimo)
  int          atributos; // Nmero de atributos

  TableDataset dataset;   // Casos de entrenamiento (todos)
  TableDataset learning;  // Casos de aprendizaje
  TableDataset test;      // Casos de prueba

  // Constructor
  // -----------

  public TableData (Tabla t, String a[], boolean classification)
  {
    this (t, a, classification, 0.7);   // 70% Learning
                                        // 30% Test
  }

  public TableData (Tabla t, String attr[], boolean classification, double p)
  {
    label     = classification;
    tabla     = t;
    id        = attr;
    atributos = (label)? id.length - 1: id.length;
    dataset   = new TableDataset(this);
    learning  = new TableDataset(this);
    test      = new TableDataset(this);

    init(dataset);

    // Distribucin aleatoria de las muestras

    int i;

    for (i=0; i<dataset.instances(); i++) {
        if (Math.random()<p) {
           learning.add ( dataset.pattern(i) );
        } else {
           test.add ( dataset.pattern(i) );
        }
    }
  }


  private void init (TableDataset datos)
  {
    int       i;
    Statement SQL   = null;
    ResultSet tupla = null;
    String    attr;

    float[]   valor;
    String    clase;

    try {

  	   SQL = tabla.acceso.createStatement();

	   // Atributos de la tabla de datos

	   attr = id[0];

	   for (i=1; i<id.length; i++)
	       attr += "," + id[i];

  	   // Recorrido secuencial por la tabla

	   tupla = SQL.executeQuery ( "SELECT " + attr + " FROM " + tabla.id );

	   while (tupla.next()) {

          valor = new float[atributos];

          for (i=0; i<atributos; i++) {
              valor[i] = tupla.getFloat (i+1);
          }

          if (label)
             clase = tupla.getString(atributos+1);
          else
             clase = null;

          if (clase!=null)
             clase = clase.trim();

          datos.add ( new Pattern(valor,clase) );        
	   }

    } catch (SQLException e) {

       while (e != null) {
	      System.out.println(e.getMessage());
	      e = e.getNextException();
	   }
    }
  }


  // CONTABILIDAD GENERAL
  // --------------------

  // Nmero de atributos (sin contar la clase)

  public int attributes ()
  {
    return atributos;
  }

  // Nmero total de casos disponibles

  public int instances ()
  {
    return dataset.instances();
  }

  // CONJUNTO DE CASOS (PATRONES)
  // ----------------------------

  public Dataset patterns ()
  {
    return dataset;
  }

  // CONJUNTO DE APRENDIZAJE

  public Dataset learningSet ()
  {
    return learning;
  }

  // Nmero de casos del conjunto de aprendizaje

  public int learningInstances ()
  {
    return learning.instances();
  }

  // CONJUNTO DE PRUEBA

  public Dataset testSet ()
  {
    return test;
  }

  // Nmero de casos del conjunto de prueba

  public int testInstances ()
  {
    return test.instances();
  }

  // CONJUNTO DE DATOS POR CLASIFICAR

  public Dataset unlabeledSet ()
  {
    return null;
  }

  // Nmero de datos por clasificar

  public int unlabeledInstances ()
  {
    return 0;
  }


  // PRUEBA
  // ------
/*
  public static void main (String argv[])
  {
    String user     = "system";
    String password = "manager";
    String URL      = "jdbc:odbc:POLite"; // ODBC Bridge
    String JDBC     = "sun.jdbc.odbc.JdbcOdbcDriver";

    Connection connect = null;
    Tabla      tabla;

    String idTabla  = "C45";
    String attr[]   = { "TEMPERATURE", "HUMIDITY", "PLAY" };

    // JDBC

    System.out.print("Cargando el driver JDBC...\n");

    try {
        Class.forName(JDBC);
    } catch (Exception error) {
    }

    System.out.print("Estableciendo la conexin...\n");

    try {
        connect = DriverManager.getConnection(URL,user,password);
    } catch (Exception error) {
    }

    System.out.print("Accediendo a la tabla...\n");

    tabla   = new Tabla(connect,idTabla);

    // Conjunto de datos

    TableData data = new TableData (tabla,attr);

    // Info

    System.out.println(" Casos = "+data.instances());
    System.out.println(" - Aprendizaje "+data.learningInstances());
    System.out.println(" - Prueba "+data.testInstances());

    // Primer dato

    System.out.println(data.patterns().pattern(0));
  }
*/

}
