package rf.distance;

// Distancia de Mahalanobis

import fbg.util.math.Matrix;

import rf.Statistics;
import rf.data.Dataset;
import rf.data.Pattern;
import rf.clustering.Cluster;


/**
 *  Distance metrics (dissimilarity measure): Squared Mahalanobis distance
 */

public class MahalanobisDistance extends Distance
{
  int     noa;        // Nmero de atributos
  Matrix  invcov;     // Inversa de la matriz de covarianza


  // Constructor

  public MahalanobisDistance ( Dataset data )
  {
    noa  = data.attributes();

    Statistics stats      = new Statistics (data);
    Matrix     covariance = stats.covariance();

    invcov = covariance.inverse();

    if (invcov==null)
       throw ( new ArithmeticException ("Singular covariance matrix") );
  }


  // Distancia entre patrones
  // Mide la distancia de un patrn a otro (medida de disimilaridad)

  public float distance (Pattern p, Pattern q)
  {
    Matrix diff;
    Matrix difft;
    Matrix mahalanobis;
    int    i;

    diff = new Matrix (noa, 1);

    for (i=0; i<noa; i++)
        diff.setCell (i, 0, p.attribute(i) - q.attribute(i) );

    difft = diff.transpose();

    mahalanobis = difft.mult ( invcov.mult ( diff ) );

    return (float) mahalanobis.getCell(0,0);
  }

  public float distance (Cluster c, Pattern p)
  {
    Matrix diff;
    Matrix difft;
    Matrix mahalanobis;
    float  dist;
    int    i;

    if (c.size()>0) {

       diff = new Matrix (noa, 1);

       for (i=0; i<noa; i++)
           diff.setCell (i, 0, c.centroid(i) - p.attribute(i) );

       difft = diff.transpose();

       mahalanobis = difft.mult ( invcov.mult ( diff ) );

       dist = (float) mahalanobis.getCell(0,0);

    } else {

       dist = 1e30f;
    }

    return dist;
  }

  public float distance (Cluster c1, Cluster c2)
  {
    Matrix diff;
    Matrix difft;
    Matrix mahalanobis;
    int    i;

    diff = new Matrix (noa, 1);

    for (i=0; i<noa; i++)
        diff.setCell (i, 0, c1.centroid(i) - c2.centroid(i) );

    difft = diff.transpose();

    mahalanobis = difft.mult ( invcov.mult ( diff ) );

    return (float) mahalanobis.getCell(0,0);
  }
}