package rf.distance;

// Distancia eucldea normalizada al cuadrado

import rf.data.Dataset;
import rf.data.Pattern;
import rf.clustering.Cluster;

/**
 *  Distance metrics (dissimilarity measure): Squared normalized euclidean distance
 */

public class NormalizedEuclideanDistance extends Distance
{
  int   noa;          // Nmero de atributos
  float range[];      // Rango de cada atributo: max-min

  // Constructor

  public NormalizedEuclideanDistance ( Dataset data )
  {
    noa   = data.attributes();
    range = new float[noa];

    int     N         = data.instances();
    float   min[]     = new float[noa];
    float   max[]     = new float[noa];
    float   val;
    int     i,k;

    // Mnimos y mximos de cada variable

    for (i=0; i<noa; i++) {
        min[i] = data.value(0,i);
        max[i] = data.value(0,i);
    }

    for (k=1; k<N; k++) {
        for (i=0; i<noa; i++) {

            val = data.value(k,i);

            if (val>max[i])
               max[i] = val;
            else if (val<min[i])
               min[i] = val;
        }
    }

    for (i=0; i<noa; i++)
        range[i] = max[i] - min[i];
  }

  // Distancia entre patrones
  // Mide la distancia de un patrn a otro (medida de disimilaridad)

  public float distance (Pattern p, Pattern q)
  {
    float diff;
    float dist = 0;
    int   i;

    for (i=0; i<noa; i++) {
        if (range[i]>0) {
           diff  = ( p.attribute(i) - q.attribute(i) ) / range[i];
           dist += diff*diff;
        }
    }

    return dist;
  }

  public float distance (Cluster c, Pattern p)
  {
    float diff;
    float dist = 0;
    int   i;

    if (c.size()>0) {

       for (i=0; i<noa; i++) {
           if (range[i]>0) {
              diff  = ( c.centroid(i) - p.attribute(i) ) / range[i];
              dist += diff*diff;
           }
       }

    } else {

       dist = 1e30f;
    }

    return dist;
  }

  public float distance (Cluster c1, Cluster c2)
  {
    float diff;
    float dist = 0;
    int   i;

    for (i=0; i<noa; i++) {
        if (range[i]>0) {
           diff  = ( c1.centroid(i) - c2.centroid(i) ) / range[i];
           dist += diff*diff;
        }
    }

    return dist;
  }
}