package rf.distance;

// Distancia eucldea ponderada al cuadrado

import rf.Statistics;
import rf.data.Dataset;
import rf.data.Pattern;
import rf.clustering.Cluster;

/**
 *  Distance metrics (dissimilarity measure): Squared weighted Euclidean distance
 */

public class WeightedEuclideanDistance extends Distance
{
  int   noa;          // Nmero de atributos
  float variance[];   // Varianza de cada atributo

  // Constructor

  public WeightedEuclideanDistance ( Dataset data )
  {
    noa      = data.attributes();
    variance = new float[noa];

    Statistics stats = new Statistics(data);
    int        i;

    for (i=0; i<noa; i++)
        variance[i] = stats.variance(i);
  }

  // Distancia entre patrones
  // Mide la distancia de un patrn a otro (medida de disimilaridad)

  public float distance (Pattern p, Pattern q)
  {
    float diff;
    float d = 0;
    int   i;

    for (i=0; i<noa; i++) {
        if (variance[i]>0) {
           diff  = p.attribute(i) - q.attribute(i);
           d    += diff*diff/variance[i];
        }
    }

    return d;
  }

  public float distance (Cluster c, Pattern p)
  {
    float diff;
    float d = 0;
    int   i;

    if (c.size()>0) {

       for (i=0; i<noa; i++) {
           if (c.variance(i)>0) {
              diff  = c.centroid(i) - p.attribute(i);
              d    += diff*diff/c.variance(i);
           }
       }

    } else {

       d = 1e30f;
    }

    return d;
  }

  public float distance (Cluster c1, Cluster c2)
  {
    float diff;
    float d = 0;
    int   i;

    for (i=0; i<noa; i++) {
        if (variance[i]>0) {
           diff  = c1.centroid(i) - c2.centroid(i);
           d    += diff*diff/variance[i];
        }
    }

    return d;
  }
}