package rf.edit;

// Algoritmo de condensado de Hart

import fbg.util.Benchmark;

import rf.*;
import rf.data.*;

import rf.classification.nn.NNClassifier;


public class Hart
{
  DataConfiguration cfg;
  Dataset           original;
  TableDataset      result;

  int               noc;
  String            classes[];

  // Constructor

  public Hart (DataConfiguration config, Dataset set)
  {
    int          i,N;
    boolean      end;
    TableDataset tmp;
    Pattern      p;

    cfg      = config;
    original = set;
    noc      = cfg.totalClasses();
    classes  = cfg.getClasses();


    N = original.instances();

    result = new TableDataset (cfg.getData());
    tmp    = new TableDataset (cfg.getData());

    for (i=0; i<N; i++)
        tmp.add( original.pattern(i) );


    do {

       tmp.shuffle();

       end = true;
       i   = 0;

       while (i<tmp.instances()) {

             p = tmp.pattern(i);

             if ( diff(result,p) ) {
                tmp.remove (i);
                result.add (p);
                end = false;
             } else {
                i++;
             }
       }

       System.out.println(result.instances());

    } while (!end && tmp.instances()>0);

  }

  // diff: true si la clase del patrn p no coincide con la proporcionada
  // por la regla 1-NN al buscar el vecino ms cercano de p en s.

  boolean diff (Dataset s, Pattern p)
  {
    int          i,c;
    boolean      dif;
    String       label;
    NNClassifier nn;

    label = p.label();

    for (c=0; c<noc && !label.equals(classes[c]); c++)
        ;

    if (c<noc) {

       nn = new NNClassifier (cfg,s);

       if (c!=nn.classify(p))
          dif = true;
       else
          dif = false;

    } else {
       dif = true;
    }

    return dif;
  }


  // Conjunto de entrenamiento original

  public Dataset getOriginal ()
  {
    return original;
  }

  // Conjunto de entrenamiento editado

  public Dataset getResult ()
  {
    return result;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   c        = null;
    Data                d        = null;
    Dataset             set      = null;
    int                 k        = 1;
    Hart                condense = null;
    Benchmark           crono    = null;

    if (args.length<1) {

       System.out.println ("Sintaxis: Hart <fichero_CFG> [<fichero_DAT>]");

    } else {

       c = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       printStringArray ( c.getDescription() );

       d  = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();


       if (args.length==2) {
          set = AsciiData.load ( d, args[1] );
       } else {
          set = d.learningSet();
       }


       crono = new Benchmark();

       crono.start();

       condense = new Hart (c, set );

       crono.stop();

       System.out.println ( "CONDENSADO DE HART: "
                          + condense.getOriginal().instances()
                          + " ---> "
                          + condense.getResult().instances()
                          + " ("+crono+")" );

       AsciiData.save(condense.getResult(),"hart.dat");
    }
  }

  private static void printStringArray (String array[])
  {
    int i;

    if (array!=null)
       for (i=0; i<array.length; i++)
           System.out.println(array[i]);
  }

}