package rf.edit;

// Edicin por particiones

import rf.*;
import rf.data.*;

import rf.classification.nn.kNNClassifier;


public class PartitionEdition
{
  DataConfiguration cfg;
  Dataset           original;
  TableDataset      result;

  int               noc;
  String            classes[];

  // Constructor

  public PartitionEdition (DataConfiguration config, Dataset set, int m, int k)
  {
    int          i,j,N;
    TableDataset tmp[] = new TableDataset[m];
    Pattern      p;

    cfg      = config;
    original = set;
    noc      = cfg.totalClasses();
    classes  = cfg.getClasses();

    result   = new TableDataset (cfg.getData());

    // Difusin: Particin del conjunto de entrenamiento

    for (i=0; i<m; i++)
        tmp[i] = new TableDataset (cfg.getData());

    N = original.instances();

    for (i=0; i<N; i++)
        tmp[(int) (m*Math.random())].add ( original.pattern(i) );
/*
    for (i=0; i<N; i++)
        tmp[i%m].add ( original.pattern(i) );
*/
    // Edicin

    for (i=0; i<m; i++) {

        j = 0;

        while (j<tmp[i].instances()) {

            p = tmp[i].pattern(j);

            if ( diff ( tmp[(i+1)%m], p, k ) )
               tmp[i].remove ( j );
            else
               j++;
        }
    }

    // Confusin

    for (i=0; i<m; i++)
        for (j=0; j<tmp[i].instances(); j++)
            result.add ( tmp[i].pattern(j) );
  }

  // diff: true si la clase del patrn p no coincide con la proporcionada
  // por la regla k-NN al buscar los vecinos de p en s.

  boolean diff (Dataset s, Pattern p, int k)
  {
    int           i,c;
    boolean       dif;
    String        label;
    kNNClassifier knn;

    label = p.label();

    for (c=0; c<noc && !label.equals(classes[c]); c++)
        ;

    if (c<noc) {

       knn = new kNNClassifier (cfg,s,k);

       if (c!=knn.classify(p))
          dif = true;
       else
          dif = false;

    } else {
       dif = true;
    }

    return dif;
  }


  // Conjunto de entrenamiento original

  public Dataset getOriginal ()
  {
    return original;
  }

  // Conjunto de entrenamiento editado

  public Dataset getResult ()
  {
    return result;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   c    = null;
    Data                d    = null;
    int                 m,k;
    PartitionEdition    edit = null;

    if (args.length!=3) {

       System.out.println ("Sintaxis: PartitionEdition <fichero_CFG> <m> <k>");

    } else {

       c = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       m = (new Integer(args[1])).intValue();
       k = (new Integer(args[2])).intValue();

       if (k<1)
          k = 1;

       if (m<2)
          m = 2;

       printStringArray ( c.getDescription() );

       d  = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();


       edit = new PartitionEdition (c, d.learningSet(), m, k);

       System.out.println ( edit.getOriginal().instances()
                          + " ---> "
                          + edit.getResult().instances() );

    }
  }

  private static void printStringArray (String array[])
  {
    int i;

    if (array!=null)
       for (i=0; i<array.length; i++)
           System.out.println(array[i]);
  }

}