package rf.edit;

// Edicin de Wilson

import rf.*;
import rf.data.*;

import rf.classification.nn.kNNClassifier;


public class WilsonEdition
{
  DataConfiguration cfg;
  Dataset           original;
  TableDataset      result;

  int               noc;
  String            classes[];

  // Constructor

  public WilsonEdition (DataConfiguration config, Dataset set, int k)
  {
    int          i,j,N;
    TableDataset aux;

    cfg      = config;
    original = set;
    noc      = cfg.totalClasses();
    classes  = cfg.getClasses();

    // Bucle principal

    N = original.instances();

    result = new TableDataset (cfg.getData());

    for (i=0; i<N; i++)
        result.add( original.pattern(i) );

    for (i=0; i<N; i++) {

        aux = new TableDataset(cfg.getData());

        for (j=0; j<N; j++)
            if (j!=i)
               aux.add (original.pattern(j));

        if ( diff(aux,original.pattern(i),k) )
           result.remove ( original.pattern(i) );

        aux = null;
    }
  }

  // diff: true si la clase del patrn p no coincide con la proporcionada
  // por la regla k-NN al buscar los vecinos de p en s.

  boolean diff (Dataset s, Pattern p, int k)
  {
    int           i,c;
    boolean       dif;
    String        label;
    kNNClassifier knn;

    label = p.label();

    for (c=0; c<noc && !label.equals(classes[c]); c++)
        ;

    if (c<noc) {

       knn = new kNNClassifier (cfg,s,k);

       if (c!=knn.classify(p))
          dif = true;
       else
          dif = false;

    } else {
       dif = true;
    }

    return dif;
  }


  // Conjunto de entrenamiento original

  public Dataset getOriginal ()
  {
    return original;
  }

  // Conjunto de entrenamiento editado

  public Dataset getResult ()
  {
    return result;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   c    = null;
    Data                d    = null;
    int                 k    = 1;
    WilsonEdition       edit = null;

    if (args.length!=2) {

       System.out.println ("Sintaxis: WilsonEdition <fichero_CFG> <k>");

    } else {

       c = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       k = (new Integer(args[1])).intValue();

       if (k<1)
          k = 1;

       printStringArray ( c.getDescription() );

       d  = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();


       edit = new WilsonEdition (c, d.learningSet(), k);

       System.out.println ( edit.getOriginal().instances()
                          + " ---> "
                          + edit.getResult().instances() );

    }
  }

  private static void printStringArray (String array[])
  {
    int i;

    if (array!=null)
       for (i=0; i<array.length; i++)
           System.out.println(array[i]);
  }

}