package  rf.gui;

// Title:       Numerical Cruncher
// Version:     2.0
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;

import fbg.gui.XYLayout;
import fbg.gui.XYConstraints;
import fbg.util.Environment;
import fbg.util.Loader;

// Version 1.0 Jan'99 Standard labels
// Version 2.0 Jun'99 Animated Java logo

public class AboutBox extends Dialog
{
  // Components

  Panel     panel = new Panel();

  XYLayout  xYLayout = new XYLayout();
  
  Label     labelTitle = new Label();
  Label     labelAuthor = new Label();
  
  Button    button = new Button();

  AboutBoxAnimation logo = new AboutBoxAnimation();

  // Constructor

  public AboutBox(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    xYLayout.setWidth(210);
    xYLayout.setHeight(155);

    labelTitle.setForeground(SystemColor.activeCaption);
    labelTitle.setFont(new Font("Dialog", 1, 12));
    labelTitle.setAlignment(1);

    labelAuthor.setAlignment(1);

    button.setActionCommand("");
    button.setLabel("OK");
    button.addActionListener(new AboutBox_buttonAdapter(this));
    
    this.addWindowListener(new AboutBox_windowAdapter(this));

    panel.setLayout(xYLayout);
    panel.add(logo,        new XYConstraints(80, 10,  50, 50));
    panel.add(labelTitle,  new XYConstraints( 5, 65, 200, 25));
    panel.add(labelAuthor, new XYConstraints( 5, 90, 200, 25));
    panel.add(button,      new XYConstraints(65,125,  80, 25));

    add(panel);
    pack();

    if (Environment.settings!=Environment.SPANISH) {
       this.setTitle("About NC...");
       labelTitle.setText("Numerical Cruncher");
    } else {
       labelTitle.setText("Analizador Numrico");
    }

    labelAuthor.setText(" 1999 Fernando Berzal Galiano");

    logo.start();
  }

  public AboutBox(Frame frame, String title)
  {
    this(frame, title, false);
  }

  public AboutBox(Frame frame)
  {
    this(frame, "Acerca de...", false);
  }

  // Close the dialog

  void button_actionPerformed(ActionEvent e)
  {
    logo.stop(); 
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    logo.stop();
    dispose();
  }


  // Test

  public static void main (String args[])
  {
    Frame    frame  = new Frame();
    AboutBox dialog = new AboutBox(frame);

    dialog.show();
  }
}

// Animacin
// ---------

class AboutBoxLogo
{
  // Image animation variables.

  protected Image image;
  protected int   numFrames;
  public    int   width; 
  public    int   height;
  public    int   currentFrame;
  protected int   hFrames;

  private   Panel owner;
  
  // Constructor

  public AboutBoxLogo (Panel owner) 
  {
    int imageHeight;
    int imageWidth;

    this.owner = owner;
    this.image = Loader.loadImage(getClass(), "image/java.gif");

    numFrames = 16;
    hFrames   = 4;

    do {
       imageHeight= image.getHeight (owner);
    } while (imageHeight == -1);

    do {
       imageWidth= image.getWidth (owner);
    } while (imageWidth == -1);

    width  = imageWidth  / 4; 
    height = imageHeight / 4;

    currentFrame= 0;
  } 

  // Change animation frame

  public void tick() 
  {
    if (++currentFrame>=numFrames) currentFrame= 0;
  }

  // Draw it

  public void draw (Graphics g)
  {
    double   offsetx= -(currentFrame%hFrames)*width;
    double   offsety= -Math.floor(currentFrame/hFrames) * height;

    Graphics g2= g.create ((int)0, (int)0, width, height);

    g2.drawImage(image, (int)offsetx, (int)offsety, owner);
    g2.dispose ();
  }

} 

// Panel

class AboutBoxAnimation extends java.awt.Panel implements Runnable
{
  // Main thread, used for ticks.

  public Thread runner = null;
  
  // Time keeper.

  public long currentTickTimeMillis = System.currentTimeMillis();
  public long lastTickTimeMillis;
  
  // Sleep time for the thread (ms)

  public static int   SLEEP = 50;

  // Logo

  public AboutBoxLogo logo = new AboutBoxLogo (this);

  // Start the thread

  public void start() 
  {
    if (runner==null) {
       runner = new Thread(this); 
       runner.start();
    }
  }

  // Stop the thread

  public void stop() 
  {
    runner = null;
  }

  // Execution loop:  Used to call tick().

  public void run() 
  {
    while (runner != null){

          try {
	      Thread.sleep (SLEEP);
	  } catch(InterruptedException e) { } 

	  lastTickTimeMillis= currentTickTimeMillis;
	  currentTickTimeMillis= System.currentTimeMillis();
	  logo.tick();
	  repaint();
    }
  }

  // Override update to avoid screen clears.

  public void update (Graphics g) 
  {	
    paint(g);
  }

  // paint

  public void paint(Graphics g) 
  {
    logo.draw (g); 
  }

}


// Manejo de eventos
// -----------------


class AboutBox_buttonAdapter implements ActionListener
{
  AboutBox adaptee;

  AboutBox_buttonAdapter(AboutBox adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.button_actionPerformed(e);
  }
}

class AboutBox_windowAdapter extends WindowAdapter
{
  AboutBox adaptee;

  AboutBox_windowAdapter(AboutBox adaptee)
  {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e)
  {
    adaptee.this_windowClosing(e);
  }
}

