package rf.gui;

// Title:       2D Plot (Numerical Cruncher)
// Version:     1.0
// Copyright:   January 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import rf.*;
import rf.data.*;
import java.awt.*;


public class Chart2D extends Component
{
  // Variables
  // ---------

  String     classes[];
  int        noc;

  Color      colors[];

  Dataset    set;
  Statistics stats;

  int        x;
  int        y;

  // Constructor
  // -----------

  public Chart2D ( DataConfiguration cfg, Dataset set, int x, int y )
  {
    this.colors  = null;
    this.classes = cfg.getClasses();
    this.set     = set;
    this.x       = x;
    this.y       = y;

    stats = new Statistics (set);

    if (this.classes!=null)
       noc = this.classes.length;
    else
       noc = 0;
  }


  public Chart2D ( DataConfiguration cfg, Dataset set, int x, int y, Color colors[] )
  {
    this.colors  = colors;
    this.classes = null;
    this.noc     = 0;
    this.set     = set;
    this.x       = x;
    this.y       = y;

    stats = new Statistics (set);
  }

  // Configuration
  // -------------

  public void setX (int n)
  {
    x = n;
  }

  public void setY (int n)
  {
    y = n;
  }

  public void setDataset (Dataset set)
  {
    this.set = set;

    stats = new Statistics (set);
  }

  // Visualization
  // -------------

  public void paint (Graphics g)
  {
    int       k;                   // Contador
    float     vx, vy;              // Valores
    int       cx, cy;              // Coordenadas
    float     scaleX,scaleY;       // Escala
    float     minX,minY,maxX,maxY; // Valores extremos
    Dimension dim;
    int       width, height;

    // Size

    dim    = getSize();
    width  = dim.width;
    height = dim.height;


    // min & max

    minX = stats.min(x);
    maxX = stats.max(x);

    minY = stats.min(y);
    maxY = stats.max(y);

    // scale

    if (maxX>minX)
       scaleX = ((float)width-2) / (maxX-minX);
    else
       scaleX = 0;

    if (maxY>minY)
       scaleY = ((float)height-2) / (maxY-minY);
    else
       scaleY = 0;

    // Border

    g.drawRect(0,0,width-1,height-1);

    // Data

    for (k=0; k<set.instances(); k++) {

        vx = set.value(k,x);
        vy = set.value(k,y);

        cx = (int) ((vx-minX)*scaleX);
        cy = (int) (getSize().height - 2 - (vy-minY)*scaleY);

	if (colors!=null)
           g.setColor ( colors[k] );
        else
           g.setColor ( displayColor(set.label(k)) );

        g.fillRect ( cx, cy, 3, 3 );
    }
  }


  private Color displayColor (String label)
  {
    int   c;
    Color color;

    if (noc==0) {

       color = Color.black;

    } else {

       for (c=0; c<noc && !classes[c].equals(label); c++)
           ;

       if (c<noc)
          color = ColorMap.color[c%ColorMap.size];
       else
          color = Color.darkGray;
    }

    return color;
  }
}