package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.01
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import fbg.gui.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.distance.*;
import rf.clustering.*;

// Adaptive Clustering

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

public class DialogAdaptiveClustering extends JDialog
{
  // Data

  DataConfiguration cfg  = null;
  Dataset           data = null;
  Distance          dist = null;

  // Algorithm

  String  title = null;

  // Algorithm parameters

  float   tau   = 1.0f;
  float   theta = 0.8f;

  // Main frame

  MainMenu nc = null;

  // Standard Output

  Output out = null;

  // Online help

  Help help = null;

  // Components

  JPanel panel = new JPanel();
  JPanel panelOptions = new JPanel();

  Border borderOptions = new BevelBorder(BevelBorder.RAISED);

  XYLayout xyLayout = new XYLayout();
  XYLayout xyLayoutOptions = new XYLayout();

  Label labelTAU = new Label();
  Label labelTHETA = new Label();

  JTextField textFieldTAU = new JTextField();
  JTextField textFieldTHETA = new JTextField();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructors

  public DialogAdaptiveClustering(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try {
      init();
      pack();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public DialogAdaptiveClustering (DataConfiguration config, Dataset patterns, Distance dm,
                                   MainMenu frame, Output output, Help help_system)
  {
   this(frame, "", false);

   cfg  = config;
   data = patterns;
   dist = dm;

   nc   = frame;
   out  = output;
   help = help_system;

   // Language

   if (Environment.settings==Environment.ENGLISH) {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Run algorithm");
      buttonCancel.setText("Cancel");
      buttonCancel.setToolTipText("Back to menu");
      buttonHelp.setText("Help");
      buttonHelp.setToolTipText("Online help");

      title = "Adaptive Clustering";

      setTitle(title);

      labelTAU.setText("Distance threshold");
      labelTHETA.setText("Theta");

   } else {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Aplicar algoritmo");
      buttonCancel.setText("Cancelar");
      buttonCancel.setToolTipText("Volver al men");
      buttonHelp.setText("Ayuda");
      buttonHelp.setToolTipText("Ayuda en lnea");

      title = "Agrupamiento adaptativo";

      setTitle(title);

      labelTAU.setText("Umbral de distancia");
      labelTHETA.setText("Theta");
   }

   textFieldTAU.setText(" "+tau);
   textFieldTHETA.setText(" "+theta);
  }

  // GUI

  void init() throws Exception
  {
    panel.setBackground(Color.lightGray);
    panel.setLayout(xyLayout);
    this.getContentPane().add(panel);

    xyLayout.setWidth(320);
    xyLayout.setHeight(140);

    textFieldTAU.setBackground(Color.white);
    textFieldTHETA.setBackground(Color.white);

    buttonOK.addActionListener(new DialogAdaptiveClustering_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new DialogAdaptiveClustering_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new DialogAdaptiveClustering_jButtonHelp_actionAdapter(this));

    panelOptions.setLayout(xyLayoutOptions);
    panelOptions.setBorder(borderOptions);

    this.addWindowListener(new DialogAdaptiveClustering_this_windowAdapter(this));

    panel.add(buttonOK, new XYConstraints(20, 105, 90, 25));
    panel.add(buttonCancel, new XYConstraints(115, 105, 90, 25));
    panel.add(buttonHelp, new XYConstraints(210, 105, 90, 25));

    panel.add(panelOptions, new XYConstraints(10, 10, 300, 80));

    panelOptions.add(labelTAU, new XYConstraints(10, 10, 130, 25));
    panelOptions.add(labelTHETA, new XYConstraints(10, 40, 130, 25));
    panelOptions.add(textFieldTAU, new XYConstraints(145, 10, 138, 27));
    panelOptions.add(textFieldTHETA, new XYConstraints(145, 40, 138, 27));
  }

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    ClusteringAlgorithm clustering;
    Benchmark           crono;
    boolean             ok;
    String              error;

    // Parmetros

    ok = true;

    try {

      tau = (new Float(textFieldTAU.getText().trim())).floatValue();
      theta = (new Float(textFieldTHETA.getText().trim())).floatValue();

    } catch (Exception exception) {

       ok = false;
    }

    if (!ok || tau<=0 || theta<=0) {

       if (Environment.settings==Environment.ENGLISH) {
          error = "Algorithm parameters must be positive real numbers";
       } else {
          error = "Los parmetros del algoritmo deben ser nmeros reales positivos";
       }

       Message.showMessageDialog ( (Component)null,
                                   error,
                                   "Error",
                                   Message.ERROR_MESSAGE);

    } else {

       dispose();

       out.message ("tau = "+tau+" ; theta = "+theta+"\n");

       // Aplicar algoritmo de clustering

       GUI.setWaitCursor(this);

       if (Environment.settings==Environment.SPANISH)
          out.message("Aplicacin del mtodo de agrupamiento... ");
       else
          out.message("Clustering data... ");

       crono = new Benchmark();
       crono.start();

       clustering = new AdaptiveClustering(cfg,data,dist,tau,theta);

       crono.stop();
       out.message(crono+"\n");

       GUI.setDefaultCursor(this);

       nc.showClustering(clustering,data,title);
    }
  }

  // Cancel

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }

  // Help

  void jButtonHelp_actionPerformed(ActionEvent e)
  {
    help.help("nc/clustering/adaptive.html");
  }
}

// Event Handlers
// --------------

class DialogAdaptiveClustering_buttonOK_actionAdapter implements ActionListener{
  DialogAdaptiveClustering adaptee;

  DialogAdaptiveClustering_buttonOK_actionAdapter(DialogAdaptiveClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class DialogAdaptiveClustering_buttonCancel_actionAdapter implements ActionListener{
  DialogAdaptiveClustering adaptee;

  DialogAdaptiveClustering_buttonCancel_actionAdapter(DialogAdaptiveClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class DialogAdaptiveClustering_this_windowAdapter extends WindowAdapter {
  DialogAdaptiveClustering adaptee;

  DialogAdaptiveClustering_this_windowAdapter(DialogAdaptiveClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class DialogAdaptiveClustering_jButtonHelp_actionAdapter implements java.awt.event.ActionListener
{
 DialogAdaptiveClustering adaptee;


 DialogAdaptiveClustering_jButtonHelp_actionAdapter(DialogAdaptiveClustering adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.jButtonHelp_actionPerformed(e);
 }
}
