package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.01
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import fbg.gui.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.distance.*;
import rf.clustering.*;

// Batchelor & Wilkins' Clustering Algorithm

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

public class DialogBatchelorWilkins extends JDialog
{
  // Data

  DataConfiguration cfg  = null;
  Dataset           data = null;
  Distance          dist = null;

  // Algorithm

  String  title = null;

  // Algorithm parameters

  float   f     = 0.8f;

  // Main frame

  MainMenu nc = null;

  // Standard Output

  Output out = null;

  // Online help

  Help help = null;

  // Components

  JPanel panel = new JPanel();
  JPanel panelOptions = new JPanel();

  Border borderOptions = new BevelBorder(BevelBorder.RAISED);

  XYLayout xyLayout = new XYLayout();
  XYLayout xyLayoutOptions = new XYLayout();

  Label labelF = new Label();

  JTextField textFieldF = new JTextField();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructors

  public DialogBatchelorWilkins(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try {
      init();
      pack();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public DialogBatchelorWilkins (DataConfiguration config, Dataset patterns, Distance dm,
                                   MainMenu frame, Output output, Help help_system)
  {
   this(frame, "", false);

   cfg  = config;
   data = patterns;
   dist = dm;

   nc   = frame;
   out  = output;
   help = help_system;

   // Language

   if (Environment.settings==Environment.ENGLISH) {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Run algorithm");
      buttonCancel.setText("Cancel");
      buttonCancel.setToolTipText("Back to menu");
      buttonHelp.setText("Help");
      buttonHelp.setToolTipText("Online help");

      title = "Batchelor and Wilkins' Algorithm";

      setTitle(title);

      labelF.setText("Fraction f");

   } else {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Aplicar algoritmo");
      buttonCancel.setText("Cancelar");
      buttonCancel.setToolTipText("Volver al men");
      buttonHelp.setText("Ayuda");
      buttonHelp.setToolTipText("Ayuda en lnea");

      title = "Algoritmo de Batchelor y Wilkins";

      setTitle(title);

      labelF.setText("Fraccin f");
   }

   textFieldF.setText(" "+f);
  }

  // GUI

  void init() throws Exception
  {
    panel.setBackground(Color.lightGray);
    panel.setLayout(xyLayout);
    this.getContentPane().add(panel);

    xyLayout.setWidth(320);
    xyLayout.setHeight(110);

    textFieldF.setBackground(Color.white);

    buttonOK.addActionListener(new DialogBatchelorWilkins_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new DialogBatchelorWilkins_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new DialogBatchelorWilkins_jButtonHelp_actionAdapter(this));

    panelOptions.setLayout(xyLayoutOptions);
    panelOptions.setBorder(borderOptions);

    this.addWindowListener(new DialogBatchelorWilkins_this_windowAdapter(this));

    panel.add(buttonOK, new XYConstraints(20, 75, 90, 25));
    panel.add(buttonCancel, new XYConstraints(115, 75, 90, 25));
    panel.add(buttonHelp, new XYConstraints(210, 75, 90, 25));

    panel.add(panelOptions, new XYConstraints(10, 10, 300, 50));

    panelOptions.add(labelF, new XYConstraints(10, 10, 130, 25));
    panelOptions.add(textFieldF, new XYConstraints(145, 10, 138, 27));
  }

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    ClusteringAlgorithm clustering;
    Benchmark           crono;
    boolean             ok = true;
    String              error;

    // Parmetros

    try {

        f = (new Float(textFieldF.getText().trim())).floatValue();

    } catch (Exception exception) {

        ok = false;

    }

    if (!ok || f<0 ) {

       if (Environment.settings==Environment.ENGLISH) {
          error = "The parameter must be greater than zero";
       } else {
          error = "El parmetro del algoritmo debe ser un real positivo";
       }

       Message.showMessageDialog ( (Component)null,
                                   error,
                                   "Error",
                                   Message.ERROR_MESSAGE);
    } else {

       dispose();

       out.message ("f = "+f+"\n");

       // Aplicar algoritmo de clustering

       GUI.setWaitCursor(this);

       if (Environment.settings==Environment.SPANISH)
          out.message("Aplicacin del mtodo de agrupamiento... ");
       else
          out.message("Clustering data... ");

       crono = new Benchmark();
       crono.start();

       clustering = new BatchelorWilkins(cfg,data,dist,f);

       crono.stop();
       out.message(crono+"\n");

       GUI.setDefaultCursor(this);

       nc.showClustering(clustering,data,title);
    }
  }

  // Cancel

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }

  // Help

  void jButtonHelp_actionPerformed(ActionEvent e)
  {
    help.help("nc/clustering/BatchelorWilkins.html");
  }
}

// Event Handlers
// --------------

class DialogBatchelorWilkins_buttonOK_actionAdapter implements ActionListener{
  DialogBatchelorWilkins adaptee;

  DialogBatchelorWilkins_buttonOK_actionAdapter(DialogBatchelorWilkins adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class DialogBatchelorWilkins_buttonCancel_actionAdapter implements ActionListener{
  DialogBatchelorWilkins adaptee;

  DialogBatchelorWilkins_buttonCancel_actionAdapter(DialogBatchelorWilkins adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class DialogBatchelorWilkins_this_windowAdapter extends WindowAdapter {
  DialogBatchelorWilkins adaptee;

  DialogBatchelorWilkins_this_windowAdapter(DialogBatchelorWilkins adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class DialogBatchelorWilkins_jButtonHelp_actionAdapter implements java.awt.event.ActionListener
{
 DialogBatchelorWilkins adaptee;


 DialogBatchelorWilkins_jButtonHelp_actionAdapter(DialogBatchelorWilkins adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.jButtonHelp_actionPerformed(e);
 }
}
