package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.01
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import fbg.gui.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.distance.*;
import rf.clustering.*;

// Clustering using GRASP

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

public class DialogGRASPClustering extends JDialog
{
  // Data

  DataConfiguration cfg  = null;
  Dataset           data = null;
  Distance          dist = null;

  // Algorithm

  String  title = null;

  // Algorithm parameters

  int K = 5;
  int I = 3;
  int C = 1;

  // Main frame

  MainMenu nc = null;

  // Standard Output

  Output out = null;

  // Online help

  Help help = null;

  // Components

  JPanel panel = new JPanel();
  JPanel panelOptions = new JPanel();

  Border borderOptions = new BevelBorder(BevelBorder.RAISED);

  XYLayout xyLayout = new XYLayout();
  XYLayout xyLayoutOptions = new XYLayout();

  Label labelK = new Label();
  Label labelI = new Label();
  Label labelC = new Label();

  JTextField textFieldK = new JTextField();
  JTextField textFieldI = new JTextField();
  JTextField textFieldC = new JTextField();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructors

  public DialogGRASPClustering(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try {
      init();
      pack();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public DialogGRASPClustering (DataConfiguration config, Dataset patterns, Distance dm,
                                   MainMenu frame, Output output, Help help_system)
  {
   this(frame, "", false);

   cfg  = config;
   data = patterns;
   dist = dm;

   nc   = frame;
   out  = output;
   help = help_system;

   // Language

   if (Environment.settings==Environment.ENGLISH) {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Run algorithm");
      buttonCancel.setText("Cancel");
      buttonCancel.setToolTipText("Back to menu");
      buttonHelp.setText("Help");
      buttonHelp.setToolTipText("Online help");

      title = "Clustering using GRASP";

      setTitle(title);

      labelK.setText("K");
      labelI.setText("Iterations");
      labelC.setText("Cycle Length");

   } else {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Aplicar algoritmo");
      buttonCancel.setText("Cancelar");
      buttonCancel.setToolTipText("Volver al men");
      buttonHelp.setText("Ayuda");
      buttonHelp.setToolTipText("Ayuda en lnea");

      title = "Agrupamiento con GRASP";

      setTitle(title);

      labelK.setText("K");
      labelI.setText("Iteraciones");
      labelC.setText("Longitud de ciclo");
   }

   textFieldK.setText(" "+K);
   textFieldI.setText(" "+I);
   textFieldC.setText(" "+C);
  }

  // GUI

  void init() throws Exception
  {
    panel.setBackground(Color.lightGray);
    panel.setLayout(xyLayout);
    this.getContentPane().add(panel);

    xyLayout.setWidth(320);
    xyLayout.setHeight(170);

    textFieldK.setBackground(Color.white);
    textFieldI.setBackground(Color.white);
    textFieldC.setBackground(Color.white);

    buttonOK.addActionListener(new DialogGRASPClustering_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new DialogGRASPClustering_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new DialogGRASPClustering_jButtonHelp_actionAdapter(this));

    panelOptions.setLayout(xyLayoutOptions);
    panelOptions.setBorder(borderOptions);

    this.addWindowListener(new DialogGRASPClustering_this_windowAdapter(this));

    panel.add(buttonOK, new XYConstraints(20, 135, 90, 25));
    panel.add(buttonCancel, new XYConstraints(115, 135, 90, 25));
    panel.add(buttonHelp, new XYConstraints(210, 135, 90, 25));

    panel.add(panelOptions, new XYConstraints(10, 10, 300, 110));

    panelOptions.add(labelK, new XYConstraints(10, 10, 130, 25));
    panelOptions.add(labelI, new XYConstraints(10, 40, 130, 25));
    panelOptions.add(labelC, new XYConstraints(10, 70, 130, 25));

    panelOptions.add(textFieldK, new XYConstraints(145, 10, 138, 27));
    panelOptions.add(textFieldI, new XYConstraints(145, 40, 138, 27));
    panelOptions.add(textFieldC, new XYConstraints(145, 70, 138, 27));
  }

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    ClusteringAlgorithm clustering;
    Benchmark           crono;
    boolean             ok;
    String              error;
    String              param;

    // Parmetros

    ok = true;
    param = "";

    try {
        K = (new Integer(textFieldK.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "K";
    }

    if (K<2) {
       ok = false;
       param = "K";
    }

    try {
        I = (new Integer(textFieldI.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "I";
    }

    if (I<1) {
       ok = false;
       param = "I";
    }

    try {
        C = (new Integer(textFieldC.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "C";
    }

    if (C<1) {
       ok = false;
       param = "C";
    }


    if (!ok) {

       if (Environment.settings==Environment.ENGLISH) {
          error = "Incorrect parameter: "+param;
       } else {
          error = "Parmetro "+param+" incorrecto";
       }

       Message.showMessageDialog ( (Component)null,
                                   error,
                                   "Error",
                                   Message.ERROR_MESSAGE);

    } else {

       dispose();

       out.message ("K = "+K+" ; I = "+I+" ; C = "+C+"\n");

       // Aplicar algoritmo de clustering

       GUI.setWaitCursor(this);

       if (Environment.settings==Environment.SPANISH)
          out.message("Aplicacin del mtodo de agrupamiento... ");
       else
          out.message("Clustering data... ");

       crono = new Benchmark();
       crono.start();

       clustering = new GRASPClustering(cfg,data,dist,K,I,C);

       crono.stop();
       out.message(crono+"\n");

       GUI.setDefaultCursor(this);

       nc.showClustering(clustering,data,title);
    }
  }

  // Cancel

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }

  // Help

  void jButtonHelp_actionPerformed(ActionEvent e)
  {
    help.help("nc/clustering/GRASP.html");
  }
}

// Event Handlers
// --------------

class DialogGRASPClustering_buttonOK_actionAdapter implements ActionListener{
  DialogGRASPClustering adaptee;

  DialogGRASPClustering_buttonOK_actionAdapter(DialogGRASPClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class DialogGRASPClustering_buttonCancel_actionAdapter implements ActionListener{
  DialogGRASPClustering adaptee;

  DialogGRASPClustering_buttonCancel_actionAdapter(DialogGRASPClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class DialogGRASPClustering_this_windowAdapter extends WindowAdapter {
  DialogGRASPClustering adaptee;

  DialogGRASPClustering_this_windowAdapter(DialogGRASPClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class DialogGRASPClustering_jButtonHelp_actionAdapter implements java.awt.event.ActionListener
{
 DialogGRASPClustering adaptee;


 DialogGRASPClustering_jButtonHelp_actionAdapter(DialogGRASPClustering adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.jButtonHelp_actionPerformed(e);
 }
}
