package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.01
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import fbg.gui.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.distance.*;
import rf.clustering.*;

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

// ISODATA: Iterative Self-Organizing Techniques
// ---------------------------------------------
// K -> Nmero deseado de agrupamientos
// A -> Nmero inicial de clusters
// N -> Nmero mnimo de patrones en un agrupamiento
// s -> Dispersin mxima permitida en un cluster
// c -> Distancia mnima entre clusters
// L -> Nmero de de pares de agrupamientos que pueden mezclarse en una iteracin
// I -> Mximo nmero de iteraciones permitidas

public class DialogISODATA extends JDialog
{
  // Data

  DataConfiguration cfg  = null;
  Dataset           data = null;
  Distance          dist = null;

  // Algorithm

  String  title = null;

  // Algorithm parameters

  int   K = 5;
  int   A = 3;
  int   N = 10;
  float s = 3.0f;
  float c = 10.0f;
  int   L = 7;
  int   I = 2;

  // Main frame

  MainMenu nc = null;

  // Standard Output

  Output out = null;

  // Online help

  Help help = null;

  // Components

  JPanel panel = new JPanel();
  JPanel panelOptions = new JPanel();

  Border borderOptions = new BevelBorder(BevelBorder.RAISED);

  XYLayout xyLayout = new XYLayout();
  XYLayout xyLayoutOptions = new XYLayout();

  Label labelK = new Label();
  Label labelA = new Label();
  Label labelN = new Label();
  Label labels = new Label();
  Label labelc = new Label();
  Label labelL = new Label();
  Label labelI = new Label();

  JTextField textFieldK = new JTextField();
  JTextField textFieldA = new JTextField();
  JTextField textFieldN = new JTextField();
  JTextField textFields = new JTextField();
  JTextField textFieldc = new JTextField();
  JTextField textFieldL = new JTextField();
  JTextField textFieldI = new JTextField();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructors

  public DialogISODATA(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try {
      init();
      pack();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public DialogISODATA (DataConfiguration config, Dataset patterns, Distance dm,
                                   MainMenu frame, Output output, Help help_system)
  {
   this(frame, "", false);

   cfg  = config;
   data = patterns;
   dist = dm;

   nc   = frame;
   out  = output;
   help = help_system;

   // Language

   if (Environment.settings==Environment.ENGLISH) {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Run algorithm");
      buttonCancel.setText("Cancel");
      buttonCancel.setToolTipText("Back to menu");
      buttonHelp.setText("Help");
      buttonHelp.setToolTipText("Online help");

      title = "ISODATA";

      setTitle(title);

   } else {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Aplicar algoritmo");
      buttonCancel.setText("Cancelar");
      buttonCancel.setToolTipText("Volver al men");
      buttonHelp.setText("Ayuda");
      buttonHelp.setToolTipText("Ayuda en lnea");

      title = "Algoritmo ISODATA";

      setTitle(title);
   }

   labelK.setText("K");
   labelA.setText("A");
   labelN.setText("N");
   labels.setText("s");
   labelc.setText("c");
   labelL.setText("L");
   labelI.setText("I");

   textFieldK.setText(" "+K);
   textFieldA.setText(" "+A);
   textFieldN.setText(" "+N);
   textFields.setText(" "+s);
   textFieldc.setText(" "+c);
   textFieldL.setText(" "+L);
   textFieldI.setText(" "+I);
  }

  // GUI

  void init() throws Exception
  {
    panel.setBackground(Color.lightGray);
    panel.setLayout(xyLayout);
    this.getContentPane().add(panel);

    xyLayout.setWidth(320);
    xyLayout.setHeight(290);

    textFieldK.setBackground(Color.white);
    textFieldA.setBackground(Color.white);
    textFieldN.setBackground(Color.white);
    textFields.setBackground(Color.white);
    textFieldc.setBackground(Color.white);
    textFieldL.setBackground(Color.white);
    textFieldI.setBackground(Color.white);

    buttonOK.addActionListener(new DialogISODATA_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new DialogISODATA_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new DialogISODATA_jButtonHelp_actionAdapter(this));

    panelOptions.setLayout(xyLayoutOptions);
    panelOptions.setBorder(borderOptions);

    this.addWindowListener(new DialogISODATA_this_windowAdapter(this));

    panel.add(buttonOK, new XYConstraints(20, 255, 90, 25));
    panel.add(buttonCancel, new XYConstraints(115, 255, 90, 25));
    panel.add(buttonHelp, new XYConstraints(210, 255, 90, 25));

    panel.add(panelOptions, new XYConstraints(10, 10, 300, 230));

    panelOptions.add(labelK, new XYConstraints(10, 10, 130, 25));
    panelOptions.add(labelA, new XYConstraints(10, 40, 130, 25));
    panelOptions.add(labelN, new XYConstraints(10, 70, 130, 25));
    panelOptions.add(labels, new XYConstraints(10,100, 130, 25));
    panelOptions.add(labelc, new XYConstraints(10,130, 130, 25));
    panelOptions.add(labelL, new XYConstraints(10,160, 130, 25));
    panelOptions.add(labelI, new XYConstraints(10,190, 130, 25));

    panelOptions.add(textFieldK, new XYConstraints(145, 10, 138, 27));
    panelOptions.add(textFieldA, new XYConstraints(145, 40, 138, 27));
    panelOptions.add(textFieldN, new XYConstraints(145, 70, 138, 27));
    panelOptions.add(textFields, new XYConstraints(145,100, 138, 27));
    panelOptions.add(textFieldc, new XYConstraints(145,130, 138, 27));
    panelOptions.add(textFieldL, new XYConstraints(145,160, 138, 27));
    panelOptions.add(textFieldI, new XYConstraints(145,190, 138, 27));
  }

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    ClusteringAlgorithm clustering;
    Benchmark           crono;
    boolean             ok;
    String              error = null;;
    String              param = null;

    // Parmetros

    ok = true;

    try {
        K = (new Integer(textFieldK.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "K";
    }

    if (K<2) {
       ok = false;
       param = "K";
    }

    try {
        A = (new Integer(textFieldA.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "A";
    }

    if (A<1) {
       ok = false;
       param = "A";
    }

    try {
        N = (new Integer(textFieldN.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "N";
    }

    if (N<1) {
       ok = false;
       param = "N";
    }

    try {
        s = (new Float(textFields.getText().trim())).floatValue();
    } catch (Exception exception) {
        ok = false;
        param = "s";
    }

    if (s<0) {
       ok = false;
       param = "s";
    }

    try {
        c = (new Float(textFieldc.getText().trim())).floatValue();
    } catch (Exception exception) {
        ok = false;
        param = "c";
    }

    if (c<0) {
       ok = false;
       param = "c";
    }

    try {
        L = (new Integer(textFieldL.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "L";
    }

    if (L<2) {
       ok = false;
       param = "L";
    }

    try {
        I = (new Integer(textFieldI.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "I";
    }

    if (I<1) {
       ok = false;
       param = "I";
    }


    if (!ok) {

       if (Environment.settings==Environment.ENGLISH) {
          error = "Incorrect parameter: "+param;
       } else {
          error = "Parmetro "+param+" incorrecto";
       }

       Message.showMessageDialog ( (Component)null,
                                   error,
                                   "Error",
                                   Message.ERROR_MESSAGE);
    } else {

       dispose();

       out.message ("K = "+K+" ; A = "+A+" ; N = "+N+" ; c = "+c+" ; s = "+s+" ; L = "+L+" ; I = "+I+"\n");

       // Aplicar algoritmo de clustering

       GUI.setWaitCursor(this);

       if (Environment.settings==Environment.SPANISH)
          out.message("Aplicacin del mtodo de agrupamiento... ");
       else
          out.message("Clustering data... ");

       crono = new Benchmark();
       crono.start();

       clustering = new ISODATA(cfg,data,dist,K,A,N,s,c,L,I);

       crono.stop();
       out.message(crono+"\n");

       GUI.setDefaultCursor(this);

       nc.showClustering(clustering,data,title);
    }
  }

  // Cancel

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }

  // Help

  void jButtonHelp_actionPerformed(ActionEvent e)
  {
    help.help("nc/clustering/ISODATA.html");
  }
}

// Event Handlers
// --------------

class DialogISODATA_buttonOK_actionAdapter implements ActionListener{
  DialogISODATA adaptee;

  DialogISODATA_buttonOK_actionAdapter(DialogISODATA adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class DialogISODATA_buttonCancel_actionAdapter implements ActionListener{
  DialogISODATA adaptee;

  DialogISODATA_buttonCancel_actionAdapter(DialogISODATA adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class DialogISODATA_this_windowAdapter extends WindowAdapter {
  DialogISODATA adaptee;

  DialogISODATA_this_windowAdapter(DialogISODATA adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class DialogISODATA_jButtonHelp_actionAdapter implements java.awt.event.ActionListener
{
 DialogISODATA adaptee;


 DialogISODATA_jButtonHelp_actionAdapter(DialogISODATA adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.jButtonHelp_actionPerformed(e);
 }
}
