package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.01
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import fbg.gui.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.distance.*;
import rf.clustering.*;

// K-Means

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

public class DialogKMeans extends JDialog
{
  // Data

  DataConfiguration cfg  = null;
  Dataset           data = null;
  Distance          dist = null;

  // Algorithm

  String  title = null;

  // Algorithm parameters

  int K = 5;

  // Main frame

  MainMenu nc = null;

  // Standard Output

  Output out = null;

  // Online help

  Help help = null;

  // Components

  JPanel panel = new JPanel();
  JPanel panelOptions = new JPanel();

  Border borderOptions = new BevelBorder(BevelBorder.RAISED);

  XYLayout xyLayout = new XYLayout();
  XYLayout xyLayoutOptions = new XYLayout();

  Label labelK = new Label();

  JTextField textFieldK = new JTextField();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructors

  public DialogKMeans(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try {
      init();
      pack();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public DialogKMeans (DataConfiguration config, Dataset patterns, Distance dm,
                                   MainMenu frame, Output output, Help help_system)
  {
   this(frame, "", false);

   cfg  = config;
   data = patterns;
   dist = dm;

   nc   = frame;
   out  = output;
   help = help_system;

   // Language

   if (Environment.settings==Environment.ENGLISH) {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Run algorithm");
      buttonCancel.setText("Cancel");
      buttonCancel.setToolTipText("Back to menu");
      buttonHelp.setText("Help");
      buttonHelp.setToolTipText("Online help");

      title = "K-MEANS";

      setTitle(title);

      labelK.setText("K");

   } else {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Aplicar algoritmo");
      buttonCancel.setText("Cancelar");
      buttonCancel.setToolTipText("Volver al men");
      buttonHelp.setText("Ayuda");
      buttonHelp.setToolTipText("Ayuda en lnea");

      title = "Algoritmo de las K Medias";

      setTitle(title);

      labelK.setText("K");
   }

   textFieldK.setText(" "+K);
  }

  // GUI

  void init() throws Exception
  {
    panel.setBackground(Color.lightGray);
    panel.setLayout(xyLayout);
    this.getContentPane().add(panel);

    xyLayout.setWidth(320);
    xyLayout.setHeight(110);

    textFieldK.setBackground(Color.white);

    buttonOK.addActionListener(new DialogKMeans_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new DialogKMeans_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new DialogKMeans_jButtonHelp_actionAdapter(this));

    panelOptions.setLayout(xyLayoutOptions);
    panelOptions.setBorder(borderOptions);

    this.addWindowListener(new DialogKMeans_this_windowAdapter(this));

    panel.add(buttonOK, new XYConstraints(20, 75, 90, 25));
    panel.add(buttonCancel, new XYConstraints(115, 75, 90, 25));
    panel.add(buttonHelp, new XYConstraints(210, 75, 90, 25));

    panel.add(panelOptions, new XYConstraints(10, 10, 300, 50));

    panelOptions.add(labelK, new XYConstraints(10, 10, 130, 25));
    panelOptions.add(textFieldK, new XYConstraints(145, 10, 138, 27));
  }

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    ClusteringAlgorithm clustering;
    Benchmark           crono;
    boolean             ok;
    String              error;

    // Parmetros

    ok = true;

    try {

        K = (new Integer(textFieldK.getText().trim())).intValue();

    } catch (Exception exception) {

        ok = false;
    }

    if (!ok || K<2 ) {

       if (Environment.settings==Environment.ENGLISH) {
          error = "The parameter K must be an integer greater than one";
       } else {
          error = "El parmetro K debe ser un entero mayor que uno";
       }

       Message.showMessageDialog ( (Component)null,
                                   error,
                                   "Error",
                                   Message.ERROR_MESSAGE);
    } else {

       dispose();

       out.message ("K = "+K+"\n");

       // Aplicar algoritmo de clustering

       GUI.setWaitCursor(this);

       if (Environment.settings==Environment.SPANISH)
          out.message("Aplicacin del mtodo de agrupamiento... ");
       else
          out.message("Clustering data... ");

       crono = new Benchmark();
       crono.start();

       clustering = new kMeans(cfg,data,dist,K);

       crono.stop();
       out.message(crono+"\n");

       GUI.setDefaultCursor(this);

       nc.showClustering(clustering,data,title);
    }
  }

  // Cancel

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }

  // Help

  void jButtonHelp_actionPerformed(ActionEvent e)
  {
    help.help("nc/clustering/KMeans.html");
  }
}

// Event Handlers
// --------------

class DialogKMeans_buttonOK_actionAdapter implements ActionListener{
  DialogKMeans adaptee;

  DialogKMeans_buttonOK_actionAdapter(DialogKMeans adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class DialogKMeans_buttonCancel_actionAdapter implements ActionListener{
  DialogKMeans adaptee;

  DialogKMeans_buttonCancel_actionAdapter(DialogKMeans adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class DialogKMeans_this_windowAdapter extends WindowAdapter {
  DialogKMeans adaptee;

  DialogKMeans_this_windowAdapter(DialogKMeans adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class DialogKMeans_jButtonHelp_actionAdapter implements java.awt.event.ActionListener
{
 DialogKMeans adaptee;


 DialogKMeans_jButtonHelp_actionAdapter(DialogKMeans adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.jButtonHelp_actionPerformed(e);
 }
}
