package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.01
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import fbg.gui.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.distance.*;
import rf.clustering.*;

// Sequential Clustering

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

public class DialogSequentialClustering extends JDialog
{
  // Data

  DataConfiguration cfg  = null;
  Dataset           data = null;
  Distance          dist = null;

  // Algorithm

  String  title = null;

  // Algorithm parameters

  int     K     = 5;
  float   R     = 100f;
  float   C     = 10f;
  int     M     = 200;
  float   T     = 0.05f;

  // Main frame

  MainMenu nc = null;

  // Standard Output

  Output out = null;

  // Online help

  Help help = null;

  // Components

  JPanel panel = new JPanel();
  JPanel panelOptions = new JPanel();

  Border borderOptions = new BevelBorder(BevelBorder.RAISED);

  XYLayout xyLayout = new XYLayout();
  XYLayout xyLayoutOptions = new XYLayout();

  Label labelK = new Label();
  Label labelR = new Label();
  Label labelC = new Label();
  Label labelM = new Label();
  Label labelT = new Label();

  JTextField textFieldK = new JTextField();
  JTextField textFieldR = new JTextField();
  JTextField textFieldC = new JTextField();
  JTextField textFieldM = new JTextField();
  JTextField textFieldT = new JTextField();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructors

  public DialogSequentialClustering(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try {
      init();
      pack();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public DialogSequentialClustering (DataConfiguration config, Dataset patterns, Distance dm,
                                   MainMenu frame, Output output, Help help_system)
  {
   this(frame, "", false);

   cfg  = config;
   data = patterns;
   dist = dm;

   nc   = frame;
   out  = output;
   help = help_system;

   // Language

   if (Environment.settings==Environment.ENGLISH) {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Run algorithm");
      buttonCancel.setText("Cancel");
      buttonCancel.setToolTipText("Back to menu");
      buttonHelp.setText("Help");
      buttonHelp.setToolTipText("Online help");

      title = "Sequential Clustering";

      setTitle(title);

   } else {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Aplicar algoritmo");
      buttonCancel.setText("Cancelar");
      buttonCancel.setToolTipText("Volver al men");
      buttonHelp.setText("Ayuda");
      buttonHelp.setToolTipText("Ayuda en lnea");

      title = "Algoritmo de agrupamiento secuencial";

      setTitle(title);
   }

   labelK.setText("K");
   labelR.setText("R");
   labelC.setText("C");
   labelM.setText("M");
   labelT.setText("T");

   textFieldK.setText(" "+K);
   textFieldR.setText(" "+R);
   textFieldC.setText(" "+C);
   textFieldM.setText(" "+M);
   textFieldT.setText(" "+T);
  }

  // GUI

  void init() throws Exception
  {
    panel.setBackground(Color.lightGray);
    panel.setLayout(xyLayout);
    this.getContentPane().add(panel);

    xyLayout.setWidth(320);
    xyLayout.setHeight(230);

    textFieldK.setBackground(Color.white);
    textFieldR.setBackground(Color.white);
    textFieldC.setBackground(Color.white);
    textFieldM.setBackground(Color.white);
    textFieldT.setBackground(Color.white);

    buttonOK.addActionListener(new DialogSequentialClustering_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new DialogSequentialClustering_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new DialogSequentialClustering_jButtonHelp_actionAdapter(this));

    panelOptions.setLayout(xyLayoutOptions);
    panelOptions.setBorder(borderOptions);

    this.addWindowListener(new DialogSequentialClustering_this_windowAdapter(this));

    panel.add(buttonOK, new XYConstraints(20, 195, 90, 25));
    panel.add(buttonCancel, new XYConstraints(115, 195, 90, 25));
    panel.add(buttonHelp, new XYConstraints(210, 195, 90, 25));

    panel.add(panelOptions, new XYConstraints(10, 10, 300, 170));

    panelOptions.add(labelK, new XYConstraints(10, 10, 130, 25));
    panelOptions.add(labelR, new XYConstraints(10, 40, 130, 25));
    panelOptions.add(labelC, new XYConstraints(10, 70, 130, 25));
    panelOptions.add(labelM, new XYConstraints(10,100, 130, 25));
    panelOptions.add(labelT, new XYConstraints(10,130, 130, 25));

    panelOptions.add(textFieldK, new XYConstraints(145, 10, 138, 27));
    panelOptions.add(textFieldR, new XYConstraints(145, 40, 138, 27));
    panelOptions.add(textFieldC, new XYConstraints(145, 70, 138, 27));
    panelOptions.add(textFieldM, new XYConstraints(145,100, 138, 27));
    panelOptions.add(textFieldT, new XYConstraints(145,130, 138, 27));
  }

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    ClusteringAlgorithm clustering;
    Benchmark           crono;
    boolean             ok;
    String              error = null;;
    String              param = null;

    // Parmetros

    ok = true;

    try {
        K = (new Integer(textFieldK.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "K";
    }

    if (K<2) {
       ok = false;
       param = "K";
    }

    try {
        R = (new Float(textFieldR.getText().trim())).floatValue();
    } catch (Exception exception) {
        ok = false;
        param = "R";
    }

    if (R<0) {
       ok = false;
       param = "R";
    }

    try {
        C = (new Float(textFieldC.getText().trim())).floatValue();
    } catch (Exception exception) {
        ok = false;
        param = "C";
    }

    if (C<0) {
       ok = false;
       param = "C";
    }

    try {
        M = (new Integer(textFieldM.getText().trim())).intValue();
    } catch (Exception exception) {
        ok = false;
        param = "M";
    }

    if (M<1) {
       ok = false;
       param = "M";
    }

    try {
        T = (new Float(textFieldT.getText().trim())).floatValue();
    } catch (Exception exception) {
        ok = false;
        param = "T";
    }

    if (T<0.0f || T>1.0f) {
       ok = false;
       param = "T";
    }

    if (!ok) {

       if (Environment.settings==Environment.ENGLISH) {
          error = "Incorrect parameter: "+param;
       } else {
          error = "Parmetro "+param+" incorrecto";
       }

       Message.showMessageDialog ( (Component)null,
                                   error,
                                   "Error",
                                   Message.ERROR_MESSAGE);
    } else {

       dispose();

       out.message ("K = "+K+" ; R = "+R+" ; C = "+C+" ; M = "+M+" ; T = "+T+"\n");

       // Aplicar algoritmo de clustering

       GUI.setWaitCursor(this);

       if (Environment.settings==Environment.SPANISH)
          out.message("Aplicacin del mtodo de agrupamiento... ");
       else
          out.message("Clustering data... ");

       crono = new Benchmark();
       crono.start();

       clustering = new SequentialClustering(cfg,data,dist,K,R,C,M,T);

       crono.stop();
       out.message(crono+"\n");

       GUI.setDefaultCursor(this);

       nc.showClustering(clustering,data,title);
    }
  }

  // Cancel

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }

  // Help

  void jButtonHelp_actionPerformed(ActionEvent e)
  {
    help.help("nc/clustering/sequential.html");
  }
}

// Event Handlers
// --------------

class DialogSequentialClustering_buttonOK_actionAdapter implements ActionListener{
  DialogSequentialClustering adaptee;

  DialogSequentialClustering_buttonOK_actionAdapter(DialogSequentialClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class DialogSequentialClustering_buttonCancel_actionAdapter implements ActionListener{
  DialogSequentialClustering adaptee;

  DialogSequentialClustering_buttonCancel_actionAdapter(DialogSequentialClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class DialogSequentialClustering_this_windowAdapter extends WindowAdapter {
  DialogSequentialClustering adaptee;

  DialogSequentialClustering_this_windowAdapter(DialogSequentialClustering adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class DialogSequentialClustering_jButtonHelp_actionAdapter implements java.awt.event.ActionListener
{
 DialogSequentialClustering adaptee;


 DialogSequentialClustering_jButtonHelp_actionAdapter(DialogSequentialClustering adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.jButtonHelp_actionPerformed(e);
 }
}
