package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.01
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import fbg.gui.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.distance.*;
import rf.clustering.*;

// Graph Theoretic Clustering: Similarity Matrix

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

public class DialogSimilarityMatrix extends JDialog
{
  // Data

  DataConfiguration cfg  = null;
  Dataset           data = null;
  Distance          dist = null;

  // Algorithm

  String  title = null;

  // Algorithm parameters

  float   threshold = 1.0f;

  // Main frame

  MainMenu nc = null;

  // Standard Output

  Output out = null;

  // Online help

  Help help = null;

  // Components

  JPanel panel = new JPanel();
  JPanel panelOptions = new JPanel();

  Border borderOptions = new BevelBorder(BevelBorder.RAISED);

  XYLayout xyLayout = new XYLayout();
  XYLayout xyLayoutOptions = new XYLayout();

  Label labelK = new Label();

  JTextField textFieldK = new JTextField();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructors

  public DialogSimilarityMatrix (Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try {
      init();
      pack();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public DialogSimilarityMatrix (DataConfiguration config, Dataset patterns, Distance dm,
                                   MainMenu frame, Output output, Help help_system)
  {
   this(frame, "", false);

   cfg  = config;
   data = patterns;
   dist = dm;

   nc   = frame;
   out  = output;
   help = help_system;

   // Language

   if (Environment.settings==Environment.ENGLISH) {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Run algorithm");
      buttonCancel.setText("Cancel");
      buttonCancel.setToolTipText("Back to menu");
      buttonHelp.setText("Help");
      buttonHelp.setToolTipText("Online help");

      title = "Similarity Matrix";

      setTitle(title);

      labelK.setText("Distance Threshold");

   } else {

      buttonOK.setText("OK");
      buttonOK.setToolTipText("Aplicar algoritmo");
      buttonCancel.setText("Cancelar");
      buttonCancel.setToolTipText("Volver al men");
      buttonHelp.setText("Ayuda");
      buttonHelp.setToolTipText("Ayuda en lnea");

      title = "Matriz de similaridad";

      setTitle(title);

      labelK.setText("Umbral de distancia");
   }

   textFieldK.setText(" "+threshold);
  }

  // GUI

  void init() throws Exception
  {
    panel.setBackground(Color.lightGray);
    panel.setLayout(xyLayout);
    this.getContentPane().add(panel);

    xyLayout.setWidth(320);
    xyLayout.setHeight(110);

    textFieldK.setBackground(Color.white);

    buttonOK.addActionListener(new DialogSimilarityMatrix_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new DialogSimilarityMatrix_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new DialogSimilarityMatrix_jButtonHelp_actionAdapter(this));

    panelOptions.setLayout(xyLayoutOptions);
    panelOptions.setBorder(borderOptions);

    this.addWindowListener(new DialogSimilarityMatrix_this_windowAdapter(this));

    panel.add(buttonOK, new XYConstraints(20, 75, 90, 25));
    panel.add(buttonCancel, new XYConstraints(115, 75, 90, 25));
    panel.add(buttonHelp, new XYConstraints(210, 75, 90, 25));

    panel.add(panelOptions, new XYConstraints(10, 10, 300, 50));

    panelOptions.add(labelK, new XYConstraints(10, 10, 130, 25));
    panelOptions.add(textFieldK, new XYConstraints(145, 10, 138, 27));
  }

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    ClusteringAlgorithm clustering;
    Benchmark           crono;
    boolean             ok;
    String              error;

    // Parmetros

    ok = true;

    try {

        threshold = (new Float(textFieldK.getText().trim())).floatValue();

    } catch (Exception exception) {

        ok = false;
    }

    if (!ok || threshold<0 ) {

       if (Environment.settings==Environment.ENGLISH) {
          error = "The parameter must be a positive real number";
       } else {
          error = "El parmetro debe ser un nmero positivo";
       }

       Message.showMessageDialog ( (Component)null,
                                   error,
                                   "Error",
                                   Message.ERROR_MESSAGE);
    } else {

       dispose();

       out.message ("theta = "+threshold+"\n");

       // Aplicar algoritmo de clustering

       GUI.setWaitCursor(this);

       if (Environment.settings==Environment.SPANISH)
          out.message("Aplicacin del mtodo de agrupamiento... ");
       else
          out.message("Clustering data... ");

       crono = new Benchmark();
       crono.start();

       clustering = new SimilarityMatrix(cfg,data,dist,threshold);

       crono.stop();
       out.message(crono+"\n");

       GUI.setDefaultCursor(this);

       nc.showClustering(clustering,data,title);
    }
  }

  // Cancel

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }

  // Help

  void jButtonHelp_actionPerformed(ActionEvent e)
  {
    help.help("nc/clustering/SimilarityMatrix.html");
  }
}

// Event Handlers
// --------------

class DialogSimilarityMatrix_buttonOK_actionAdapter implements ActionListener{
  DialogSimilarityMatrix adaptee;

  DialogSimilarityMatrix_buttonOK_actionAdapter(DialogSimilarityMatrix adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class DialogSimilarityMatrix_buttonCancel_actionAdapter implements ActionListener{
  DialogSimilarityMatrix adaptee;

  DialogSimilarityMatrix_buttonCancel_actionAdapter(DialogSimilarityMatrix adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class DialogSimilarityMatrix_this_windowAdapter extends WindowAdapter {
  DialogSimilarityMatrix adaptee;

  DialogSimilarityMatrix_this_windowAdapter(DialogSimilarityMatrix adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class DialogSimilarityMatrix_jButtonHelp_actionAdapter implements java.awt.event.ActionListener
{
 DialogSimilarityMatrix adaptee;


 DialogSimilarityMatrix_jButtonHelp_actionAdapter(DialogSimilarityMatrix adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.jButtonHelp_actionPerformed(e);
 }
}
