package rf.gui;

// Title:       Numerical Cruncher
// Version:     1.1
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import fbg.gui.*;

import fbg.util.*;

import rf.DataConfiguration;
import rf.data.Dataset;


public class Frame2D extends Frame
{
  XYLayout xyLayout = new XYLayout();
  Chart2D  chart = null;
  Button   buttonOK = new Button();
  Label    labelX = new Label();
  Label    labelY = new Label();
  Choice   choiceX = new Choice();
  Choice   choiceY = new Choice();

  // Constructors

  public Frame2D (DataConfiguration cfg, Dataset set)
  {
    chart = new Chart2D(cfg,set,0,0);

    GUI();
    lists(cfg);
  }

  public Frame2D (DataConfiguration cfg, Dataset set, Color colors[])
  {
    chart = new Chart2D(cfg,set,0,0,colors);

    GUI();
    lists(cfg);
  }

  // Attribute lists

  private void lists (DataConfiguration cfg)
  {
    int    i;
    String id;

    for (i=0; i<cfg.totalAttributes(); i++) {
        id = cfg.getAttribute(i);
        choiceX.add(id);
        choiceY.add(id);
    }

    if (Environment.settings!=Environment.SPANISH) {
       this.setTitle("2D Plot");
       labelX.setText("X axis");
       labelY.setText("Y axis");
    }
  }

  // GUI

  private void GUI()
  {
   Image icon = Loader.loadImage(getClass(),"image/2d.gif");

   this.setIconImage(icon);
   this.setTitle("Representacin 2D");
   this.addWindowListener(new Frame2D_this_windowAdapter(this));
   xyLayout.setHeight(355);
   xyLayout.setWidth(225);
   this.setBackground(Color.lightGray);
   this.setSize( new Dimension(225,355));
   this.setLayout(xyLayout);
   buttonOK.setLabel("OK");
   buttonOK.addActionListener(new Frame2D_buttonOK_actionAdapter(this));
   labelX.setText("Eje X");
   labelY.setText("Eje Y");
   choiceY.addItemListener(new Frame2D_choiceY_itemAdapter(this));
   choiceX.addItemListener(new Frame2D_choiceX_itemAdapter(this));

   this.add(chart,    new XYConstraints(10, 10, 200, 200));
   this.add(labelX,   new XYConstraints(10, 225, 50, 20));
   this.add(labelY,   new XYConstraints(10, 260, 50, 20));
   this.add(choiceX,  new XYConstraints(60, 225, 150, 20));
   this.add(choiceY,  new XYConstraints(60, 260, 150, 20));
   this.add(buttonOK, new XYConstraints(60, 295, 150, 25));
  }

  // Events

  void choiceX_itemStateChanged(ItemEvent e)
  {
    chart.setX(choiceX.getSelectedIndex());
    chart.repaint();
  }

  void choiceY_itemStateChanged(ItemEvent e)
  {
    chart.setY(choiceY.getSelectedIndex());
    chart.repaint();
  }

  void buttonOK_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void this_windowClosing(WindowEvent e)
  {
    dispose();
  }
}

// Event handlers
// --------------

class Frame2D_choiceX_itemAdapter implements java.awt.event.ItemListener
{
 Frame2D adaptee;

 Frame2D_choiceX_itemAdapter(Frame2D adaptee)
 {
  this.adaptee = adaptee;
 }

 public void itemStateChanged(ItemEvent e)
 {
  adaptee.choiceX_itemStateChanged(e);
 }
}

class Frame2D_choiceY_itemAdapter implements java.awt.event.ItemListener
{
 Frame2D adaptee;

 Frame2D_choiceY_itemAdapter(Frame2D adaptee)
 {
  this.adaptee = adaptee;
 }

 public void itemStateChanged(ItemEvent e)
 {
  adaptee.choiceY_itemStateChanged(e);
 }
}

class Frame2D_buttonOK_actionAdapter implements java.awt.event.ActionListener
{
 Frame2D adaptee;

 Frame2D_buttonOK_actionAdapter(Frame2D adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.buttonOK_actionPerformed(e);
 }
}

class Frame2D_this_windowAdapter extends java.awt.event.WindowAdapter
{
 Frame2D adaptee;

 Frame2D_this_windowAdapter(Frame2D adaptee)
 {
  this.adaptee = adaptee;
 }

 public void windowClosing(WindowEvent e)
 {
  adaptee.this_windowClosing(e);
 }
}

