package rf.gui;

// Title:       Numerical Cruncher
// Version:     1.3
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.JButton;

import fbg.gui.GUI;

import fbg.util.*;

import rf.*;
import rf.data.*;

// Class stats
// -----------

public class FrameClasses extends Frame
{
  // Data

  DataConfiguration cfg;
  Dataset           all;
  int               labels[];
  TableDataset      set[];
  Statistics        stats[];

  // Components
  
  Panel panelClass = new Panel();
  Panel panelAttr = new Panel();
  Panel panelCovariance = new Panel();
  Panel panelButtons = new Panel();

  GridLayout gridLayoutframe = new GridLayout();

  Label labelClass = new Label();
  Label labelAttr = new Label();
  Label labelRango = new Label();
  Label labelAvg = new Label();
  Label labelVar = new Label();
  Label labelCov = new Label();
  Label labelCovAux = new Label();
  Label labelPatterns = new Label();

  Choice choiceClass = new Choice();
  Choice choiceAttr  = new Choice();
  Choice choiceCov   = new Choice();

  JButton buttonImage = new JButton();
  JButton buttonView  = new JButton();
  JButton buttonOK    = new JButton();

  // Constructor
  // -----------

  public FrameClasses (DataConfiguration cfg, Dataset all)
  {
    int    i,k,c;
    int    noc,noa,N;
    String label,id;
    String classes[];

    // Class distribution

    this.cfg = cfg;
    this.all = all;

    noc     = cfg.totalClasses();
    noa     = cfg.totalAttributes();
    classes = cfg.getClasses();
    N       = all.instances();

    set    = new TableDataset[noc];
    stats  = new Statistics[noc];
    labels = new int[N];


    for (c=0; c<noc; c++)
	set[c] = new TableDataset(cfg.getData());

    for (k=0; k<N; k++) {

	label = all.label (k);

	for (c=0; c<noc && !label.equals(classes[c]); c++)
	    ;

	if (c<noc)
	   set[c].add ( all.pattern(k) );

	labels[k] = c;
    }

    for (c=0; c<noc; c++)
	stats[c] = new Statistics(set[c]);

    // GUI (Interfaz grfica)

    try {
      jbInit();
      this.setTitle ("Estadsticas por clases");
    } catch (Exception e) {
      e.printStackTrace();
    }

    for (i=0; i<noa; i++) {
	id = cfg.getAttribute(i);
	choiceAttr.add (id);
	choiceCov.add (id);
    }

    for (i=0; i<noc; i++)
	choiceClass.add ( cfg.getClass(i) );

    //  Image data ?

    if (cfg.getFormat()==DataConfiguration.IMAGE_DATA) {
       buttonImage.setVisible(true);
    } else {
       buttonImage.setVisible(false);
    }

    // Language

    if (Environment.settings!=Environment.SPANISH) {
       this.setTitle ("Class statistics");
       labelAttr.setText("Attribute");
       labelRango.setText("Range = [XXX, XXX]");
       labelAvg.setText("Mean value = XXX");
       labelVar.setText("Variance = XXX");
       labelCovAux.setText("Covariance with");
       labelClass.setText("Class");
       labelPatterns.setText("XXX patterns");
       buttonView.setText("View scatter graph");
       buttonImage.setText("View image");
    }

    // Stats

    showStatistics(0,0,0);
  }

  private void jbInit() throws Exception
  {
   Image icon = Loader.loadImage(getClass(),"image/list.gif");

   this.setIconImage(icon);
   this.setBackground(Color.lightGray);
   this.setSize(new Dimension(450, 300));
   this.setLayout(gridLayoutframe);
   this.addWindowListener(new FrameClasses_windowAdapter(this));

   labelAttr.setForeground(Color.blue);
   labelAttr.setFont(new Font("Dialog", 3, 12));
   labelAttr.setAlignment(1);
   labelAttr.setText("Atributo");
   labelRango.setAlignment(1);
   labelRango.setText("Rango = [XXX, XXX]");
   labelAvg.setAlignment(1);
   labelAvg.setText("Media = XXX");
   labelVar.setAlignment(1);
   labelVar.setText("Varianza = XXX");
   labelCov.setAlignment(1);
   labelCov.setText("= COV");
   labelCovAux.setText("Covarianza con");
   labelClass.setFont(new Font("Dialog", 3, 12));
   labelClass.setText("Clase");
   labelPatterns.setAlignment(1);
   labelPatterns.setText("XXX patrones");

   choiceClass.addItemListener(new FrameClasses_choiceAdapter(this));
   choiceCov.addItemListener(new FrameClasses_choiceAdapter(this));
   choiceAttr.addItemListener(new FrameClasses_choiceAdapter(this));

   buttonImage.setText("Ver imagen");
   buttonImage.addActionListener(new FrameClasses_buttonImageAdapter(this));
   buttonView.setText("Ver nube de puntos");
   buttonView.addActionListener(new FrameClasses_buttonViewAdapter(this));
   buttonOK.setText("OK");
   buttonOK.addActionListener(new FrameClasses_buttonAdapter(this));

   gridLayoutframe.setRows(8);
   gridLayoutframe.setColumns(1);

   this.add(panelClass, null);
   panelClass.add(labelClass, null);
   panelClass.add(choiceClass, null);
   this.add(labelPatterns, null);
   this.add(panelAttr, null);
   panelAttr.add(labelAttr, null);
   panelAttr.add(choiceAttr, null);
   this.add(labelRango, null);
   this.add(labelAvg, null);
   this.add(labelVar, null);
   this.add(panelCovariance, null);
   panelCovariance.add(labelCovAux, null);
   panelCovariance.add(choiceCov, null);
   panelCovariance.add(labelCov, null);
   this.add(panelButtons, null);
   panelButtons.add(buttonImage, null);
   panelButtons.add(buttonView, null);
   panelButtons.add(buttonOK, null);
  }

  // Stats visualization
  // -------------------

  void showStatistics (int clase, int n, int c)
  {
    int pct  = (100*set[clase].instances())/all.instances();
    int pctd = (1000*set[clase].instances())/all.instances() - 10*pct;

    if (Environment.settings==Environment.SPANISH) {
       labelPatterns.setText( set[clase].instances() +" de "+ all.instances() + " patrones ("+pct+"."+pctd+"%)");
       labelRango.setText( "Rango = ["+stats[clase].min(n)+","+stats[clase].max(n)+"]");
       labelAvg.setText( "Media = " + stats[clase].mean(n) );
       labelVar.setText( "Varianza = " + stats[clase].variance(n) );
    } else {
       labelPatterns.setText( set[clase].instances() +" out of "+ all.instances() + " patterns ("+pct+"."+pctd+"%)");
       labelRango.setText( "Range = ["+stats[clase].min(n)+","+stats[clase].max(n)+"]");
       labelAvg.setText( "Mean value = " + stats[clase].mean(n) );
       labelVar.setText( "Variance = " + stats[clase].variance(n) );
    }

    labelCov.setText( "= " + stats[clase].covariance(n,c) );
  }

  // Events
  // ------

  void stateChanged ()
  {
    showStatistics( choiceClass.getSelectedIndex(),
		    choiceAttr.getSelectedIndex(),
		    choiceCov.getSelectedIndex() );
  }

  void exit ()
  {
    dispose();
  }

  // Visualizacin de los datos de la clase actual

  void view ()
  {
    int       i;
    int       selected = choiceClass.getSelectedIndex();
    Dataset   patterns = all;
    Color     colors[] = new Color[patterns.instances()];
    Dimension dim      = Toolkit.getDefaultToolkit().getScreenSize();
    Frame2D   frame2D  = null;

    for (i=0; i<patterns.instances(); i++)
        if (labels[i]==selected)
	   colors[i] = Color.red;
	else
	   colors[i] = Color.blue;
    
    frame2D = new Frame2D (cfg, patterns, colors );

    frame2D.setLocation((dim.width - frame2D.getSize().width) / 2, (dim.height - frame2D.getSize().height) / 2);
    frame2D.setVisible(true);
  }

  // Ver imagen


  void viewImage ()
  {
    FrameImage frame;
    Dimension  dim  = Toolkit.getDefaultToolkit().getScreenSize();
    ImageData  data = (ImageData) cfg.getData();
    Image      image;
    String     label;
    String     selected = choiceClass.getSelectedItem();
    int        anchura = data.getWidth();
    int        altura = data.getHeight();
    int        pixels[] = new int[anchura*altura];
    int        x,y,ox;
    int        fg = Color.white.getRGB();
    int        bg = Color.darkGray.getRGB();

    GUI.setWaitCursor(this);

    for (x=0; x<anchura; x++) {

        ox = x*anchura;

        for (y=0; y<altura; y++) {

	    label = data.getPattern(x,y).label();

	    if (label!=null) {

	       if (label.equals(selected))
	          pixels[ox+y] = fg;
	       else
	          pixels[ox+y] = bg;

	    } else {
               pixels[ox+y] = bg;
	    }
        }
    }

    // pixels -> Image

    image = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(anchura, altura, pixels, 0, anchura));

    GUI.setDefaultCursor(this);

    // Visualization

    frame = new FrameImage ( image, selected, 300);

    frame.setLocation( (dim.width - frame.getSize().width) / 2,
                       (dim.height - frame.getSize().height) / 2 );

    frame.setVisible(true);
  }

}

// Event handlers
// --------------


class FrameClasses_windowAdapter extends java.awt.event.WindowAdapter
{
 FrameClasses adaptee;

 FrameClasses_windowAdapter(FrameClasses adaptee)
 {
  this.adaptee = adaptee;
 }

 public void windowClosing(WindowEvent e)
 {
  adaptee.exit();
 }
}

class FrameClasses_buttonAdapter implements java.awt.event.ActionListener
{
 FrameClasses adaptee;

 FrameClasses_buttonAdapter(FrameClasses adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.exit();
 }
}

class FrameClasses_buttonViewAdapter implements java.awt.event.ActionListener
{
 FrameClasses adaptee;

 FrameClasses_buttonViewAdapter(FrameClasses adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.view();
 }
}

class FrameClasses_buttonImageAdapter implements java.awt.event.ActionListener
{
 FrameClasses adaptee;

 FrameClasses_buttonImageAdapter(FrameClasses adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.viewImage();
 }
}

class FrameClasses_choiceAdapter implements java.awt.event.ItemListener
{
 FrameClasses adaptee;

 FrameClasses_choiceAdapter(FrameClasses adaptee)
 {
  this.adaptee = adaptee;
 }

 public void itemStateChanged(ItemEvent e)
 {
  adaptee.stateChanged();
 }
}
