package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.1
// Copyright:   January 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.border.*;

import fbg.gui.*;
import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.classification.*;


// FRAME
// -----

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Jan'99  JDK 1.2

public class FrameClassifier extends Frame implements LayoutManager
{
  // Data

  DataConfiguration cfg;
  Classifier        classifier;

  // I/O

  Output  out        = null;
  boolean standalone = true;

  // Components

  FrameClassifier_ContingencyTable table;

  JScrollPane      tableAggregate;
  JTable           tableView;

  JButton          buttonOK   = new JButton();
  JButton          buttonView = new JButton();

  Label            labelTest = new Label();
  Label            labelLearning = new Label();
  Label            labelPct = new Label();

  // Constructor
  // -----------

  public FrameClassifier ( DataConfiguration cfg,
                           Classifier        classifier,
                           Dataset           learningSet,
                           Dataset           testSet,
                           String            title,
                           Output            out,
			   boolean           standalone)
  {
    int test[][] = classifier.evaluation(testSet);
    int i,ok,total;

    this.classifier = classifier;
    this.cfg        = cfg;
    this.out        = out;
    this.standalone = standalone;

    table = new FrameClassifier_ContingencyTable(test,cfg.getClasses());

    initGUI();

    this.setTitle(title);

    //  Image data ?

    if (cfg.getFormat()==DataConfiguration.IMAGE_DATA) {
       buttonView.setVisible(true);
    } else {
       buttonView.setVisible(false);
    }

    // Language

    buttonOK.setText("OK");

    if (Environment.settings==Environment.SPANISH) {
       labelLearning.setText("Clasificador construido a partir de "+learningSet.instances()+" patrones");
       labelTest.setText("Clasificador evaluado utilizando "+testSet.instances()+" patrones");
       buttonView.setText("Ver");
    } else {
       labelLearning.setText("Classifier built using "+learningSet.instances()+" patterns");
       labelTest.setText("Classifier tested with "+testSet.instances()+" patterns");
       buttonView.setText("View");
    }

    ok    = 0;
    total = testSet.instances();

    for (i=0; i<classifier.noc; i++) {
        ok += test[i][i];
    }

    if (Environment.settings==Environment.SPANISH) {
       labelPct.setText ("Bondad estimada del clasificador: "+ pct((float)ok/(float)total) + " ("+ok+"/"+total+")");
    } else {
       labelPct.setText ("Classifier accuracy: "+ pct((float)ok/(float)total) + " ("+ok+"/"+total+")");
    }
  }

  private String pct (float f)
  {
    float p = 100*f;
    int   n = (int) p;
    int   d = (int) ((p-n)*10);
    int   c = (int) (((p-n)*100)%10);

    if (n<10)
       return "  "+n+"."+d+c+"%";
    else if (n<100)
       return " "+n+"."+d+c+"%";
    else
       return n+"."+d+c+"%";
  }


  private void initGUI()
  {
   Image icon = Loader.loadImage(getClass(),"image/table.gif");

   this.setIconImage(icon);
   this.setLayout(this);
   this.addWindowListener(new FrameClassifier_windowAdapter(this));
   this.setBackground(Color.lightGray);
   this.setSize(new Dimension(500, 300));

   labelLearning.setAlignment(Label.CENTER);
   labelTest.setAlignment(Label.CENTER);
   labelPct.setAlignment(Label.LEFT);

   buttonView.addActionListener(new FrameClassifier_buttonView_actionAdapter(this));
   buttonOK.addActionListener(new FrameClassifier_buttonOK_actionAdapter(this));

   tableView = new JTable(table);

   tableAggregate = new JScrollPane(tableView);
   tableAggregate.setBackground(new Color(224, 224, 224));
   tableAggregate.setBorder(new BevelBorder(BevelBorder.LOWERED));

   this.add(labelLearning, null);
   this.add(labelTest, null);
   this.add(labelPct, null);
   this.add(buttonView, null);
   this.add(buttonOK, null);
   this.add(tableAggregate, null);
  }

  // Events

  void exit ()
  {
    dispose();
  }

  void view ()
  {
    FrameImage frame;
    Dimension  dim  = Toolkit.getDefaultToolkit().getScreenSize();
    ImageData  data = (ImageData) cfg.getData();
    Image      image;
    Pattern    pattern;
    int        clase;
    int        anchura = data.getWidth();
    int        altura = data.getHeight();
    int        pixels[] = new int[anchura*altura];
    int        x,y,ox;
    Benchmark  crono = new Benchmark();

    // Classification

    GUI.setWaitCursor(this);

    crono.start();

    if (Environment.settings==Environment.ENGLISH) {
       out.message ("Classification in progress... ");
    } else {
       out.message ("Clasificando muestras... ");
    }

    for (x=0; x<anchura; x++) {

        ox = x*anchura;

        for (y=0; y<altura; y++) {
            pattern = data.getPattern(x,y);
            clase   = classifier.classify(pattern);

            if (clase>=0)
               pixels[ox+y] = ColorMap.color[ clase%ColorMap.size ].getRGB();
        }
    }

    // Result -> Image

    image = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(anchura, altura, pixels, 0, anchura));

    crono.stop();

    out.message(crono+"\n");

    GUI.setDefaultCursor(this);

    // Visualization

    frame = new FrameImage ( image, getTitle(), 300);

    frame.setLocation( (dim.width - frame.getSize().width) / 2,
                       (dim.height - frame.getSize().height) / 2 );

    frame.setVisible(true);
  }

  // LAYOUT

  Dimension origin = new Dimension(450, 300);

  public Dimension preferredLayoutSize (Container c)
  {
    return origin;
  }

  public Dimension minimumLayoutSize (Container c)
  {
    return origin;
  }

  public void addLayoutComponent (String s, Component c)
  {
  }

  public void removeLayoutComponent(Component c)
  {
  }

  public void layoutContainer(Container c)
  {
    Rectangle b = c.getBounds();

    int width        = b.width;
    int height       = b.height;
    int inset        = 10;
    int labelHeight  = 15;
    int buttonWidth  = 60;
    int buttonHeight = 25;
    int title        = 20;
    int warning      = ( standalone ? 0: 20 );  // applet ?

    height -= (title+warning);

    labelLearning.setBounds ( inset,
			      title+inset,
			      width-2*inset,
			      labelHeight);

    labelTest.setBounds ( inset,
			  title+inset+labelHeight,
			  width-2*inset,
			  labelHeight);

    tableAggregate.setBounds ( inset,
		               title+2*(inset+labelHeight),
			       width-2*inset,
			       height - (4*inset+2*labelHeight+buttonHeight) );

    buttonOK.setBounds ( width-(buttonWidth+inset),
		         title+height-(inset+buttonHeight),
		         buttonWidth,
			 buttonHeight );

    buttonView.setBounds ( width-2*(buttonWidth+inset),
		           title+height-(inset+buttonHeight),
		           buttonWidth,
			   buttonHeight );

    labelPct.setBounds ( inset,
			 title+height-(inset+20),
			 width-2*(2*inset+buttonWidth),
			 labelHeight);
  }
}

// Event handling
// --------------

class FrameClassifier_windowAdapter extends java.awt.event.WindowAdapter
{
 FrameClassifier adaptee;

 FrameClassifier_windowAdapter(FrameClassifier adaptee)
 {
  this.adaptee = adaptee;
 }

 public void windowClosing(WindowEvent e)
 {
  adaptee.exit();
 }
}


class FrameClassifier_buttonOK_actionAdapter implements java.awt.event.ActionListener
{
 FrameClassifier adaptee;

 FrameClassifier_buttonOK_actionAdapter(FrameClassifier adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.exit();
 }
}

class FrameClassifier_buttonView_actionAdapter implements java.awt.event.ActionListener
{
 FrameClassifier adaptee;

 FrameClassifier_buttonView_actionAdapter(FrameClassifier adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.view();
 }
}

// Adapter: Classifier evaluation <---> TableModel interface
// ---------------------------------------------------------

class FrameClassifier_ContingencyTable extends AbstractTableModel implements TableModelListener
{
  int      noc;
  String   columnID[];
  int      data[][];
  String   pct[];
  
  // Constructor
  
  public FrameClassifier_ContingencyTable (int test[][], String id[])
  {
    int column;
    
    // Data
    
    noc  = test.length;
    data = test;
    
    // Header

    columnID = new String[noc+2];
    
    if (Environment.settings==Environment.SPANISH)
       columnID[0] = "Clase";
    else
       columnID[0] = "Class";
       
    for (column=0; column<noc; column++)
        columnID[column+1] = id[column];
        
    columnID[noc+1] = "%";
    
    // Classification percentages
    
    int   i,j;
    int   row;

    pct = new String[noc];

    for (i=0; i<noc; i++) {      // Instancias de la clase i

        row = 0;

        for (j=0; j<noc; j++)
            row += test[i][j];

        pct[i] = pct((float)test[i][i]/(float)row);
    }

    // Tell the listeners a new table has arrived

    fireTableChanged(null);
  }

  private String pct (float f)
  {
    float p = 100*f;
    int   n = (int) p;
    int   d = (int) ((p-n)*10);
    int   c = (int) (((p-n)*100)%10);

    if (n<10)
       return "  "+n+"."+d+c+"%";
    else if (n<100)
       return " "+n+"."+d+c+"%";
    else
       return n+"."+d+c+"%";
  }

  // TableModel Interface
  // --------------------
  
  // MetaData
  
  public String getColumnName (int column)
  {
    if (columnID[column] != null)
       return columnID[column];
    else
       return "";
  }

  public Class getColumnClass(int column)
  {
    if (column>0 && column<=noc)
       return Integer.class;
    else
       return String.class;
  }

  public boolean isCellEditable(int row, int column)
  {
    return false;
  }

  public int getColumnCount()
  {
    return noc+2;
  }

  public int getRowCount()
  {
    return noc;
  }

  public Object getValueAt (int row, int column)
  {
    if (column==0)
       return columnID[row+1];
    else if (column<=noc)
       return new Integer(data[row][column-1]);
    else if (column==noc+1)
       return pct[row];
    else
       return null;
  }

  public void setValueAt(Object value, int row, int column)
  {
    // Not supported
  }

  // Implementation of the TableModelListener interface:
  // By default forward all events to all the listeners.

  public void tableChanged(TableModelEvent e)
  {
    fireTableChanged(e);
  }
}
