package rf.gui;

// Title:       Numerical Cruncher
// Version:     2.1
// Copyright:   January 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import fbg.gui.*;
import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.clustering.*;

import tminer.TMiner;

// Clustering results
// ------------------

// v1.0  Jan'99  JDK 1.1 & SWING 1.0.1
// v2.0  Feb'99  JDK 1.2

public class FrameClusters extends Frame
{
  ClusteringAlgorithm clusters;
  Dataset             patterns;
  DataConfiguration   config;

  boolean             classification;
  boolean             standalone;

  // I/O

  Output out    = null;
  Help   help   = null;
  TMiner tminer = null;

  // Componentes

  Panel panelTop = new Panel();
  Panel panelButtons = new Panel();
  Panel panelClusters = new Panel();
  Panel panelInfo = new Panel();
  Panel panelAttribute = new Panel();
  Panel panelClass = new Panel();

  BorderLayout        borderLayoutFrame = new BorderLayout();
  GridLayout          gridLayoutTop = new GridLayout();
  GridLayout          gridLayoutInfo = new GridLayout();
  GridLayout          gridLayoutAttr = new GridLayout();
  GridLayout          gridLayoutClass = new GridLayout();
  FrameClustersLayout layoutClusters = new FrameClustersLayout(this);

  Label labelTitle = new Label();
  Label labelClusters = new Label();
  Label labelSelected = new Label();
  Label labelN = new Label();
  Label labelDispersion = new Label();
  Label labelAttribute = new Label();
  Label labelAvg = new Label();
  Label labelVar = new Label();
  Label labelClass = new Label();
  Label labelNClass = new Label();

  List listClusters = new List();

  Choice choiceAttribute = new Choice();
  Choice choiceClass     = new Choice();

  JButton buttonClassifier = new JButton();
  JButton buttonTMiner     = new JButton();
  JButton buttonView       = new JButton();
  JButton buttonOK         = new JButton();
  JButton buttonHelp       = new JButton();


  // Constructor

  public FrameClusters ( DataConfiguration   cfg,
                         ClusteringAlgorithm clustering,
                         Dataset             data,
                         String              label,
                         Output              output,
                         Help                help_system,
			 TMiner              miner,
			 boolean             application)
  {
    int    i,c;
    String prefix,id;

    clusters       = clustering;
    patterns       = data;
    config         = cfg;
    classification = (cfg.getClasses()!=null);
    standalone     = application;
    tminer         = miner;

    out      = output;
    help     = help_system;

    this.setTitle(label);

    initGUI();

    // Language

    if (Environment.settings==Environment.ENGLISH) {
       labelTitle.setText("Clustering algorithm results");
       labelClusters.setText(clustering.clusters()+" clusters obtained");
       labelAttribute.setText("Attribute");
       buttonOK.setText("OK");
       buttonOK.setToolTipText("Close window");
       buttonView.setText("View");
       buttonView.setToolTipText("View selected clusters data");
       buttonTMiner.setText("TMiner");
       buttonTMiner.setToolTipText("Analyze selected clusters");
       buttonClassifier.setText("Associated classifier");
       buttonClassifier.setToolTipText("Test associated classifier");
       buttonHelp.setText("Help");
       buttonHelp.setToolTipText("On-line help");
    } else {
       labelTitle.setText("Resultados del algoritmo de agrupamiento");
       labelClusters.setText(clustering.clusters()+" agrupamientos obtenidos");
       labelAttribute.setText("Atributo");
       buttonOK.setText("OK");
       buttonOK.setToolTipText("Cerrar ventana");
       buttonView.setText("Visualizar");
       buttonView.setToolTipText("Visualizar datos de los agrupamientos seleccionados");
       buttonTMiner.setText("TMiner");
       buttonTMiner.setToolTipText("Analizar agrupamientos seleccionados");
       buttonClassifier.setText("Clasificador asociado");
       buttonClassifier.setToolTipText("Clasificador asociado");
       buttonHelp.setText("Ayuda");
       buttonHelp.setToolTipText("Ayuda en lnea");
    }

    // Clasificador disponible ?

    if (classification) {
       buttonClassifier.setVisible(true);
    } else {
       buttonClassifier.setVisible(false);
    }

    // Conexin con TMiner

    // if (miner!=null)
    //    buttonTMiner.setVisible(true);
    // else
       buttonTMiner.setVisible(false);

    // Sin clusters seleccionados

    buttonTMiner.setEnabled(false);
    buttonView.setEnabled(false);

    // Lista de clusters

    if (Environment.settings==Environment.ENGLISH)
       prefix = "Cluster ";
    else
       prefix = "Agrupamiento ";

    for (c=1; c<=clustering.clusters(); c++)
        listClusters.add(prefix+c);

    // Lista de atributos

    for (i=0; i<cfg.totalAttributes(); i++) {
        id = cfg.getAttribute(i);
        choiceAttribute.add (id);
    }

    // Lista de clases

    if (classification)
       for (c=0; c<config.getClasses().length; c++)
           choiceClass.add (config.getClass(c));

    // Estadsticas

    showStats();
  }


  private void initGUI()
  {
    Image icon = Loader.loadImage(getClass(),"image/set.gif");

    this.setIconImage(icon);
    this.setSize(new Dimension(500, 300));
    this.setBackground(Color.lightGray);
    this.addWindowListener(new FrameClusters_this_windowAdapter(this));

    gridLayoutTop.setRows(2);
    gridLayoutTop.setColumns(1);
    borderLayoutFrame.setVgap(5);
    borderLayoutFrame.setHgap(5);
    gridLayoutInfo.setRows(8);
    gridLayoutInfo.setColumns(1);
    gridLayoutClass.setColumns(2);

    labelTitle.setForeground(Color.blue);
    labelTitle.setFont(new Font("Dialog", Font.BOLD, 12));
    labelTitle.setAlignment(1);
    labelClusters.setFont(new Font("Dialog", Font.ITALIC, 12));
    labelClusters.setAlignment(1);
    labelSelected.setForeground(Color.red);
    labelSelected.setAlignment(1);
    labelN.setAlignment(1);
    labelDispersion.setAlignment(1);
    labelAttribute.setForeground(Color.blue);
    labelAttribute.setAlignment(1);
    labelAvg.setAlignment(1);
    labelVar.setAlignment(1);
    labelClass.setForeground(new Color(0, 128, 0));
    labelClass.setAlignment(1);
    labelNClass.setAlignment(1);

    listClusters.setMultipleMode(true);
    listClusters.setBackground(new Color(224, 224, 224));

    choiceAttribute.addItemListener(new FrameClusters_choiceAttribute_itemAdapter(this));
    choiceClass.addItemListener(new FrameClusters_choiceClass_itemAdapter(this));
    listClusters.addItemListener(new FrameClusters_listClusters_itemAdapter(this));

    buttonClassifier.addActionListener(new FrameClusters_buttonClassifier_actionAdapter(this));
    buttonTMiner.addActionListener(new FrameClusters_buttonTMiner_actionAdapter(this));
    buttonView.addActionListener(new FrameClusters_buttonView_actionAdapter(this));
    buttonOK.addActionListener(new FrameClusters_buttonOK_actionAdapter(this));
    buttonHelp.addActionListener(new FrameClusters_buttonHelp_actionAdapter(this));

    panelInfo.setLayout(gridLayoutInfo);
    panelAttribute.setLayout(gridLayoutAttr);
    panelClass.setLayout(gridLayoutClass);
    panelClusters.setLayout(layoutClusters);
    panelTop.setLayout(gridLayoutTop);
    this.setLayout(borderLayoutFrame);

    this.add(panelTop, BorderLayout.NORTH);
    panelTop.add(labelTitle, null);
    panelTop.add(labelClusters, null);
    this.add(panelButtons, BorderLayout.SOUTH);
    panelButtons.add(buttonClassifier,null);
    panelButtons.add(buttonTMiner, null);
    panelButtons.add(buttonView, null);
    panelButtons.add(buttonOK, null);
    panelButtons.add(buttonHelp, null);
    this.add(panelClusters, BorderLayout.CENTER);
    panelClusters.add(listClusters, null);
    panelClusters.add(panelInfo, null);
    panelInfo.add(labelSelected, null);
    panelInfo.add(labelN, null);
    panelInfo.add(labelDispersion, null);
    panelInfo.add(panelAttribute, null);
    panelAttribute.add(labelAttribute, null);
    panelAttribute.add(choiceAttribute, null);
    panelInfo.add(labelAvg, null);
    panelInfo.add(labelVar, null);
    panelInfo.add(labelClass, null);
    panelInfo.add(panelClass, null);
    panelClass.add(labelNClass, null);
    panelClass.add(choiceClass, null);
  }

  // Estadsticas

  private void showStats ()
  {
    int     i,j;
    int     index[]  = listClusters.getSelectedIndexes();
    int     selected = index.length;
    int     total;
    float   dev;
    Cluster cluster;
    int     size;
    int     max;
    int     freq;
    int     clase;
    String  id = "";


    if (selected==1) {

       if (Environment.settings==Environment.ENGLISH)
          labelSelected.setText("Cluster No. "+(index[0]+1));
       else
          labelSelected.setText("Agrupamiento "+(index[0]+1));

    } else {

       if (Environment.settings==Environment.ENGLISH)
          labelSelected.setText(selected+" clusters selected");
       else
          labelSelected.setText(selected+" agrupamientos seleccionados");
    }

    if (selected==0) {

       buttonTMiner.setEnabled(false);
       buttonView.setEnabled(false);

       labelN.setVisible(false);
       labelDispersion.setVisible(false);
       panelAttribute.setVisible(false);
       labelAvg.setVisible(false);
       labelVar.setVisible(false);
       labelClass.setVisible(false);
       panelClass.setVisible(false);

    } else {

       buttonTMiner.setEnabled(true);
       buttonView.setEnabled(true);

       labelN.setVisible(true);
       labelDispersion.setVisible(true);
       panelAttribute.setVisible(true);
       labelAvg.setVisible(true);
       labelVar.setVisible(true);

       if (classification) {
          labelClass.setVisible(true);
          panelClass.setVisible(true);
       } else {
          labelClass.setVisible(false);
          panelClass.setVisible(false);
       }

       // Dispersin

       total = 0;
       dev   = 0;

       for (i=0; i<selected; i++) {
           cluster = clusters.cluster(index[i]);
           size    = cluster.size();
           dev     = (total*dev + size*cluster.dispersion())/(total+size);
           total  += size;
       }

       // Clase ms frecuente

       if (classification) {

          clase = -1;
          max = 0;

          for (j=0; j<config.getClasses().length; j++) {

              freq = 0;

              for (i=0; i<selected; i++) {
                  cluster = clusters.cluster(index[i]);
                  freq   += cluster.freq(j);
              }

              if (freq>max) {
                 max   = freq;
                 clase = j;
                 id    = config.getClass(j);
              }
          }

          if (clase!=-1) {
             choiceClass.select(clase);
             showClassFrequency();
          }
       }

       // Display

       if (Environment.settings==Environment.ENGLISH) {

          if (tminer!=null)
             labelN.setText(total+" tuples ("+pct((float)total/(float)patterns.instances())+")");
	  else
             labelN.setText(total+" patterns ("+pct((float)total/(float)patterns.instances())+")");
	  
          labelDispersion.setText("Average dispersion = "+dev);
          labelClass.setText("Class "+id);
       } else {

	  if (tminer!=null)
             labelN.setText(total+" tuplas ("+pct((float)total/(float)patterns.instances())+")");
	  else
             labelN.setText(total+" patrones ("+pct((float)total/(float)patterns.instances())+")");

          labelDispersion.setText("Dispersin media = "+dev);
          labelClass.setText("Clase "+id);
       }

       showAttrStats();
    }
  }

  private void showAttrStats()
  {
    int     i;
    int     index[]  = listClusters.getSelectedIndexes();
    int     attr     = choiceAttribute.getSelectedIndex();
    int     selected = index.length;
    float   average  = 0;
    float   variance = 0;
    int     total    = 0;
    Cluster cluster;
    float   avg1,avg2;
    float   var1,var2;
    int     n1,n2;

    for (i=0; i<selected; i++) {
        cluster  = clusters.cluster(index[i]);

        n1   = total;
        n2   = cluster.size();
        avg1 = average;
        avg2 = cluster.centroid(attr);
        var1 = variance;
        var2 = cluster.variance(attr);

        // Composicin de poblaciones

        total = n1+n2;

        // Media en composicin de poblaciones:
        // Media de las medias

        average = (n1*avg1 + n2*avg2)/total;

        // Varianza en composicin de poblaciones:
        // Media ponderada de las varianzas + Varianza de las medias

        variance = (n1*var1 + n2*var2)/total
                 + (n1*(avg1-average)*(avg1-average)+n2*(avg2-average)*(avg2-average))/total;
    }

    if (Environment.settings==Environment.ENGLISH) {
       labelAvg.setText("Average = "+average);
       labelVar.setText("Variance = "+variance);
    } else {
       labelAvg.setText("Media = "+average);
       labelVar.setText("Varianza = "+variance);
    }
  }

  private void showClassFrequency()
  {
    int     i;
    int     index[]  = listClusters.getSelectedIndexes();
    int     c        = choiceClass.getSelectedIndex();
    int     selected = index.length;
    int     actual   = 0;
    int     total    = 0;
    Cluster cluster;

    for (i=0; i<selected; i++) {
        cluster  = clusters.cluster(index[i]);
        actual  += cluster.freq(c);
	total   += cluster.size();
    }

    if (Environment.settings==Environment.ENGLISH) {
       labelNClass.setText(actual+" ("+pct((float)actual/(float)total)+") of the class");
    } else {
       labelNClass.setText(actual+" ("+pct((float)actual/(float)total)+") de la clase");
    }
  }

  private String pct (float f)
  {
    float p = 100*f;
    int   n = (int) p;
    int   d = (int) ((p-n)*10);

    return n+"."+d+"%";
  }

  // Cerrar ventana

  void exit ()
  {
    dispose();
  }

  // Conexin con TMiner para analizar los datos seleccionados

  void runTMiner ()
  {
    // Not supported yet
  }

  // Visualizacin de los datos de los clusters seleccionados

  void view ()
  {
    int       i;
    boolean   selected[] = selectedClustersPatterns();
    Color     colors[]   = new Color[patterns.instances()];
    Dimension dim        = Toolkit.getDefaultToolkit().getScreenSize();
    Frame2D   frame2D    = null;

    for (i=0; i<patterns.instances(); i++)
        if (selected[i])
	   colors[i] = Color.red;
	else
	   colors[i] = Color.blue;
    
    frame2D = new Frame2D (config, patterns, colors );

    frame2D.setLocation((dim.width - frame2D.getSize().width) / 2, (dim.height - frame2D.getSize().height) / 2);
    frame2D.setVisible(true);
  }

  // Devuelve un array que indica si cada dato est entre los clusters seleccionados

  public boolean[] selectedClustersPatterns ()
  {
    int      i,j;
    int      cluster;
    int      selected[] = listClusters.getSelectedIndexes();
    boolean  index[]    = new boolean[clusters.clusters()];
    boolean  result[]   = new boolean[patterns.instances()];
    Pattern  pattern;
    float    dist;
    float    mindist;

    // Clusters seleccionados

    for (i=0; i<index.length; i++)
        index[i] = false;

    for (i=0; i<selected.length; i++)
        index[selected[i]] = true;

    // Datos correspondientes a los clusters seleccionados

    for (i=0; i<patterns.instances(); i++) {
     
        pattern = patterns.pattern(i);
	cluster = 0;
        mindist = clusters.metrics.distance ( clusters.cluster(0), pattern );

	for (j=1; j<clusters.clusters(); j++) {

            dist = clusters.metrics.distance ( clusters.cluster(j), pattern );

	    if (dist<mindist) {
               cluster = j;
	       mindist = dist;
	    }
	}

	if (index[cluster])
           result[i] = true;
	else
	   result[i] = false;
    }

    return result;
  }

  // Clasificador asociado

  void associatedClassifier ()
  {
    Dimension       dim = Toolkit.getDefaultToolkit().getScreenSize();
    FrameClassifier frameClassifier;

    frameClassifier = new FrameClassifier (config, clusters, patterns, config.getData().testSet(), getTitle(), out, standalone);
    frameClassifier.setLocation((dim.width - frameClassifier.getSize().width) / 2, (dim.height - frameClassifier.getSize().height) / 2);
    frameClassifier.setVisible(true);
  }

  // Ayuda en lnea

  void onlineHelp ()
  {
    help.help ( "nc/clustering/index.html" );
  }

  // Seleccin de agrupamientos

  void listClusters_itemStateChanged(ItemEvent e)
  {
    showStats();
  }

  // Lista de atributos:
  // Actualizacin de estadsticas del atributo seleccionado

  void choiceAttribute_itemStateChanged(ItemEvent e)
  {
    showAttrStats();
  }

  // Lista de clases:
  // Patrones de la clase seleccionada

  void choiceClass_itemStateChanged(ItemEvent e)
  {
    showClassFrequency();
  }
}

// Event handlers
// --------------

class FrameClusters_this_windowAdapter extends java.awt.event.WindowAdapter
{
 FrameClusters adaptee;

 FrameClusters_this_windowAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void windowClosing(WindowEvent e)
 {
  adaptee.exit();
 }
}

class FrameClusters_buttonOK_actionAdapter implements java.awt.event.ActionListener
{
 FrameClusters adaptee;

 FrameClusters_buttonOK_actionAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.exit();
 }
}

class FrameClusters_buttonClassifier_actionAdapter implements java.awt.event.ActionListener
{
 FrameClusters adaptee;

 FrameClusters_buttonClassifier_actionAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.associatedClassifier ();
 }
}

class FrameClusters_buttonTMiner_actionAdapter implements java.awt.event.ActionListener
{
 FrameClusters adaptee;

 FrameClusters_buttonTMiner_actionAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.runTMiner ();
 }
}

class FrameClusters_buttonView_actionAdapter implements java.awt.event.ActionListener
{
 FrameClusters adaptee;

 FrameClusters_buttonView_actionAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.view();
 }
}

class FrameClusters_buttonHelp_actionAdapter implements java.awt.event.ActionListener
{
 FrameClusters adaptee;

 FrameClusters_buttonHelp_actionAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.onlineHelp ();
 }
}

class FrameClusters_listClusters_itemAdapter implements java.awt.event.ItemListener
{
 FrameClusters adaptee;


 FrameClusters_listClusters_itemAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void itemStateChanged(ItemEvent e)
 {
  adaptee.listClusters_itemStateChanged(e);
 }
}

class FrameClusters_choiceAttribute_itemAdapter implements java.awt.event.ItemListener
{
 FrameClusters adaptee;

 FrameClusters_choiceAttribute_itemAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void itemStateChanged(ItemEvent e)
 {
  adaptee.choiceAttribute_itemStateChanged(e);
 }
}

class FrameClusters_choiceClass_itemAdapter implements java.awt.event.ItemListener
{
 FrameClusters adaptee;

 
 FrameClusters_choiceClass_itemAdapter(FrameClusters adaptee)
 {
  this.adaptee = adaptee;
 }

 public void itemStateChanged(ItemEvent e)
 {
  adaptee.choiceClass_itemStateChanged(e);
 }
}

// LAYOUT
// ------

class FrameClustersLayout implements LayoutManager
{
  // Frame

  FrameClusters frame;

  // Constructor

  public FrameClustersLayout ( FrameClusters frame )
  {
    this.frame = frame;
  }

  // LayoutManager

  Dimension origin = new Dimension(300, 200);

  public Dimension preferredLayoutSize (Container c)
  {
    return origin;
  }

  public Dimension minimumLayoutSize (Container c)
  {
    return origin;
  }

  public void addLayoutComponent (String s, Component c)
  {
  }

  public void removeLayoutComponent(Component c)
  {
  }

  public void layoutContainer(Container c)
  {
    Rectangle b = c.getBounds();

    int width        = b.width;
    int height       = b.height;
    int inset        = 5;

    // listClusters

    frame.listClusters.setBounds ( inset,
	                           inset,
			           width/4,
			           height-2*inset);

    // panelInfo

    frame.panelInfo.setBounds ( 2*inset+width/4,
				inset,
			        width-(3*inset+width/4),
			        height-2*inset);
  }
}
