package rf.gui;

// Title:       Numerical Cruncher
// Version:     1.2
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.JButton;

import fbg.util.*;

import rf.*;
import rf.data.*;


public class FrameStats extends Frame
{
 // Data

 DataConfiguration cfg;
 Dataset           set;
 Statistics        stats;

 // Components

 Panel panelNorth = new Panel();
 Panel panelStats = new Panel();
 Panel panelCovariance = new Panel();
 Panel panelSouth = new Panel();
 
 BorderLayout borderLayout = new BorderLayout();
 GridLayout   gridLayoutStats = new GridLayout();
 FlowLayout   flowLayoutButtons = new FlowLayout();

 Label labelAttr = new Label();
 Label labelMin = new Label();
 Label labelMax = new Label();
 Label labelAvg = new Label();
 Label labelVar = new Label();
 Label labelCov = new Label();
 Label labelCovAux = new Label();

 Choice choiceAttr = new Choice();
 Choice choiceCov = new Choice();

 JButton buttonOK = new JButton();
 JButton buttonView = new JButton();

 // Constructor
 // -----------

 public FrameStats(DataConfiguration cfg, Dataset set)
 {
   this.cfg   = cfg;
   this.set   = set;
   this.stats = new Statistics(set);

   try {
     jbInit();
     this.setTitle ("Estadsticas: "+set.instances()+" patrones");
   } catch (Exception e) {
     e.printStackTrace();
   }

   //  Image data ?

   if (cfg.getFormat()==DataConfiguration.IMAGE_DATA) {
      buttonView.setVisible(true);
   } else {
      buttonView.setVisible(false);
   }

   // Lists

   int    i;
   String id;

   for (i=0; i<cfg.totalAttributes(); i++) {
       id = cfg.getAttribute(i);
       choiceAttr.add (id);
       choiceCov.add (id);
   }

   // Language

   if (Environment.settings!=Environment.SPANISH) {
      this.setTitle ("Statistics: "+set.instances()+" patterns");
      labelAttr.setText("Attribute");
      labelMin.setText("Mininum value = XXX");
      labelMax.setText("Maximum value = XXX");
      labelAvg.setText("Mean value = XXX");
      labelVar.setText("Variance = XXX");
      labelCovAux.setText("Covariance with");
      buttonView.setText("View image");
      buttonView.setToolTipText("Display the image corresponding to this attribute");
   }

   // Stats

   showStatistics(0,0);
 }

 private void jbInit() throws Exception
 {
  Image icon = Loader.loadImage(getClass(),"image/table.gif");

  this.setIconImage(icon);
  this.setBackground(Color.lightGray);
  this.setSize(new Dimension(450, 275));
  this.setLayout(borderLayout);
  this.addWindowListener(new FrameStats_windowAdapter(this));

  gridLayoutStats.setRows(5);
  gridLayoutStats.setColumns(1);

  labelAttr.setForeground(Color.blue);
  labelAttr.setFont(new Font("Dialog", 3, 12));
  labelAttr.setAlignment(1);
  labelAttr.setText("Atributo");
  panelStats.setLayout(gridLayoutStats);
  labelMin.setAlignment(1);
  labelMin.setText("Mnimo = XXX");
  labelMax.setAlignment(1);
  labelMax.setText("Mximo = XXX");
  labelAvg.setAlignment(1);
  labelAvg.setText("Media = XXX");
  labelVar.setAlignment(1);
  labelVar.setText("Varianza = XXX");
  panelSouth.setLayout(flowLayoutButtons);
  labelCov.setAlignment(1);
  labelCov.setText("= COV");
  labelCovAux.setText("Covarianza con");

  buttonView.setText("Ver imagen");
  buttonView.setToolTipText("Visualizar la imagen correspondiente a este atributo");
  buttonView.addActionListener(new FrameStats_buttonView_actionAdapter(this));
  buttonOK.setText("OK");
  buttonOK.addActionListener(new FrameStats_buttonOK_actionAdapter(this));

  choiceCov.addItemListener(new FrameStats_choiceAdapter(this));
  choiceAttr.addItemListener(new FrameStats_choiceAdapter(this));

  this.add(panelNorth, BorderLayout.NORTH);
  panelNorth.add(labelAttr, null);
  panelNorth.add(choiceAttr, null);
  this.add(panelStats, BorderLayout.CENTER);
  panelStats.add(labelMin, null);
  panelStats.add(labelMax, null);
  panelStats.add(labelAvg, null);
  panelStats.add(labelVar, null);
  panelStats.add(panelCovariance, null);
  panelCovariance.add(labelCovAux, null);
  panelCovariance.add(choiceCov, null);
  panelCovariance.add(labelCov, null);
  this.add(panelSouth, BorderLayout.SOUTH);
  panelSouth.add(buttonView, null);
  panelSouth.add(buttonOK, null);
 }

 // Stats visualization
 // -------------------

 void showStatistics (int n, int c)
 {
   if (Environment.settings==Environment.SPANISH) {
      labelMin.setText( "Mnimo = " + stats.min(n) );
      labelMax.setText( "Mximo = " + stats.max(n) );
      labelAvg.setText( "Media = " + stats.mean(n) );
      labelVar.setText( "Varianza = " + stats.variance(n) );
   } else {
      labelMin.setText( "Minimum value = " + stats.min(n) );
      labelMax.setText( "Maximum value = " + stats.max(n) );
      labelAvg.setText( "Mean value = " + stats.mean(n) );
      labelVar.setText( "Variance = " + stats.variance(n) );
   }

   labelCov.setText( "= " + stats.covariance(n,c) );
 }

 // Events
 // ------

 void stateChanged(ItemEvent e)
 {
   showStatistics( choiceAttr.getSelectedIndex(),
                   choiceCov.getSelectedIndex() );
 }

 void exit ()
 {
   dispose();
 }

 void buttonView_actionPerformed (ActionEvent e)
 {
   FrameImage frame;
   Dimension  dim  = Toolkit.getDefaultToolkit().getScreenSize();
   ImageData  data = (ImageData) cfg.getData();
   int        attr = choiceAttr.getSelectedIndex();

   frame = new FrameImage ( data.band(attr).getRawImage(),
                            cfg.getAttribute(attr), 300);

   frame.setLocation( (dim.width - frame.getSize().width) / 2,
                      (dim.height - frame.getSize().height) / 2 );

   frame.setVisible(true);
 }
}

// Event handlers
// --------------

class FrameStats_windowAdapter extends java.awt.event.WindowAdapter
{
 FrameStats adaptee;

 FrameStats_windowAdapter(FrameStats adaptee)
 {
  this.adaptee = adaptee;
 }

 public void windowClosing(WindowEvent e)
 {
  adaptee.exit();
 }
}

class FrameStats_choiceAdapter implements java.awt.event.ItemListener
{
 FrameStats adaptee;

 FrameStats_choiceAdapter(FrameStats adaptee)
 {
  this.adaptee = adaptee;
 }

 public void itemStateChanged(ItemEvent e)
 {
  adaptee.stateChanged(e);
 }
}

class FrameStats_buttonOK_actionAdapter implements java.awt.event.ActionListener
{
 FrameStats adaptee;

 FrameStats_buttonOK_actionAdapter(FrameStats adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.exit();
 }
}

class FrameStats_buttonView_actionAdapter implements java.awt.event.ActionListener
{
 FrameStats adaptee;

 FrameStats_buttonView_actionAdapter(FrameStats adaptee)
 {
  this.adaptee = adaptee;
 }

 public void actionPerformed(ActionEvent e)
 {
  adaptee.buttonView_actionPerformed(e);
 }
}
