package rf.gui;

// Title:       Numerical Cruncher (software for Pattern Recognition)
// Version:     1.2
// Copyright:   June 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

import java.io.*;
import java.net.*;

import fbg.gui.GUI;
import fbg.gui.Message;
import fbg.gui.html.HTMLFrame;

import fbg.util.Help;
import fbg.util.Loader;
import fbg.util.Output;
import fbg.util.Environment;

import rf.DataConfiguration;
import rf.gui.jdbc.FrameConnect;


public class NumericalCruncher extends    Applet
                               implements Output, Help
{
  // Constants
  // ---------

  // Size

  private static final int WIDTH  = 400;
  private static final int HEIGHT = 200;

  // HTML DOCUMENTATION

  public static final String ESP_HELP  = "help/spanish/";
  public static final String ENG_HELP  = "help/english/";

  public static final String DOC_INDEX = "nc/index.html";

  // Instance variables
  // ------------------

  boolean      standalone  = false;

  Frame        frame   = new Frame();

  TextArea     msg = new TextArea();
  Button       buttonStart = new Button();
  BorderLayout layout = new BorderLayout();
  Panel        panelHeader = new Panel();
  Label        labelTitle = new Label();
  Label        labelCopyright = new Label();

  // Start

  FrameConnect login;
  FileDialog   open; 

  // Help system

  HTMLFrame     frameHTML;

  // Constructor
  // -----------
  // Construct the applet

  public NumericalCruncher ()
  {
    this (false);  // APPLET
  }

  public NumericalCruncher (boolean application)
  {
    GUI.Look(GUI.SYSTEM);

    this.setLayout(layout);
    this.setBackground(Color.lightGray);
    this.setSize(WIDTH,HEIGHT);

    //msg.setBackground(Color.lightGray);
    msg.setBackground(new Color(208,208,208));
    msg.setEditable(false);

    buttonStart.setBackground(Color.lightGray);
    buttonStart.setFont(new Font("Serif", 2, 14));
    buttonStart.setLabel("Inicio");
    buttonStart.addActionListener(new StartButton_actionAdapter(this));

    layout.setHgap(5);
    layout.setVgap(5);

    labelTitle.setFont(new Font("Serif", 3, 18));
    labelTitle.setText("Analizador Numrico");
    labelCopyright.setFont(new Font("Serif", 2, 12));
    labelCopyright.setText(" 1999 Fernando Berzal Galiano");

    panelHeader.setBackground(Color.lightGray);
    panelHeader.add(labelTitle, null);
    panelHeader.add(labelCopyright, null);

    this.add(msg, BorderLayout.CENTER);
    this.add(buttonStart, BorderLayout.SOUTH);

    if (application)
       this.add(panelHeader, BorderLayout.NORTH);

    // Screen
    
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    // Startup

    if (application) {
       // File dialog
       open = new FileDialog(frame,"Seleccione un fichero de configuracin adecuado",FileDialog.LOAD);
       open.setLocation((d.width - open.getSize().width) / 2, (d.height - open.getSize().height) / 2);
    } else {
       // JDBC Connection
       login = new FrameConnect(this,this);
       login.setLocation((d.width - login.getSize().width) / 2, (d.height - login.getSize().height) / 2);
    }
    // HTML help

    if (application) {
       frameHTML = new HTMLFrame("Ayuda en lnea", ESP_HELP+DOC_INDEX );
       frameHTML.setLocation((d.width - frameHTML.getSize().width) / 2, (d.height - frameHTML.getSize().height) / 2);
    } else {
       frameHTML = null;
    }
  }

  // Language options

  private void EnglishVersion ()
  {
    Environment.set(Environment.ENGLISH);

    if (open!=null)
       open.setTitle("Select a data configuration file");

    if (login!=null)
       login.English();

    buttonStart.setLabel("Start");
    labelTitle.setText("Numerical Cruncher");

    if (frameHTML!=null) {
       frameHTML.setTitle ("Numerical Cruncher Help");
       frameHTML.setHome (ENG_HELP+DOC_INDEX);
    }
  }

  private void SpanishVersion ()
  {
    Environment.set(Environment.SPANISH);
  }

  // Methods
  // -------

  // Get a parameter value

  public String getParameter(String key, String def)
  {
    return standalone ? System.getProperty(key, def) :
                       (getParameter(key)!=null ? getParameter(key) : def);
  }

  // Initialize the applet

  public void init()
  {  
    if (!standalone)
       if (getParameter("Language","Spanish").equals("English"))
          EnglishVersion();
       else
          SpanishVersion();

    this.setEnabled(true);
    this.setBackground(Color.lightGray);
    this.setSize(WIDTH, HEIGHT);
    this.setLayout(layout);
  }

  // Start the applet (after init or stop)

  public void start()
  {
  }

  // Stop the applet

  public void stop()
  {
  }

  // Destroy the applet

  public void destroy()
  {
  }

  // Update

  public void update(Graphics g)
  {
    paint(g);
  }

  // Get Applet information

  public String getAppletInfo()
  {
    return "Numerical Cruncher\n\nFernando Berzal Galiano\n\nberzal@acm.org";
  }

  // Get parameter info

  public String[][] getParameterInfo()
  {
    return null;
  }

  // Help system
  // -----------

  private String helpBase ()
  {
    if (Environment.settings==Environment.ENGLISH)
       return ENG_HELP;
    else
       return ESP_HELP;
  }

  private URL helpURL (String str)
  {
    try {

      if (standalone) {

         File file = new File(helpBase()+str);

         return file.toURL();

      } else {

         return new URL(getDocumentBase(),helpBase()+str);

      }

    } catch (MalformedURLException e) {

        Message.showMessageDialog ( (Component)null,
                                    "URL: "+str, // e.toString()
                                    "Error",
                                    Message.ERROR_MESSAGE);
    } catch (IOException e) {

        Message.showMessageDialog ( (Component)null,
                                    "File: "+str, // e.toString()
                                    "Error",
                                    Message.ERROR_MESSAGE);
    }

    return null;
  }

  public void help ()
  {
    if (standalone) { 

       frameHTML.show();

    } else {

       try {
         getAppletContext().showDocument( helpURL(DOC_INDEX), "Numerical Cruncher Help" );
       } catch (Exception error) {
         System.out.println(error);
       }
    }
  }

  public void help (String url)
  {
    if (standalone) {

       frameHTML.jump( helpURL(url) );
       frameHTML.show();

    } else {

       try {
         getAppletContext().showDocument( helpURL(url), "Numerical Cruncher Help" );
       } catch (Exception error) {
         System.out.println(error);
       }
    }
  }

  // Messages
  // --------

  public void message (String str)
  {
    msg.append(str);
  }


  // Application startup
  // -------------------

  public void NCapplication ()
  {
    String            fileID;
    DataConfiguration cfg;
    MainMenu          menu;
    Dimension         d = Toolkit.getDefaultToolkit().getScreenSize();

    open.show();

    fileID = open.getDirectory() + "/" + open.getFile();

    if (open.getFile()!=null) {

       GUI.setWaitCursor(frame);

       cfg = new DataConfiguration(fileID);

       if (cfg.totalAttributes()>0) {

          menu = new MainMenu(cfg,this,this,this.standalone);
          menu.setLocation((d.width - menu.getSize().width) / 2, (d.height - menu.getSize().height) / 2);
          menu.setVisible(true);

          GUI.setDefaultCursor(frame);

       } else {

          Message.showMessageDialog ( (Component)null,
                                      fileID,
                                      "Error",
                                      Message.ERROR_MESSAGE);
       }
    }
  }

  // Applet startup
  // --------------

  public void NCapplet ()
  {
    login.setVisible(true);
  }

  // Main method
  // -----------

  public static void main(String[] args)
  {
    int               i;
    NumericalCruncher applet  = new NumericalCruncher(true);
    MainWindowAdapter wl      = new MainWindowAdapter();
    DataConfiguration cfg;
    MainMenu          menu;

    try {

        applet.standalone = true;

        applet.frame.addWindowListener(wl);
        applet.frame.setTitle("Numerical Cruncher");
        applet.frame.add(applet, BorderLayout.CENTER);

        Dimension d    = Toolkit.getDefaultToolkit().getScreenSize();
        Image     icon = Loader.loadImage(NumericalCruncher.class,"image/start.gif");

        applet.frame.setIconImage(icon);
        applet.frame.setSize(WIDTH,HEIGHT);
        applet.frame.setLocation((d.width - applet.frame.getSize().width) / 2,0);

        applet.frame.setVisible(true);

        applet.init();
        applet.start();
        // applet.frame.pack();

        if (args.length>0) {

           for (i=0; i<args.length; i++) {

               if (args[i].toUpperCase().equals("ENGLISH")) {

                  applet.EnglishVersion();

               } else if (args[i].toUpperCase().equals("SPANISH")) {

                  applet.SpanishVersion();

               } else {

                  GUI.setWaitCursor(applet.frame);

                  cfg = new DataConfiguration(args[i]);

                  if (cfg.totalAttributes()>0) {

                     menu = new MainMenu(cfg,applet,applet,applet.standalone);
                     menu.setLocation((d.width - menu.getSize().width) / 2, (d.height - menu.getSize().height) / 2);
                     menu.setVisible(true);

                     GUI.setDefaultCursor(applet.frame);

                  } else {

                     Message.showMessageDialog ( (Component)null,
                                                 args[i],
                                                 "Error",
                                                 Message.ERROR_MESSAGE);
                  }
               }
           }
        }

    } catch (Throwable e) { // Exception | Error
        Message.showMessageDialog ( (Component)null,
                                    e.toString(),  // e.getMessage()
                                    "Numerical Cruncher Error",
                                    Message.ERROR_MESSAGE);
    }
  }

  // Events
  // ------

  void buttonStart_actionPerformed(ActionEvent e)
  {
    if (standalone)
       NCapplication();
    else
       NCapplet();
  }
}

// Event handling
// --------------

// WindowListener

class MainWindowAdapter extends WindowAdapter
{
  public void windowClosing (WindowEvent e)
  {
    System.exit(0);
  }
}

// Start

class StartButton_actionAdapter implements java.awt.event.ActionListener
{
  NumericalCruncher adaptee;

  StartButton_actionAdapter (NumericalCruncher adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonStart_actionPerformed(e);
  }
}

