package rf.gui.jdbc;

// Title:       JDBC Connection
// Version:     1.71
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.sql.*;

import fbg.gui.*;
import fbg.util.*;

import rf.gui.NumericalCruncher;

/* This frame is used to establish a JDBC connection */

public class FrameConnect extends Frame
{
  // Output messages

  Output    out  = null;

  // Help system

  Help      help = null;

  // Connection parameters

  String     user     = null;
  String     password = null;
  String     URL      = null;
  String     JDBC     = null;

  Connection connect  = null;

  // Components

  Panel panelConnect = new Panel();
  Panel panelButtons = new Panel();
  Panel panelConfig = new Panel();

  GridLayout layoutConnect = new GridLayout();
  GridLayout gridLayoutConfig = new GridLayout();
  GridLayout gridLayoutButtons = new GridLayout();
  GridBagLayout layout = new GridBagLayout();
  GridBagConstraints constraints = new GridBagConstraints();

  Label labelConfig = new Label();
  Label labelUser = new Label();
  Label labelPassword = new Label();
  Label labelURL = new Label();
  Label labelJDBC = new Label();

  TextField textFieldUser = new TextField();
  TextField textFieldPassword = new TextField();
  TextField textFieldURL = new TextField();
  TextField textFieldJDBC = new TextField();

  CheckboxGroup cbgConfig = new CheckboxGroup();

  Checkbox checkboxManual = new Checkbox();
  Checkbox checkboxOracle = new Checkbox();
  Checkbox checkboxNano   = new Checkbox();

  JButton buttonConnect = new JButton();
  JButton buttonExit = new JButton();
  JButton buttonHELP = new JButton();

  // Constructor

  public FrameConnect (Output output, Help help_system)
  {
    try {
      GUI();
      repaint();
    } catch (Exception e) {
      e.printStackTrace();
    }

    out  = output;
    help = help_system;

    if (Environment.settings==Environment.SPANISH)
       Spanish();
    else
       English();
  }

  private void GUI() throws Exception
  {
    Image icon = Loader.loadImage(NumericalCruncher.class,"image/connect.gif");

    this.setIconImage(icon);
    this.addWindowListener(new FrameConnect_this_windowAdapter(this));
    this.setBackground(Color.lightGray);
    this.setSize(new Dimension(500, 250));
    
    labelUser.setFont(new Font("Dialog", 3, 12));
    labelUser.setAlignment(1);
    labelPassword.setFont(new Font("Dialog", 3, 12));
    labelPassword.setAlignment(1);
    labelURL.setFont(new Font("Dialog", 3, 12));
    labelURL.setAlignment(1);
    labelJDBC.setFont(new Font("Dialog", 3, 12));
    labelJDBC.setAlignment(1);
    labelConfig.setFont(new Font("Dialog", 3, 12));
    labelConfig.setAlignment(1);

    textFieldPassword.setEchoChar('*');

    checkboxManual.setCheckboxGroup(cbgConfig);
    checkboxManual.addItemListener(new FrameConnect_checkboxManual_itemAdapter(this));
    checkboxNano.setCheckboxGroup(cbgConfig);
    checkboxNano.addItemListener(new FrameConnect_checkboxNano_itemAdapter(this));
    checkboxOracle.setCheckboxGroup(cbgConfig);
    checkboxOracle.addItemListener(new FrameConnect_checkboxOracle_itemAdapter(this));
    cbgConfig.setSelectedCheckbox(checkboxManual);

    buttonHELP.addActionListener(new FrameConnect_buttonHELP_actionAdapter(this));
    buttonConnect.addActionListener(new FrameConnect_buttonConnect_actionAdapter(this));
    buttonExit.addActionListener(new FrameConnect_buttonExit_actionAdapter(this));

    gridLayoutConfig.setRows(4);
    gridLayoutConfig.setColumns(1);
    panelConfig.setLayout(gridLayoutConfig);

    gridLayoutButtons.setHgap(10);
    gridLayoutButtons.setRows(3);
    gridLayoutButtons.setColumns(1);
    gridLayoutButtons.setVgap(10);
    panelButtons.setLayout(gridLayoutButtons);

    layoutConnect.setRows(8);
    layoutConnect.setColumns(1);
    panelConnect.setLayout(layoutConnect);

    this.setLayout(layout);

    constraints.anchor = GridBagConstraints.CENTER;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.insets = new Insets(5,5,5,5);

    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.gridwidth  = 6;
    constraints.gridheight = 2;
    constraints.weightx = 2.0;
    constraints.weighty = 1.0;

    panelConnect.add(labelUser, null);
    panelConnect.add(textFieldUser, null);
    panelConnect.add(labelPassword, null);
    panelConnect.add(textFieldPassword, null);
    panelConnect.add(labelURL, null);
    panelConnect.add(textFieldURL, null);
    panelConnect.add(labelJDBC, null);
    panelConnect.add(textFieldJDBC, null);
    this.add(panelConnect, constraints);

    constraints.gridx = 6;
    constraints.gridy = 0;
    constraints.gridwidth  = 2;
    constraints.gridheight = 1;
    constraints.weightx = 0.5;
    constraints.weighty = 0.5;

    panelConfig.add(labelConfig, null);
    panelConfig.add(checkboxManual, null);
    panelConfig.add(checkboxNano, null);
    panelConfig.add(checkboxOracle, null);
    this.add(panelConfig, constraints);

    constraints.gridx = 6;
    constraints.gridy = 1;
    constraints.gridwidth  = 2;
    constraints.gridheight = 1;
    constraints.weightx = 0.5;
    constraints.weighty = 0.5;

    panelButtons.add(buttonConnect, null);
    panelButtons.add(buttonExit, null);
    panelButtons.add(buttonHELP, null);
    this.add(panelButtons, constraints);
  }

  // Spanish version

  public void Spanish ()
  {
    this.setTitle("Establecimiento de la conexin");

    labelUser.setText("Usuario");
    labelPassword.setText("Clave de acceso");
    labelURL.setText("URL");
    labelJDBC.setText("Driver JDBC");
    labelConfig.setText("Configuracin");

    checkboxManual.setLabel("Configuracin manual");
    checkboxNano.setLabel("FBG Nano Server");
    checkboxOracle.setLabel("Oracle Thin");

    buttonHELP.setText("Ayuda");
    buttonHELP.setToolTipText("Ayuda en lnea");
    buttonConnect.setText("Conectar");
    buttonConnect.setToolTipText("Establecer conexin");
    buttonExit.setText("Salir");
    buttonExit.setToolTipText("No establecer conexin");
  }

  // English Version

  public void English ()
  {
    this.setTitle("JDBC Connection");

    labelUser.setText("User");
    labelPassword.setText("Password");
    labelURL.setText("URL");
    labelJDBC.setText("JDBC Driver");
    labelConfig.setText("Configuration");

    checkboxManual.setLabel("Custom");
    checkboxNano.setLabel("FBG Nano Server");
    checkboxOracle.setLabel("Oracle Thin");

    buttonHELP.setText("Help");
    buttonHELP.setToolTipText("Online help");
    buttonConnect.setText("Connect");
    buttonConnect.setToolTipText("Open JDBC Connection");
    buttonExit.setText("Exit");
    buttonExit.setToolTipText("Don't open connection");
  }

  // Events
  // ------

  // JDBC Presets

  void checkboxNano_itemStateChanged(ItemEvent e)
  {
    // JDBC Configuration = FBG Nano Server

    if (checkboxNano.getState()==true) {
       textFieldUser.setText("guest");
       textFieldPassword.setText("*****");
       textFieldURL.setText("jdbc:fbg@localhost:6666:data\\ascii.db");
       textFieldJDBC.setText("fbg.jdbc.client.ClientDriver");
       textFieldJDBC.setEnabled(false);
    }
  }

  void checkboxOracle_itemStateChanged(ItemEvent e)
  {
    // JDBC Configuration: Oracle Thin JDBC Driver

    if (checkboxOracle.getState()==true) {
       textFieldURL.setText("jdbc:oracle:thin:@<host>:<port>:<db>");
       textFieldJDBC.setText("oracle.jdbc.driver.OracleDriver");
       textFieldJDBC.setEnabled(false);
    }
  }

  void checkboxManual_itemStateChanged(ItemEvent e)
  {
    // Custom configuration

    textFieldURL.setEnabled(true);
    textFieldJDBC.setEnabled(true);
  }


  // Close window

  void this_windowClosing(WindowEvent e)
  {
    setVisible(false);
  }

  void buttonExit_actionPerformed(ActionEvent e)
  {
    setVisible(false);
  }

  // Open connection

  void buttonConnect_actionPerformed(ActionEvent e)
  {
    setVisible(false);

    // JDBC parameters

    user     = textFieldUser.getText();
    password = textFieldPassword.getText();
    URL      = textFieldURL.getText();
    JDBC     = textFieldJDBC.getText();

    // Establecer conexin con el servidor a travs de JDBC

    if (URL==null || URL.equals("") ) {

       if (Environment.settings==Environment.SPANISH)
          out.message("ERROR: Debe especificar una URL vlida.\n");
       else
          out.message("ERROR: An URL must be specified.\n");

    } else if (JDBC==null || JDBC.equals("") ) {

       if (Environment.settings==Environment.SPANISH)
          out.message("ERROR: Debe especificar un driver JDBC adecuado.\n");
       else
          out.message("ERROR: A JDBC driver must be selected.\n");

    } else {

       if (Environment.settings==Environment.SPANISH)
          out.message ("Cargando el driver JDBC...\n");
       else
          out.message ("Loading JDBC driver...\n");

       try {
           Class.forName(JDBC);

           if (Environment.settings==Environment.SPANISH)
              out.message("Driver '"+JDBC+"' cargado.\n");
           else
              out.message("'"+JDBC+"' loaded successfuly.\n");

           // JDBC Connection

           if (Environment.settings==Environment.SPANISH)
              out.message ("Estableciendo la conexin...\n");
           else
              out.message ("Opening JDBC connection...\n");

           try {
               connect = DriverManager.getConnection(URL,user,password);
               out.message ("OK\n");
           } catch (Exception error) {
               out.message (error.getMessage()+"\n");
           }
              
       } catch (Exception error) {

           if (Environment.settings==Environment.SPANISH)
              out.message("Driver '"+JDBC+"' no encontrado.\n");
           else
              out.message("'"+JDBC+"' not found.\n");
       }

    }

    // Select one of the available tables

    if (connect!=null) {

       FrameTables tables = new FrameTables (out,help,connect);

       Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
       tables.setLocation((d.width - tables.getSize().width) / 2, (d.height - tables.getSize().height) / 2);

       connect = null;
       tables.setVisible(true);
    }
  }

  // Help system

  void buttonHELP_actionPerformed(ActionEvent e)
  {
    help.help ();
  }

  // Test

  public static void main (String args[])
  {
    FrameConnect frame = new FrameConnect(null,null);

    frame.setVisible(true);
  }
}

// Event handlers
// --------------

class FrameConnect_checkboxOracle_itemAdapter implements java.awt.event.ItemListener
{
  FrameConnect adaptee;

  FrameConnect_checkboxOracle_itemAdapter(FrameConnect adaptee)
  {
    this.adaptee = adaptee;
  }

  public void itemStateChanged(ItemEvent e)
  {
    adaptee.checkboxOracle_itemStateChanged(e);
  }
}


class FrameConnect_checkboxManual_itemAdapter implements java.awt.event.ItemListener
{
  FrameConnect adaptee;

  FrameConnect_checkboxManual_itemAdapter(FrameConnect adaptee)
  {
    this.adaptee = adaptee;
  }

  public void itemStateChanged(ItemEvent e)
  {
    adaptee.checkboxManual_itemStateChanged(e);
  }
}

class FrameConnect_checkboxNano_itemAdapter implements java.awt.event.ItemListener
{
  FrameConnect adaptee;

  FrameConnect_checkboxNano_itemAdapter(FrameConnect adaptee)
  {
    this.adaptee = adaptee;
  }

  public void itemStateChanged(ItemEvent e)
  {
    adaptee.checkboxNano_itemStateChanged(e);
  }
}


class FrameConnect_this_windowAdapter extends java.awt.event.WindowAdapter
{
  FrameConnect adaptee;

  FrameConnect_this_windowAdapter(FrameConnect adaptee)
  {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e)
  {
    adaptee.this_windowClosing(e);
  }
}

class FrameConnect_buttonConnect_actionAdapter implements java.awt.event.ActionListener
{
  FrameConnect adaptee;

  FrameConnect_buttonConnect_actionAdapter(FrameConnect adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonConnect_actionPerformed(e);
  }
}

class FrameConnect_buttonExit_actionAdapter implements java.awt.event.ActionListener
{
  FrameConnect adaptee;

  FrameConnect_buttonExit_actionAdapter(FrameConnect adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonExit_actionPerformed(e);
  }
}

class FrameConnect_buttonHELP_actionAdapter implements java.awt.event.ActionListener
{
  FrameConnect adaptee;


  FrameConnect_buttonHELP_actionAdapter(FrameConnect adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonHELP_actionPerformed(e);
  }
}
