package rf.gui.jdbc;

// Title:       Attribute selection
// Version:     1.71
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import javax.swing.*;

import fbg.gui.*;
import fbg.util.*;

import rf.DataConfiguration;

import rf.gui.MainMenu;
import rf.gui.NumericalCruncher;

import tminer.TMiner;

/**
 * This frame is used to select the table columns 
 * we will consider in NC
 */


public class FrameFields extends Frame
{
  // Instance variables
  // ------------------

  boolean    numeric  = false;
  String     objetivo = null;  // For classification purposes

  // JDBC Access

  String     table;
  Connection connection;

  // Output

  Output     out  = null;
  Help       help = null;

  // Caller

  Frame      frame = null;

  // TMiner 

  TMiner     miner = null;

  // Components

  Panel panelLIST = new Panel();
  Panel panelBUTTONS = new Panel();
  Panel panelAVAIL = new Panel();
  Panel panelOK = new Panel();
  Panel panelDomains = new Panel();
  Panel panelObjetivo = new Panel();

  BorderLayout borderLayout = new BorderLayout();
  GridBagLayout gridBagLayout = new GridBagLayout();
  GridLayout gridLayoutBUTTONS = new GridLayout();
  GridLayout gridLayoutLIST = new GridLayout();
  BorderLayout borderLayoutAVAIL = new BorderLayout();
  BorderLayout borderLayoutOK = new BorderLayout();
  FlowLayout flowLayoutObjetivo = new FlowLayout();

  JButton buttonADD = new JButton();
  JButton buttonADD1 = new JButton();
  JButton buttonDELETE1 = new JButton();
  JButton buttonDELETE = new JButton();
  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHELP = new JButton();

  List listAVAIL = new List();
  List listCampos = new List();

  Label labelAVAIL = new Label();
  Label labelOK = new Label();
  Label labelObjetivo = new Label();

  Choice choiceObjetivo = new Choice();


  // Constructors
  // ------------

  public FrameFields (String t, Connection c, Output o, Help h)
  {
    table      = t;    // Table
    connection = c;    // JDBC Connection
    out        = o;    // Output
    help       = h;    // Help system

    start();
  }

  public FrameFields (String t, Connection c, Frame f, Output o, Help h)
  {
    table      = t; // Table
    connection = c; // JDBC Connection
    frame      = f; // Caller
    out        = o; // Output
    help       = h; // Help system

    start();
  }

  public FrameFields (String t, Connection c, Frame f, TMiner m)
  {
    table      = t; // Table
    connection = c; // JDBC Connection
    frame      = f; // Caller
    miner      = m; // TMiner
    out        = m; // Output
    help       = m; // Help system

    start();
  }

  private void start()
  {
    // GUI init

    gui(); 

    fields();

    // Language

    if (Environment.settings==Environment.SPANISH)
       Spanish();
    else
       English();
  }


  private void gui()
  {
    Image icon = Loader.loadImage(NumericalCruncher.class,"image/dual.gif");

    this.setIconImage(icon);
    this.setBackground(Color.lightGray);
    this.setSize(new Dimension(427, 248));
    this.addWindowListener(new FrameFields_this_windowAdapter(this));

    borderLayout.setVgap(5);

    buttonOK.addActionListener(new FrameFields_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new FrameFields_buttonCancel_actionAdapter(this));
    buttonHELP.addActionListener(new FrameFields_buttonHELP_actionAdapter(this));

    listCampos.setBackground(Color.white);
    listCampos.setMultipleMode(true);
    listAVAIL.setBackground(Color.white);
    listAVAIL.setMultipleMode(true);

    buttonADD.setText(">>");
    buttonADD.addActionListener(new FrameFields_buttonADD_actionAdapter(this));

    buttonADD1.setText(">");
    buttonADD1.addActionListener(new FrameFields_buttonADD1_actionAdapter(this));

    buttonDELETE1.setText("<");
    buttonDELETE1.addActionListener(new FrameFields_buttonDELETE1_actionAdapter(this));

    buttonDELETE.setText("<<");
    buttonDELETE.addActionListener(new FrameFields_buttonDELETE_actionAdapter(this));

    gridLayoutLIST.setHgap(10);
    gridLayoutLIST.setColumns(2);
    gridLayoutLIST.setVgap(10);

    labelAVAIL.setFont(new Font("Dialog", 2, 12));
    labelAVAIL.setAlignment(1);
    labelOK.setFont(new Font("Dialog", 2, 12));
    labelOK.setAlignment(1);

    gridLayoutBUTTONS.setColumns(4);
    gridLayoutBUTTONS.setVgap(5);

    panelObjetivo.setVisible(true);
    panelObjetivo.setLayout(flowLayoutObjetivo);
    labelObjetivo.setFont(new Font("Dialog", 1, 12));

    choiceObjetivo.setBackground(Color.white);
    choiceObjetivo.addItemListener(new FrameFields_choiceObjetivo_itemAdapter(this));

    gridLayoutBUTTONS.setRows(2);
    gridLayoutBUTTONS.setHgap(5);

    panelOK.setLayout(borderLayoutOK);
    panelAVAIL.setLayout(borderLayoutAVAIL);
    panelBUTTONS.setLayout(gridLayoutBUTTONS);
    panelLIST.setLayout(gridLayoutLIST);

    this.setLayout(borderLayout);
    this.add(panelBUTTONS, BorderLayout.SOUTH);
    panelBUTTONS.add(buttonADD, null);
    panelBUTTONS.add(buttonADD1, null);
    panelBUTTONS.add(buttonDELETE1, null);
    panelBUTTONS.add(buttonDELETE, null);
    panelBUTTONS.add(panelDomains, null);
    panelBUTTONS.add(buttonOK, null);
    panelBUTTONS.add(buttonCancel, null);
    panelBUTTONS.add(buttonHELP, null);
    this.add(panelLIST, BorderLayout.CENTER);
    panelLIST.add(panelAVAIL, null);
    panelAVAIL.add(labelAVAIL, BorderLayout.NORTH);
    panelAVAIL.add(listAVAIL, BorderLayout.CENTER);
    panelLIST.add(panelOK, null);
    panelOK.add(labelOK, BorderLayout.NORTH);
    panelOK.add(listCampos, BorderLayout.CENTER);
    this.add(panelObjetivo, BorderLayout.NORTH);
    panelObjetivo.add(labelObjetivo, null);
    panelObjetivo.add(choiceObjetivo, null);
  }

  private void fields ()
  {
    String schema;
    String id;
    String column;
    int    j;
    int    pos = table.indexOf(".");
 
    if (pos!=-1) {
       schema = table.substring(0,pos);
       id     = table.substring(pos+1);
    } else {
       schema = null;
       id     = table;
    }

    try {

       // Table columns

       DatabaseMetaData metadata = connection.getMetaData();

       // NOTES: metadata.getCatalogs() ---> Not Supported (Oracle)
       //        metadata.getSchemas()  ---> Users (Oracle)
       //        metadata.getColumns()  ---> Oracle: Error when SCHEMA is used

       ResultSet columns = metadata.getColumns(null,null,id,"%");

       // MetaData

       String owner = Schema(columns);

       // Oracle Lite
       // -----------
       // Tipo NUMBER (data_type 2): PRECISION (size) + SCALE (0=Integer)
       // Tipo CHAR   (data_type 1): LENGTH

       // Oracle 8
       // -----------
       // Tipo NUMBER (data_type 2): COLUMN_SIZE + DECIMAL_DIGITS (0=Integer)
       // Tipo CHAR   (data_type 1): COLUMN_SIZE

       while (columns.next()) {
        
          if ( schema==null || owner==null || schema.equals(columns.getString(owner)) ) {

             column = columns.getString("Column_Name");
             
	     switch (columns.getShort("Data_Type")) {

               case java.sql.Types.CHAR:
               case java.sql.Types.VARCHAR:

                    // CHAR

                    for ( j=0;
                          j<choiceObjetivo.getItemCount() && column.compareTo(choiceObjetivo.getItem(j))>0;
                          j++ )
                        ;

                    choiceObjetivo.insert(column,j);
                    break;

               case java.sql.Types.NUMERIC:
               case java.sql.Types.DECIMAL:
               case java.sql.Types.REAL:
               case java.sql.Types.FLOAT:
               case java.sql.Types.DOUBLE:
               case java.sql.Types.BIT:
               case java.sql.Types.INTEGER:
               case java.sql.Types.TINYINT:
               case java.sql.Types.SMALLINT:
               case java.sql.Types.BIGINT:

                    // NUMBER

                    for ( j=0;
                          j<listAVAIL.getItemCount() && column.compareTo(listAVAIL.getItem(j))>0;
                          j++ )
                        ;

                    listAVAIL.add(column,j);

                    for ( j=0;
                          j<choiceObjetivo.getItemCount() && column.compareTo(choiceObjetivo.getItem(j))>0;
                          j++ )
                        ;

                    choiceObjetivo.insert(column,j);
                    break;

               case java.sql.Types.DATE:
               case java.sql.Types.TIME:
               case java.sql.Types.TIMESTAMP:
               case java.sql.Types.BINARY:
                    // Not used in NC
                    break;
             }
          }
       }

    } catch (Exception error) {

       if (schema!=null) {
          out.message(schema+"."+table+": "+error.getMessage()+"\n");
       } else {
          out.message(table+": "+error.getMessage()+"\n");
       }
  
    }

    choiceObjetivo.insert("",0);
    panelObjetivo.setVisible(true);
    objetivo = choiceObjetivo.getSelectedItem();
  }

  private void Spanish ()
  {
    this.setTitle(table+": Atributos");
    buttonOK.setText("OK");
    buttonCancel.setText("Cancelar");
    buttonHELP.setText("Ayuda");
    buttonHELP.setToolTipText("Sistema de ayuda en lnea");
    buttonADD.setToolTipText("Agregar todos los atributos");
    buttonADD1.setToolTipText("Agregar los atributos seleccionados");
    buttonDELETE1.setToolTipText("Eliminar los atributos seleccionados");
    buttonDELETE.setToolTipText("Eliminar todos los atributos");
    labelAVAIL.setText("Atributos disponibles");
    labelOK.setText("Atributos seleccionados");
    labelObjetivo.setText("Clasificacin");
  }

  private void English ()
  {
    this.setTitle(table+": Attributes");
    buttonOK.setText("OK");
    buttonCancel.setText("Cancel");
    buttonHELP.setText("Help");
    buttonHELP.setToolTipText("Online help");
    buttonADD.setToolTipText("Add all attributes");
    buttonADD1.setToolTipText("Add selected attributes");
    buttonDELETE1.setToolTipText("Remove selected attributes");
    buttonDELETE.setToolTipText("Remove all attributes");
    labelAVAIL.setText("Available attributes");
    labelOK.setText("Selected attributes");
    labelObjetivo.setText("Classification");
  }

  // DataBase MetaData
  // -----------------

  private String Schema (ResultSet columnas)
                 throws SQLException
  {
    ResultSetMetaData aux = columnas.getMetaData();

    int    total = aux.getColumnCount();
    int    i;
    String id;

    for (i=1; i<=total; i++) {
        id = aux.getColumnName(i);

        if (  id.equals("TABLE_OWNER")         // Oracle 
           || id.equals("TABLE_SCHEM") )       // JDBC
           return id;
    }

    return null;
  }

  // Free JDBC connection
  // --------------------

  void freeConnection ()
  {
    if (connection!=null)
       try {
           connection.close();
       } catch (Exception error) {
           out.message(error.getMessage()+"\n");
       }
  }

  // Events
  // ------

  // OK

  void buttonOK_actionPerformed(ActionEvent e)
  {
    DataConfiguration cfg;

    buttonOK.setEnabled(false);

    if (listCampos.getItemCount()>0) {

       if (Environment.settings==Environment.SPANISH) 
          out.message ("Seleccin de columnas realizada...\n");
       else
          out.message ("Columns selected...\n");

       setVisible(false);

       if (Environment.settings==Environment.SPANISH) 
          out.message ("Cargando datos...\n");
       else
          out.message ("Loading data...\n");

       cfg = new DataConfiguration(table,objetivo,listCampos.getItems(),connection);

       MainMenu  menu = new MainMenu(cfg,out,help,miner);
       Dimension d    = Toolkit.getDefaultToolkit().getScreenSize();

       menu.setLocation((d.width - menu.getSize().width) / 2, (d.height - menu.getSize().height) / 2);
       menu.setVisible(true);

       if (miner==null)
          freeConnection();

    } else {

       if (Environment.settings==Environment.SPANISH) 
          out.message ("No se seleccion ningn atributo.\n");
       else
          out.message ("No columns were selected.\n");
    }

    buttonOK.setEnabled(true);
  }

  // CANCEL

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    this.setVisible(false);

    if (frame!=null)
       frame.show();
    else 
       freeConnection();
  }

  void this_windowClosing(WindowEvent e)
  {
    this.setVisible(false);

    if (frame!=null)
       frame.show();
    else
       freeConnection();
  }

  // Online help

  void buttonHELP_actionPerformed(ActionEvent e)
  {
    help.help();
  }


  // All columns: listAVAIL -> listCampos

  void buttonADD_actionPerformed(ActionEvent e)
  {
    int    i;
    int    j=0;
    String id;

    for (i=0; i<listAVAIL.getItemCount(); i++) {

        id = listAVAIL.getItem(i);

        while (j<listCampos.getItemCount() && id.compareTo(listCampos.getItem(j))>0 )
              j++;

        listCampos.add ( id, j );
    }

    listAVAIL.removeAll();
  }

  // Selected columns: listAVAIL -> listCampos

  void buttonADD1_actionPerformed(ActionEvent e)
  {
    int      i;
    int      j;
    String   id;
    String[] columns = listAVAIL.getSelectedItems();

    for (i=0; i<columns.length; i++) {

        id = columns[i];

        for ( j=0;
              j<listCampos.getItemCount() && id.compareTo(listCampos.getItem(j))>0;
              j++ )
            ;

        listCampos.add(id,j);
        listAVAIL.remove (id);
    }
  }

  // Selected columns: listAVAIL <- listCampos

  void buttonDELETE1_actionPerformed(ActionEvent e)
  {
    int      i;
    int      j;
    String   id;
    String[] columns = listCampos.getSelectedItems();

    for (i=0; i<columns.length; i++) {

        id = columns[i];

        for ( j=0;
              j<listAVAIL.getItemCount() && id.compareTo(listAVAIL.getItem(j))>0;
              j++ )
            ;

        listAVAIL.add (id,j);
        listCampos.remove (id);
    }
  }

  // All columns: listAVAIL <- listCampos

  void buttonDELETE_actionPerformed(ActionEvent e)
  {
    int    i;
    int    j=0;
    String id;

    for (i=0; i<listCampos.getItemCount(); i++) {

        id = listCampos.getItem(i);

        while (j<listAVAIL.getItemCount() && id.compareTo(listAVAIL.getItem(j))>0 )
              j++;

        listAVAIL.add ( id, j );
    }

    listCampos.removeAll();
  }

  // Classification objective

  void choiceObjetivo_itemStateChanged (ItemEvent e)
  {
    int    j;
    String prev = objetivo;

    for ( j=0;
          j<listAVAIL.getItemCount() && prev.compareTo(listAVAIL.getItem(j))>0;
          j++ )
        ;

    if (numeric) 
       listAVAIL.add (prev,j);

    numeric  = false;
    objetivo = choiceObjetivo.getSelectedItem();

    try {
      listAVAIL.remove(objetivo);
      numeric = true;
    } catch (IllegalArgumentException avail) {}

    try {
      listCampos.remove(objetivo);
      numeric = true;
    } catch (IllegalArgumentException campos) {}

  }

}

// Event handlers
// --------------

class FrameFields_this_windowAdapter extends java.awt.event.WindowAdapter
{
  FrameFields adaptee;

  FrameFields_this_windowAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e)
  {
    adaptee.this_windowClosing(e);
  }
}

class FrameFields_choiceObjetivo_itemAdapter implements java.awt.event.ItemListener
{
  FrameFields adaptee;

  FrameFields_choiceObjetivo_itemAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void itemStateChanged(ItemEvent e)
  {
    adaptee.choiceObjetivo_itemStateChanged(e);
  }
}

class FrameFields_buttonADD_actionAdapter implements java.awt.event.ActionListener
{
  FrameFields adaptee;

  FrameFields_buttonADD_actionAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonADD_actionPerformed(e);
  }
}

class FrameFields_buttonADD1_actionAdapter implements java.awt.event.ActionListener
{
  FrameFields adaptee;

  FrameFields_buttonADD1_actionAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonADD1_actionPerformed(e);
  }
}

class FrameFields_buttonDELETE1_actionAdapter implements java.awt.event.ActionListener
{
  FrameFields adaptee;

  FrameFields_buttonDELETE1_actionAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.buttonDELETE1_actionPerformed(e);
  }
}

class FrameFields_buttonDELETE_actionAdapter implements java.awt.event.ActionListener
{
  FrameFields adaptee;

  FrameFields_buttonDELETE_actionAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonDELETE_actionPerformed(e);
  }
}

class FrameFields_buttonOK_actionAdapter implements java.awt.event.ActionListener
{
  FrameFields adaptee;

  FrameFields_buttonOK_actionAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class FrameFields_buttonCancel_actionAdapter implements java.awt.event.ActionListener
{
  FrameFields adaptee;

  FrameFields_buttonCancel_actionAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonCancel_actionPerformed(e);
  }
}


class FrameFields_buttonHELP_actionAdapter implements java.awt.event.ActionListener
{
  FrameFields adaptee;

  FrameFields_buttonHELP_actionAdapter(FrameFields adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonHELP_actionPerformed(e);
  }
}

