package rf.gui.jdbc;

// Title:       NC
// Version:     1.61
// Copyright:   June 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.sql.*;

import fbg.gui.*;
import fbg.util.*;
import fbg.util.db.Tabla;

import rf.gui.NumericalCruncher;


public class FrameTables extends Frame
{
  // JDBC Connection

  Connection connect   = null;

  // I/O

  Output     out       = null;
  Help       help      = null;

  // Components

  GridLayout layout = new GridLayout();
  GridLayout gridLayoutRight = new GridLayout();
  GridLayout gridLayoutButtons = new GridLayout();

  List listTablas = new List();

  Panel panelRIGHT = new Panel();
  Panel panelTOP = new Panel();
  Panel panelButtons = new Panel();

  JButton buttonOK = new JButton();
  JButton buttonCancel = new JButton();
  JButton buttonHelp = new JButton();

  // Constructor
  // -----------

  public FrameTables (Output output, Help help_system, Connection conexion)
  {
    try {
      gui();
    } catch (Exception e) {
      e.printStackTrace();
    }

    out     = output;
    help    = help_system;
    connect = conexion;

    // Get catalog data

    try {

        DatabaseMetaData metadata = conexion.getMetaData();
        ResultSet        tablas   = metadata.getTables("","","%",new String[]{"TABLE"});
	String           colSchem = Tabla.Schema(tablas);

	while (tablas.next()) {

	      int    i;
	      String id = tablas.getString("Table_Name");

	      if (colSchem!=null) {

  	         String schema = tablas.getString(colSchem);  // "Table_Owner" o "Table_schem"

	         if (schema!=null) 
		    id = schema+"."+id;
	      }
	   
	      for ( i=0;
		    i<listTablas.getItemCount() && id.compareTo(listTablas.getItem(i))>0;
		    i++ )
		  ;

	      listTablas.add(id,i);
        }

    } catch (SQLException e) {

        while (e != null) {
              out.message(e.getMessage()+"\n");
              e = e.getNextException();
        }
    }

    if (Environment.settings==Environment.SPANISH)
       Spanish();
    else
       English();
  }

  // GUI

  private void gui() throws Exception
  {
    Image icon = Loader.loadImage(NumericalCruncher.class,"image/list.gif");

    this.setIconImage(icon);
    this.setBackground(Color.lightGray);
    this.setSize(new Dimension(400, 200));
    this.addWindowListener(new FrameTables_this_windowAdapter(this));

    layout.setHgap(10);
    layout.setColumns(2);
    layout.setVgap(10);

    gridLayoutRight.setRows(2);

    gridLayoutButtons.setHgap(10);
    gridLayoutButtons.setRows(3);
    gridLayoutButtons.setColumns(1);
    gridLayoutButtons.setVgap(5);

    buttonOK.addActionListener(new FrameTables_buttonOK_actionAdapter(this));
    buttonCancel.addActionListener(new FrameTables_buttonCancel_actionAdapter(this));
    buttonHelp.addActionListener(new FrameTables_buttonHelp_actionAdapter(this));

    listTablas.setBackground(Color.white);

    panelTOP.setBackground(Color.lightGray);

    this.setLayout(layout);
    this.add(listTablas, null);
    this.add(panelRIGHT, null);

    panelRIGHT.setLayout(gridLayoutRight);
    panelRIGHT.add(panelTOP, null);
    panelRIGHT.add(panelButtons, null);

    panelButtons.setLayout(gridLayoutButtons);
    panelButtons.add(buttonOK, null);
    panelButtons.add(buttonCancel, null);
    panelButtons.add(buttonHelp, null);
  }

  // Spanish version

  private void Spanish ()
  {
    this.setTitle("Tablas disponibles");
    buttonOK.setText("OK");
    buttonOK.setToolTipText("Seleccionar tabla");
    buttonCancel.setText("Cancelar");
    buttonCancel.setToolTipText("Cerrar conexin JDBC");
    buttonHelp.setText("Ayuda");
    buttonHelp.setToolTipText("Ayuda en lnea");
  }

  private void English ()
  {
    this.setTitle("Available tables");
    buttonOK.setText("OK");
    buttonOK.setToolTipText("Select current table");
    buttonCancel.setText("Cancel");
    buttonCancel.setToolTipText("Close JDBC connection");
    buttonHelp.setText("Help");
    buttonHelp.setToolTipText("Online help");
  }

  // Free JDBC connection

  void liberarConexion ()
  {
    if (connect!=null)
       try {
           connect.close();
       } catch (Exception error) {
           out.message(error.getMessage()+"\n");
       }
  }

  // Events

  void this_windowClosing(WindowEvent e)
  {
    setVisible(false);

    liberarConexion();
  }

  void buttonOK_actionPerformed(ActionEvent e)
  {
    String table = listTablas.getSelectedItem();

    setVisible(false);

    if (table!=null) {

       if (Environment.settings==Environment.SPANISH)
          out.message("Tabla '"+table+"' seleccionada.\n");
       else
          out.message("Table '"+table+"' selected.\n");

       // Fields selection

       FrameFields fields = new FrameFields (table,connect,out,help);

       Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
       fields.setLocation((d.width - fields.getSize().width) / 2, (d.height - fields.getSize().height) / 2);

       connect = null;
       fields.setVisible(true);

    } else {

       liberarConexion();
    }
  }

  void buttonCancel_actionPerformed(ActionEvent e)
  {
    setVisible(false);

    liberarConexion();
  }

  void buttonHelp_actionPerformed(ActionEvent e)
  {
    help.help();
  }

}


// Manejo de eventos
// -----------------

class FrameTables_this_windowAdapter extends java.awt.event.WindowAdapter
{
  FrameTables adaptee;

  FrameTables_this_windowAdapter(FrameTables adaptee)
  {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e)
  {
    adaptee.this_windowClosing(e);
  }
}

class FrameTables_buttonOK_actionAdapter implements java.awt.event.ActionListener
{
  FrameTables adaptee;

  FrameTables_buttonOK_actionAdapter(FrameTables adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonOK_actionPerformed(e);
  }
}

class FrameTables_buttonCancel_actionAdapter implements java.awt.event.ActionListener
{
  FrameTables adaptee;

  FrameTables_buttonCancel_actionAdapter(FrameTables adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonCancel_actionPerformed(e);
  }
}

class FrameTables_buttonHelp_actionAdapter implements java.awt.event.ActionListener
{
  FrameTables adaptee;

  FrameTables_buttonHelp_actionAdapter(FrameTables adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.buttonHelp_actionPerformed(e);
  }
}

